# Library of string manipulation functions
# -*- coding: UTF-8 -*- 

# All input and output arguments are character strings

# =============================== Internal functions ================================

"""
    from Python_SurvivalKit import *    # Used in "computeFCA" variants
"""

def Ltrim(s=None):
    "returns s where leading spaces, tabs, new-line, etc. are removed"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return ss.lstrip()

def Rtrim(s=None):
    "returns s where trailing spaces, tabs, new-line, etc. are removed"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return ss.rstrip()

def Ftrim(s=None):
    "returns s where leading and trailing spaces, tabs, new-line, etc. are removed"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return Ltrim(Rtrim(ss))

def IsNumeric(s=None):
    "returns '1' if s is a numeric expression else returns '0'"
    if s == None:
        return u'0'
    try:
        num = float(s)
    except:
        return u'0'
    else:
        return u'1'
# =============================== Internal functions ================================


def Capitalize(s=None):
    "returns s where the first character is in upper case and the others in lower case"
    if s == None:
        return u''
    if s == u'':
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return ss[0].upper() + ss[1:].lower()

def UnFrench(s=None):
    "returns s where all accented characters have been developed in pure ASCII"
    if s == None:
        return u''
    if s == u'':
        return s
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    for f in frenchChar:
        pos = frenchChar.index(f)
        ss = ss.replace(f,unFrenchChar[pos])
    return ss

def French(s=None):
    "returns s where accented characters are restored from pure ASCII"
    if s == None:
        return u''
    if s == u'':
        return s
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    for f in unFrenchChar:
        pos = unFrenchChar.index(f)
        ss = ss.replace(f,frenchChar[pos])
    return ss

def ExtractLeft(s=None,n=None):
    "returns the first n characters of s"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u'' 
    if nn >= len(ss):
        return ss
    return ss[:nn]

def ExtractRight(s=None,n=None):
    "returns the last n characters of s"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u''
    if nn >= len(ss):
        return ss
    else:
        return ss[len(ss)-nn:]


def RemoveLeft(s=None,n=None):
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u''
    nn = max(0,min(len(ss),nn)) # Clipping in [0,len(ss)]
    return ss[nn:]


def RemoveRight(s=None,n=None):
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if n == None:
        return ss
    try:
        nn = int(float(n))
    except:
        return u''
    nn = max(0,min(len(ss),nn)) # Clipping in [0,len(ss)]
    return ss[:len(ss)-nn]


def InString(s=None,arg=None,pos=None):
    "returns the position of arg in string s, starting from position pos."
    "returns 0 if none"
    if s == None:
        return u'0'
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    if arg == None:
        return '0'
    iarg = arg
    if IsNumeric(arg) == u'1':
        iarg = unicode(arg)
    if pos == None:
        pos = u'1'
    try:
        ipos = int(pos)
    except:
        return u'0'
    if ipos < 1 or ipos > len(s):
        return u'0'
    else:
        res = ss[ipos-1:].find(iarg) + 1
        if res > 0:
            res += (ipos - 1)
        return unicode(res)

def NumToChar(i=None):
    "returns the character with code (i mod 256)"
    "returns Chr(0) if i < 0"
    if i == None:
        return chr(0)
    if not IsNumeric(i) == u'1':
        return chr(0)
    else:
        ii = int(i)
    if ii < 0:
        return chr(0)
    return chr(ii % 256)

def CharToNum(s=None):
    "returns the numeric code of first character of string s"
    "returns 0 if s is empty"
    if s == None:
        return u'0'
    elif s == u'':
        return u'0'
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    return unicode(ord(ss[0]))

def SQLunquote(s=None):
    "returns string s unquoted and with double quotes reduced"
    "s missing = empty string"
    if s == None:
        return u''
    if s == u'':
        return u'0'
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    ss = ss.replace(u"''",u"'")
    if ss[0] == u"'": ss = ss[1:]
    if ss[-1] == u"'": ss = ss[:-1]
    return ss

def SQLquote(s=None):
    "returns string s quoted in SQL style"
    "s missing = empty string"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    ss = ss.replace(u"'",u"''")
    ss = u"'"+ss+u"'"
    return ss

def SQLquote2(s=None):
    "same as SQLquote but without surrounding quotes"
    "s missing = empty string"
    if s == None:
        return u''
    ss = s
    if IsNumeric(s) == u'1':
        ss = unicode(s)
    ss = ss.replace(u"'",u"''")
    return ss


def KeyValue(s=None):
    if s == None:
        return u'',u''
    pos = s.find(u'=')
    if pos == -1:
        return s
    return Ftrim(s[:pos]),Ltrim(s[pos+1:])


def ExtractTaggedString(txt,head,tail):
    # returns the fragment of "txt" between "head" and first "tail" (excluded)
    # If "head" is empty, returns the fragment preceding "tail"
    # If "tail" is empty, returns the fragment following "head"
    # If both are empty, returns "txt"
    # If "head" and/or "tail" haven't been found, returns u''
    # If one of the arguments is not a unicode string, returns u''
   
    '''for I in range(0,150):
        f = I*200
        l = f + 200
        print txt[f:l]'''
        
    if IsNumeric(head):
        head = unicode(head)
    if IsNumeric(tail):
        tail = unicode(tail)

    hoffset = len(head)
    toffset = len(tail)

    try:
        pos1 = txt.find(head)
    except Exception, e:
        print u'In "ExtractTaggedString", one of the arguments is not a valid character string'
        return u''
    
    if pos1 == -1:
        extract = u''
    else:
        if toffset == 0:
            extract = txt[pos1+hoffset:]
        else:
            try:
                pos2 = txt.find(tail,pos1+hoffset)
            except  Exception, e:
                print u'In "ExtractTaggedString", one of the arguments is not a valid character string'
                return u''                
            if pos2 == -1:
                extract = u''
            else:
                extract = txt[pos1+hoffset:pos2]

    return extract


def NextSection(txt,pos1,search,head,tail,tags=1):
    # Extract from "txt", starting at position "pos1",
    # the fragment defined by tags "search"-"tail" (included)
    # Internal fragments "head"-"tail" are kepts but skipped.
    # "head" may not be empty.
    # "tags" tells whether enclosing tags are returned. 0 = No, 1 = yes.
    #
    # Examples:
    # nextSection(u'11<a x>22<a>33</a>44</a>55',0,u'<a x>',u'<a>',u'</a>')
    #     => ('<a x>22<a>33</a>44</a>',24)
    # nextSection(u'11<a>22<a>33</a>44</a>55',0,u'<a>',u'<a>',u'</a>')
    #     => ('<a>22<a>33</a>44</a>',22)

    def nextTag(pos):
        # Search "page" from "pos" for the next "head" or "tail" tag
        # case: 0 = no tag found
        #       1 = the closest tag is a "head"
        #       2 = the closest tag is a "tail"
        # t: position of the closest tag or -1

        t1 = txt.find(head,pos)
        t2 = txt.find(tail,pos)

        if t1 == -1:
            # No head 
            if t2 == -1:
                # No head, no tail
                case = 0
                t = -1
            else:
                # No head, a tail
                case = 2
                t = t2
        else:
            # A head tag has been found
            if t2 == -1:
                # A head, no tail
                case = 1
                t = t1
            else:
                # A head, a tail
                if t1 < t2:
                    # The closest tag is a head
                    case = 1
                    t = t1
                else:
                    # The closest tag is a tail
                    case = 2
                    t = t2
        return case,t

    def nextTail(pos):
        # Find the closest "tail" tag, while skipping in-between "head"-"tail" subsections
        stack = 0   # Embedding level
        while True:
            case,npos = nextTag(pos)
            if stack == 0:
                # No subsection in process
                if case == 2:
                    ##print '1. stack,case,pos',stack,case,npos
                    # Next tail found; exit
                    pos = npos
                    break
                elif case == 1:
                    ##print '2. stack,case,pos',stack,case,npos
                    # new "head" found; skip section; loop
                    stack += 1
                    pos = npos + hoffset
                    continue
                else:
                    ##print '3. stack,case,pos',stack,case,npos
                    # No tail found; exit
                    pos = -1
                    break
            else:
                # We are examinig a subsection
                if case == 2:
                    ##print '4. stack,case,pos',stack,case,npos
                    # Next tail found; internal subsection closed; loop
                    pos = npos + toffset
                    stack -= 1
                    continue
                elif case == 1:
                    ##print '5. stack,case,pos',stack,case,npos
                    # new "head" found; skip subsection; loop
                    stack += 1
                    pos = npos + hoffset
                    continue
                else:
                    ##print '6. stack,case,pos',stack,case,npos
                    # No "tail" found; exit
                    pos = -1
                    break
        return pos

    if IsNumeric(search):
        search = unicode(search)
    if IsNumeric(head):
        head = unicode(head)
    if IsNumeric(tail):
        tail = unicode(tail)

    if IsNumeric(pos1):
        pos1 = int(float(pos1))
    else:
        print u'In "nextSection", argument "pos1" is not numeric'
        return u'',-1
    if IsNumeric(tags):
        pos1 = int(float(pos1))
    else:
        print u'In "nextSection", argument "tags" is not numeric'
        return u'',-1
        
    if head == u'':
        return u'',-1
    
    soffset = len(search)
    hoffset = len(head)
    toffset = len(tail)

    if pos1 < 0:
        pos1 = 0
    if pos1 >= len(txt):
        return u'',-1
        
    try:
        pos1 = txt.find(search,pos1)
    except Exception, e:
        print u'In "nextSection", one of the arguments is not a valid character string'
        return u'',-1
    if pos1 == -1:
        # "search" tag not found
        section = u''
        pos = -1
    else:
        # "search" tag found
        pos2 = nextTail(pos1+soffset)
        if pos2 > -1:
            # Closing "tail" tag found
            if tags == 1:
                # Add enclosing tags
                section = txt[pos1:pos2+toffset]
            else:
                # Drop enclosing tags
                section = txt[pos1+soffset:pos2]
            pos = pos2 + toffset
        else:
            # "head" found but no "tail": invalid!
            section = u''
            pos = -1

    return section,pos


# ================ PrettyPrint function ==================== #

# Simple "pretty printer" for "select" SQL queries.
# At least as good as those found on the web!
# May not produce satisfying results for some complex queries
# Returns the input string in case of problem
# Input string must be in unicode

def _ExtractConstants(query):
    # Extract string constants from "query" and create constant dictionary
    global constantDic  # dictionary of string constants
    
    quote = u'\''
    lenQ = len(query)   # length of input query
    reducedQ = []       # character list of the reduced query
    level = 0           # level of current char. 0 = outside a constant; 1 = within a constant
    iChr = 0            # index of current character "C"
    intQ = 0            # integer part of the key of the next string constant
    constant = []       # current string constant
    
    while True:
        if iChr == lenQ:
            break

        C = query[iChr]
        if C <> quote:
            # 'C' is a pure character.
            # Add it to the query or the constant according to the level
            if level == 0:
                reducedQ.append(C)
            else:
                constant.append(C)
            iChr += 1
        else:
            # 'C' is a quote.
            # 3 cases: start of constant, first of double quote, end of constant
            if level == 0:
                # Start of a constant
                constant.append(C)
                iChr += 1
                level = 1
            else:
                # Within a constant, 1 or 2 consecutive quotes?
                constant.append(C)
                iChr += 1
                if query[iChr] == quote:
                    # The next one is also a quote
                    constant.append(query[iChr])
                    iChr += 1
                else:
                    # This quote is alone: end of constant
                    quotedId = u'#'+unicode(intQ)+u'#'
                    constantDic[quotedId] =u''.join(constant)
                    level = 0
                    constant = []
                    reducedQ.append(quotedId)
                    intQ += 1

    return  u''.join(reducedQ)


def _TokenizeQuery(reducedQuery):
    # Tokenize query "reducedQuery" into list "tokens"
    # Splitting character = single space

    prepQuery = reducedQuery
    
    # Add spaces around symbols to prepare "split"
    for sym in [u',',u'(',u')',u'>=','<=',u'=','<>','>','<']:
        prepQuery = prepQuery.replace(sym,u' '+sym+u' ')

    # Reduce long space strings to single space (quick & dirty!)
    prepQuery = prepQuery.replace(u'    ',u' ').replace(u'    ',u' ')\
                .replace(u'   ',u' ')\
                .replace(u'  ',u' ').replace(u'  ',u' ')

    # Oups! Restore combined operators
    prepQuery = prepQuery\
                .replace(u'> =',u'>=')\
                .replace(u'< =',u'<=')\
                .replace(u'= >',u'>=')\
                .replace(u'= <',u'<=')\
                .replace(u'< >',u'<>')\
                .replace(u'> <',u'><')

    # Decompose the query into a list of tokens
    tokens = prepQuery.split(u' ')
    return tokens


def _InsertIndentTags(tokens):
    # Create a query in which indent tags have been inserted.
    # The SQL words depending on a "select" word are assigned the same tagID
    # Each "select" word receives a new tag, that is assigned to all its 
    # dependent words: "from", "where", "group", "order", "having", "union", etc.
    # "select" tag = '\@04@\', the other tags = '@@\\\@04@\'
    # Also performs some nice space reduction and alignment
    global indentDic
    
    level = 1                       # the current level: '(' = +1; ')' = -1
    levels = [1]                    # stack of current tag ID of each level: 
    tagID = 1                       # current tag ID. Assigned monotonically; '(' = +1
    indentDic = {u'\\@01@\\':None}  # dictionary of tag ID. Will map to real indent size
    lenTokens = len(tokens)
    for iT in range(lenTokens):
        token = tokens[iT]
        # print '-- token:',token,' - level:',level,' - lastID:',lastID,' - levels:',levels,' - indent:',indent
        levStr = '%02d' % levels[level-1]
        if token == u'(':
            level += 1
            tagID += 1
            levels.append(tagID)
            indentDic[u'\\@'+('%02d' % tagID)+u'@\\'] = None
        elif token == u')':
            del levels[level-1]
            level -= 1
        elif token.lower() == u'select':
            tokens[iT] = (u'\\@'+levStr+u'@\\')+token+u' '
        elif token.lower() in [u'from',u'where',u'and',u'or',u'group',u'order',u'having',
                               u'union',u'except',u'inter']:
            tokens[iT] = (u'@@\\\\\\@'+levStr+u'@\\')+token+u' '

    # Insert spaces around key words
    tokens2 = [u' '+el+u' ' if (el.lower() in [u'as','in',u'not',u'like',u'between',u'by',
                                               u'distinct',u'asc',u'desc',u'on',u'join',u'left'])
               else el
               for el in tokens]

    # Build query from tokens
    Q5 = u' '.join(tokens2)

    # Insert spaces around operators
    taggedQuery = Q5
    for sym in [u'>=','<=',u'=','<>','>','<']:
        taggedQuery = taggedQuery.replace(sym,u' '+sym+u' ')

    # (once more) restore combined operators
    taggedQuery = taggedQuery.replace(u' >  = ',u'>=').replace(u' <  = ',u'<=').replace(u' <  > ',u'<>')

    # Reduce space strings
    taggedQuery = taggedQuery.replace(u'   ',u' ').replace(u'  ',u' ').replace(u'  ',u' ')
    taggedQuery = taggedQuery.replace(u') )',u'))').replace(u'( (',u'((').rstrip()

    # Align 'select', 'from', 'where', 'and', 'or' key words
    taggedQuery = taggedQuery\
                  .replace(u'@\\from ',u'@\\from   ')\
                  .replace(u'@\\FROM ',u'@\\FROM   ')\
                  .replace(u'@\\where ',u'@\\where  ')\
                  .replace(u'@\\WHERE ',u'@\\WHERE  ')\
                  .replace(u'@\\and ',u'@\\and    ')\
                  .replace(u'@\\AND ',u'@\\AND    ')\
                  .replace(u'@\\or ',u'@\\or     ')\
                  .replace(u'@\\OR ',u'@\\OR     ')

    # Remove space after '(', before ')' and before ','
    taggedQuery = taggedQuery.replace(u'( ',u'(').replace(u' )',u')').replace(u' ,',u',')

    # Remove space between a function name and  '('
    taggedQuery = taggedQuery\
                  .replace(u'sum (',u'sum(')\
                  .replace(u'avg (',u'avg(')\
                  .replace(u'count (',u'count(')\
                  .replace(u'min (',u'min(')\
                  .replace(u'max (',u'max(')\
                  .replace(u'group_concat (',u'group_concat(')

    return taggedQuery

def _InjectConstant(query):
    global constantDic

    injectedQ = query
    for const in constantDic.keys():
        injectedQ = injectedQ.replace(const,constantDic[const])
    return injectedQ

def _CreateMultiLine(taggedQuery):
    # Convert query "" into a list of lines
    # Break the query on main key words (except 'select', which are protected)
    
    multiLineQuery = taggedQuery.split(u'@@\\\\')
    
    return multiLineQuery

def _CompleteIndentDictionary(multiLineQuery):
    # Assign a number of spaces to each tagID of "indentDic" dictionary
    global indentDic

    for line in multiLineQuery:
        ##print '\n   -- line:',line
        pos = 0
        inst = 0        # number of tags found in this line
        initIndent = 0  # indent of the current line
        while True:
            ##print '   -- inst:',inst
            pos = line.find(u'@\\',pos+1)
            ##print '   -- pos:',pos
            if pos == -1:
                ##print '   -- no more tag:'
                break
            ##print '   -- tag found at:',pos
            num = line[pos-2:pos]
            levKey = u'\\@'+num+u'@\\'
            ##print '   -- num:',num,'- levKey:',levKey
            if line[pos+2:].lower().startswith(u'select'):
                # "select" tag found
                ##print '   -- select tag found at:',pos
                if indentDic[levKey] is None:
                    indentDic[levKey] = pos-4 - inst*6 + initIndent
                    ##print '   -- indentDic[levKey]:',indentDic[levKey]
            else:
                # non-"select" tag found.
                # If first one in the line, record its offset
                if initIndent == 0:
                    initIndent = indentDic[levKey]
                
            inst += 1

    return

def _PrettyQuery(multiLineQuery,linebreak):
    # Assembles the lines of "multiLineQuery" into the final query
    import sysLib
    
    indentedQ = []      # list of lines of the "pretty" query
    for line in multiLineQuery:
        fline = line
        for key in indentDic.keys():
            indentKey = indentDic[key]
            if indentKey is None:
                # Skip unmapped tagID
                continue

            pos = fline.find(key)
            if fline[pos-1] == u'(' and fline[pos+6:].lower().startswith(u'select'):
                # No insertion if "select" after a parenthesis
                fline = fline.replace(key,u'')
            else:
                # Others: insert the space string
                fline = fline.replace(key,indentKey*u' ')

        nspos = sysLib.skipSpaces(fline,0)
        if fline[nspos:nspos+6].lower() in (u'union ',u'inter ',u'except'):
            # Special cases: lines with set operator + 'select' are split into two
            spos = fline.find(u' ',nspos)
            indentedQ.append(u'  '+fline[:spos])
            indentedQ.append(fline[spos+1:])
        else:
            indentedQ.append(fline)

    return linebreak.join(indentedQ)


def PrettyPrint(rawQuery,linebreak = u'eol'):
    global constantDic  # dictionary of string constants

    if not isinstance(rawQuery,unicode):
        return rawQuery
    
    if rawQuery.strip() == u'':
        return rawQuery

    if not rawQuery.lstrip().lower().startswith(u'select'):
        # Unrecognized syntax
        return u'[Unrecognized syntax in "SQLprettyPrint"] '+rawQuery

    # Remove source linebreaks and tabs
    rawQuery = rawQuery.replace(u'\n',u' ').replace(u'\t',u' ').strip()
    
    # Normalize linebreak. 'eol' --> '\n'. Otherwise, can be anything else.
    if linebreak == u'eol':
        linebreak = u'\n'
    
    rawQuery += u' '    # sentinel. Will disappear

    try:
        reducedQuery   = _ExtractConstants(rawQuery)
        tokens         = _TokenizeQuery(reducedQuery)
        taggedQuery    = _InsertIndentTags(tokens)
        injectedQuery  = _InjectConstant(taggedQuery)
        multiLineQuery = _CreateMultiLine(injectedQuery)
        _CompleteIndentDictionary(multiLineQuery)
        prettyQuery    = _PrettyQuery(multiLineQuery,linebreak)
    except:
        # Unrecognized syntax
        return u'[Unrecognized syntax in "SQLprettyPrint"] '+rawQuery

    return prettyQuery


# ================ Cryptographic functions ==================== #

# Hashing
# -------

def hash(S,form):
    # Returns the hash value of string S as a string of
    #   - 64 hexa characters    if "form" = 1
    #   - 44 base64 characters  if  "form" = 2
    # S can be numeric. Hashes '' if S is empty
    # Uses the sha256 algorithm

    from SQLfastUDFlib import SQLisNumeric
    import hashlib
    import base64

    # Check "form"
    if form not in ('1','2'):
        form = 1
    else:
        form = int(form)

    # If needed, convert "S" into char string
    try:
        if SQLisNumeric(S):
            S = unicode(S)
    except Exception,e:
        print u'Error in LStr:hash.S.',e
        return ''

    # S must be a byte string
    if type(S) == unicode:
        S = S.encode('latin-1')

    res = hashlib.sha256(S)
    if form == 1:
        res = res.hexdigest()
    else:
        try:
            res = base64.b64encode(res.digest(),'_-')
        except Exception,e:
            print u'Error in LStr:hash.res.',e
            return ''

    return res

def miningHash(S,prefix,hashNbr,form):
    # PoW mining version of hash.
    # Returns "res", the first hash value (sha256) of string "S" the binary value
    #  of which starts with the '0'-bitstring specified by "prefix"
    # ('00000' = starts with at least 5 zero bits).
    # At most "hashNbr" are computed. 
    # Also returns "nonce", the nonce that produces this hash
    #              "N", the number of hashes that have been calculated.
    # 
    # "res" is a string of
    #   - 64 hexa characters    if "form" = 'hexa'
    #   - 44 base64 characters  if "form" = 'base64'

    from SQLfastUDFlib import SQLisNumeric
    import hashlib
    import base64
    
    # Check "S"
    if S is None or S == '':
        return '','',0

    try:
        if SQLisNumeric(S):
            S = unicode(S)
    except Exception,e:
        print u'Error in LStr:miningHash.S.',e
        return '','',0

    # S must be a byte string
    if type(S) == unicode:
        S = S.encode('latin-1')

    # Check "prefix"
    if prefix is None or prefix == '':
        return '','',0

    try:
        if SQLisNumeric(prefix):
            prefix = unicode(prefix)
    except Exception,e:
        print u'Error in LStr:miningHash.prefix.',e
        return '','',0

    if len(prefix) == 0:
        print u'Error in LStr:miningHash.prefix.','Cannot be empty.'
        return '','',0
    if len(prefix) > 64:
        print u'Warning in LStr:miningHash.prefix.','Prefix too long; shortened to 64 char.'
        prefix = prefix[:64]

    # Check "hashNbr"
    if hashNbr is None or hashNbr == '':
        return '','',0

    if not SQLisNumeric(hashNbr):
        print u'Error in LStr:miningHash.hashNbr.','"hashNbr" must be a numeric value.'
        return '','',0
    else:
        hashNbr = int(float(hashNbr))

    if hashNbr < 1:
        print u'Error in LStr:miningHash.hashNbr.','"hashNbr" must be a positive number.'
        return '','',0

    if hashNbr > 100*1000*1000:
        print u'Warning in LStr:miningHash.hashNbr.','"hashNbr" too large. Reduced to 100,000,000.'
        hashNbr = 100*1000*1000

    # Check "form"
    if form not in ('hexa','base64'):
        form = 'hexa'

    # Search for a legal hash
    lenPrefix = len(prefix)
    nonce = ''
    for N in range(hashNbr):
        extS = S.encode('latin-1') + str(N)
        #print '   extS: ',extS
        Bhash = hashlib.sha256(extS)
        #print '   Bhash:',Bhash.digest()
        Chash = ''.join('{0:08b}'.format(ord(x), 'b') for x in Bhash.digest()[:lenPrefix//8+1])[:lenPrefix]
        #print '   Chash:',Chash
        if Chash == prefix:
            nonce = str(N)
            #print '   nonce:',nonce
            break

    if nonce == '':
        # No legal hash has been found. Augment the number of trials
        return '','',N
    
    if form == 'hexa':
        res = Bhash.hexdigest()
        #print '      - res.hexa():',res
    else:
        res = base64.b64encode(Bhash.digest(),'_-')
        #print '      - res.b64():',res

    return res,nonce,N


# Symmetric encryption
# --------------------

def generateSYMkey(n):
    # Generate a random key of "n" bytes converted in base64
    from SQLfastUDFlib import SQLrandomblob

    try:
        vn = float(n)
        vn = SQLint(vn)
    except Exception,e:
        vn = 1

    symKey = SQLrandomblob(n,2)

    return symKey


def encryptSYM(message,symKey):
    # Encrypts byte string "message" with key "symkey" and converts it in base64.
    from SQLfastUDFlib import SQLencrypt
    crypto = SQLencrypt(message,symKey)
    return crypto


def decryptSYM(crypto,symKey):
    # decrypts base64 character string "crypto" with key "symkey" and returns it as a byte string
    from SQLfastUDFlib import SQLdecrypt
    message = SQLdecrypt(crypto,symKey)
    return message


# RAS encryption
# --------------

def generateRSAkeys(n):
    # Generate secret and public keys, converted in base64
    # The public key is "n" bit long (default = 512)
    import rsa
    from SQLfastUDFlib import SQLint
    #print 'generateRSAkeys:n:',n
    try:
        vn = float(n)
        vn = SQLint(vn)
    except Exception,e:
        print 'LStr:generateRSAkeys.vn error:',e
        vn = 512

    # Generate secret and public keys (= usually (public,private))
    (secret,public) = rsa.newkeys(vn,accurate=False)

    # Serialize these keys
    secret_serial = secret.save_pkcs1(format='PEM')
    public_serial = public.save_pkcs1(format='PEM')

    # Condense keys
    secret_serial = secret_serial.replace(eol,'').replace(headerSecret,'').replace(footerSecret,'')
    public_serial = public_serial.replace(eol,'').replace(headerPublic,'').replace(footerPublic,'')

    return secret_serial, public_serial


def encryptRSA(mess,key):
    # encrypt message "mess" with secret key "key"
    # and convert it in base64
    import rsa
    from SQLfastUDFlib import SQLb64encode
   
    #print '\nencryptRSA:mess =',mess
    
    # Useless: # Convert '@n' into eol
    ##print '\nencryptRSA:key 1 =',key
    # Useless: key = key.replace('@n',eol)
    ##print '\nencryptRSA:key 2 =',key

    # Restore full PEM format
    #print '\nencryptRSA:key1 =',key
    key = headerSecret + eol + key + eol + footerSecret
    #print '\nencryptRSA:key2 =',key
    
    # Convert the PEM format into a secret key object
    try:
        secretKey = rsa.PublicKey.load_pkcs1(key,format='PEM')
    except Exception,e:
        print '\nencryptRAS:load_pkcs1 error:',e
    #print '\nencryptRSA:secretKey =',secretKey
    
    # Encrypt the message with the secret key
    try:
        crypto = rsa.encrypt(mess.encode('utf8'),secretKey)
    except Exception,e:
        print '\nencryptRAS:encrypt error:',e
    #print '\nencryptRSA:crypto =',crypto.encode('hex')
    
    # Convert the encrypted message in base64
    crypto = SQLb64encode(crypto,'url')
    #print '\nencryptRSA:crypto =',crypto
    
    return crypto

def decryptRSA(crypto,key):
    # decrypt encrypted message "crypto" with public key "key"
    import rsa
    from SQLfastUDFlib import SQLb64decode
    
    # Convert '@n' into eol
    # Useless: key = key.replace('@n',eol)
    
    # Restore full PEM format
    key = headerPublic + eol + key + eol + footerPublic

    # Convert the PEM format into a public key object
    publicKey = rsa.PrivateKey.load_pkcs1(key,format='PEM')

    # Restore the message in byte string from its base64 format
    crypto = SQLb64decode(crypto).encode('latin-1')
    
    # Decrypt the message with the public key
    try:
        mess = rsa.decrypt(crypto,publicKey)
    except Exception,e:
        print '\nLStr:decryptRSA error',e

    return mess



# ================================================================================ #
# ========================= Formal Concept Analysis ============================== #
# ================================================================================ #

"""
def loadContext(contextFile,sep):
    # Create in lists "G" and "I" a formal context from 'csv' input file "contextFile".
    # Each line of the input file comprises two values:
    #  - object name
    #  - list of attribute names (value separator = "sep")
    #
    # Example of content:
    #     "DIR","Emp#,Name,Level,..."
    #       ...
    #     "SEC","Emp#,Name,...,Lang"
    #
    # Properties of this file:
    #    No heading line
    #    blank/empty lines treated as end-of-file (next lines ignored)
    #    The value separator is specified in system variable "csv-separator".
    #    Both ' and " value delimiters are allowed.
    #
    # Output:
    #    "G", "I" 
    #    "G" empty if error
    # 
    # "G" is a sorted list of distinct object names.
    # "I" is a list of couples (g,mm),
    #     where "g" is an object name
    #           "mm" the list (as a string) of its attribute names (comma delimited)

    diag = 'OK'
    
    G = []
    I = []

    try:
        f = open(contextFile,'r')
    except:
        diag = u'Unable to open source data file'
        return diag,G,I

    mid = ''    # The central substring = delimiter+separator+delimiter. Ex: "," or ';'
    
    for line in f:
        line = line[:-1]
        if line.strip() == '':
            break

        # Analyze the first line to find the value delimiter
        if mid == '':
            delim = '"' if line.startswith('"') else "'"
            mid = delim+sep+delim

        rel = line.split(mid)

        if len(rel) <> 2:
            diag = u'Error in source data (must have exactly 2 fields per row)'
            break

        # Remove value delimiters
        g,mm = rel[0].lstrip(delim), rel[1].rstrip(delim)

        # Sort the attribute name list
        mm = ','.join(sorted(mm.split(',')))

        # Add the couple to I
        I.append((g,mm))

    f.close()

    # Extract G from I
    if diag == u'OK':
        G = sorted([rel[0] for rel in I])

    return diag,G,I

def printList(Lab,D):
    # Print label "Lab" then print the elements of dictionary "D{key:(val,discard)}".
    # Both "key" and "val" are unordered list coded as character strings.
    # Elements are displayed as ("ordKey")*("ordVal"),
    #     where "ordKey" is "key" sorted
    #     and   "ordVal" is "val" sorted.
    print Lab
    if len(D) == 0:
        print '   <empty>\n'
        return

    printList = []
    for key in sorted(D):
        val,discard = D[key]
        try:
            line = "    (%s)*(%s)" % (','.join((sorted(key.split(',')))),','.join((sorted(val.split(',')))))
        except:
            line = "    (%s)*(%s)" % (','.join((sorted(key.split(',')))),','.join(sorted(list(val))) )
        printList.append((line,discard,len(line)))
    maxLine = max([l[2] for l in printList])

    for line,discard,length in printList:
        padding = (maxLine - length +3)*' '
        print line,padding,discard
    return

def generateFCAfiles(Concepts,Inclusion,conceptFile,latticeFile,sep):
    # Generate "csv" output files "conceptFile" and "latticeFile" from the lists
    # from lists "Concepts" and "Inclusion".
    # Input:
    # - "Concepts": dictionary {concept:attributes}
    # - "Inclusion": list of couples (subconcept,superconconcept)
    # - "conceptFile" and "latticeFile": names of output files
    # - "sep": "csv" value separator in output files
    # Output:
    # - a "csv" file named "conceptFile" containing the concepts
    # - a "csv" file named "latticeFile" containing the inclusion relation
    # No generation if empty name.
    import os

    form = '"%s"'+sep+'"%s"\n'
    try:
        fout = open(conceptFile,'w')
        for c in Concepts.items():
            fout.write(form % c)
        fout.close()

        if latticeFile.strip() != '':
            # if os.path.exists(latticeFile):
            #     os.remove(latticeFile)
            fout = open(latticeFile,'w')
            for c in Inclusion:
                fout.write(form % tuple(c))
            fout.close()
        diag = u'OK'
    except:
        diag = u'Unable to save output data'
    return diag

def buildSize1Rectangles(I):
    # Create a dictionary with the initial rectangles from each object
    return {g:[m,0] for (g,m) in I}

def getRect(D,i):
    # Get rectangle (g,m) from "i"th item of dictionary D[i]
    g,(m,disc) = sorted(D.items())[i]
    return g,m,disc

def getRectFromList(L,i):   ###
    # Get rectangle (g,m) from "i"th item of list L[i]
    g,m = L[i]
    return g,m

def getgOfm(D,m):
    # return the (first) key (i.e., the object name) of value "m" in dictionary "D"
    g = None
    for key,[val,discard] in D.items():
        if val == m:
            g = key
            break
    return g

def getgOfmM(D,m):  ###
    try:
        g = D[m]
    except:
        g = None
    return g
    
def union(L1,L2):
    # Union of list (as strings) L1 and L2
    LL1 = L1.split(',')
    LL2 = L2.split(',')
    L = set(LL1).union(LL2)
    return ','.join(sorted(L))

def inter(L1,L2):
    # Intersection of list (as strings) "L1" and "L2"
    LL1 = L1.split(',')
    LL2 = L2.split(',')
    L = set(LL1).intersection(LL2)
    return ','.join(sorted(L))

def included(L1,L2):
    # Tells whether list "S1" (as string) is included into list "S2"
    LL1 = L1.split(',')
    LL2 = L2.split(',')
    return set(LL1).issubset(LL2)

def discard(D,g):
    # Mark rectangle "D[g]" in dictionary "D" as discarded
    [val,discard] = D[g]
    D[g] = [val,1]
    return

def addUndiscarded(R,D):
    # Copy in dictionary "R" the rectangles of "D" that are not discarded
    for key,[val,discard] in D.items():
        if discard == 0:
            R[key] = [val,discard]
    return

def extractConcepts(R):
    # Return the rectangles of dictionary "R" as a dictionary of concepts
    # Just ignore the "discard" component of "R"
    return {rect:R[rect][0] for rect in R.keys()}


def computeFCA(contextFile,conceptFile,latticeFile):
    from sysLib import getVarVal
    
    sep = getVarVal('csv-separator')

    startTimer()
    diag,G,I = loadContext(contextFile,sep) 
    print 'Context loaded:',getTimer()
    if diag != 'OK':
        return diag
    
    R = {}
    R2 = buildSize1Rectangles(I)
    
    step = 0
    while len(R2) > 1:
        step += 1
        R1 = R2
        R2 = {}
        R2m = {}    # R2m[m:g] is the inverse dictionary of R2[G:(m,d)]
        
        startTimer()
        for j in range(1,len(R1)):
            gj,mj,disc = getRect(R1,j)
            for i in range(0,j):
                gi,mi,disc = getRect(R1,i)
                mij = inter(mi,mj)
                if mij == '':
                    continue
                gij = union(gi,gj)
                gk = getgOfmM(R2m,mij)
                if gk is not None:
                    del R2[gk]
                    del R2m[mij]
                    R2[union(gk,gj)] = [mij,0]
                    R2m[mij] = union(gk,gj)
                else:
                    R2[gij] = [mij,0]
                    R2m[mij] = gij

                if mi == mij:
                    discard(R1,gi)
                if mj == mij:
                    discard(R1,gj)

        addUndiscarded(R,R1)
        print ' - Level '+str(step)+':',getTimer()

    if len(R2) > 0:
        addUndiscarded(R,R2)

    Concepts = extractConcepts(R)

    rawInclusion = []
    reducedInclusion = []
    if latticeFile != u'':
        rawInclusion = []
        for con1 in Concepts:
            for con2 in [con2 for con2 in Concepts if con2 <> con1]:
                if included(con1,con2):
                    rawInclusion.append([con1,con2])
        reducedInclusion = []
        for [con1,con2] in rawInclusion:
            status = 'basic'
            for con in [con for con in Concepts if con <> con1 and con <> con2]:
                if [con1,con] in rawInclusion and [con,con2] in rawInclusion:
                    status = 'transitive'
                    break
            if status == 'basic':
                reducedInclusion.append([con1,con2])

    diag = generateFCAfiles(Concepts,reducedInclusion,conceptFile,latticeFile,sep)

    return diag


def computeFCA_Comments(contextFile,conceptFile,latticeFile):
    # Computes the concepts lattice of a formal context
    #
    # Input
    #   contextFile: path of the 'csv' file of the source context
    #   conceptFile: path of the 'csv' file that will contain the concept descriptions
    #   latticeFile: path of the 'csv' file that will contain the concept hierarchy
    #
    # Output
    #   diag: status of the execution
    #       'OK'
    #       'Unable to open source data file'
    #       'Error in source data (lines must have exactly 2 fields)'
    #       'Unable to save output data'
    #
    # Expected result
    #   to create a 'csv' file that contains the composition of the concepts of the source context
    #   if "latticeFile" is not empty, to create a 'csv' file that contains the concept hierarchy
    #
    # Internal objects
    #   G: sorted list of the names of the objects
    #   I: list of couples (g,m) where "g" is an object name
    #                              and "m" a string representing the list of its attribute names
    #   
    #   Note: the rectangles are recorded in dictionaries R, R1, R2 {g:(m,disc)}
    #         where g is a list of object names,
    #               m a list of attribute names,
    #               disc the discarded indicator
    #   R1: dictionary of the rectangles being analyzed in the current step.
    #   R2: dictionary of the rectangles being discovered in the current step.
    #   R:  dictionary of the maximal rectangles discovered so far.
    #   Concepts: list of couples (g,m) describing the concepts discovered;
    #               will be written in output file "conceptFile"
    #   rawInclusion: list of couples of object names representing strict partial order;
    #               includes transitive couples.
    #               If (o1,o2) is in "rawInclusion", then name "o1" is a strict subset of name "o2".
    #   reducedInclusion: transitive reduction of "rawInclusion"
    #               will be written in output file "latticeFile"

    # Attention!
    # Updating the "g" part of rectangle rk = (gk)*(mk) when mk = mij in S2:
    # - Intuitively:                 R2[union(gk,gij)] = [mij,0]
    # - Standard algorithm:          R2[union(gk,gj)] = [mij,0]
    # However, this does not work: R2[union(g,gi)] = [mij,0]
    # Why?
    # Reason: according to the way i and j are chosen, rectangle ri may have been examined already.
    # So, its addition to rk is a no-op.
    # On the contrary, rj is examined for the first time.
    # So, it may change the composition of rk.

    from sysLib import getVarVal

    debug = True

    if debug: print 'Enter computeFCA'
    
    sep = getVarVal('csv-separator')

    startTimer()

    diag,G,I = loadContext(contextFile,sep) # I is a list of couples of lists 
    print 'Context loaded:',getTimer()
    if diag != 'OK':
        return diag
    
    if debug: print '\n+--------------------------'\
                    +'\n| STEP 1'\
                    +'\n+--------------------------'
    
    R = {}
    R2 = buildSize1Rectangles(I) # Build the rectangles of size 1
    if debug: printList('\nR1 initial (1-rectangles):',R2)
    
    step = 1
    while len(R2) > 1:
        startTimer()
        step += 1
        if debug: print '\n\n+--------------------------'\
                       +'\n| STEP '+str(step)\
                       +'\n+--------------------------'
        R1 = R2
        R2 = {}
        R2m = {}    # R2m[m:g] is the inverse dictionary of R2[G:(m,d)]
        
        for j in range(1,len(R1)):  ###
            gj,mj,disc = getRect(R1,j)
            #if disc:
            #   continue  # Error, though specified in the standard algorithm 
            for i in range(0,j):
                gi,mi,disc = getRect(R1,i)
                #if disc:
                #   continue  # Error though pecified in the standard algorithm
                mij = inter(mi,mj)  # gij = union(gi,gj) # Will be computed later, when needed
                if debug:
                    if mij != '':
                        print '\nMerge ('+gi+')*('+mi+') and ('+gj+')*('+mj+') from R1'
                        print '   Add new rectangle to R2: ('+union(gi,gj)+')*('+mij+')'
                    else:
                        print '\nMerge ('+gi+')*('+mi+') and ('+gj+')*('+mj+') from R1 - No match. Skip.' 

                if mij == '':
                    # No match. Skip
                    continue

                # Search "R2" for another rectangle rk = (gk)*(mk=mij)
                gk = getgOfmM(R2m,mij)
                gij = union(gi,gj)
                if gk is not None:
                    # Found one: expand it with "gij" or (better) "gj"
                    if debug:
                        print '   Found ('+gk+')*('+mij+') in R2 to merge the new rectangle with'
                        print '   Merge (!'+gij+'!)*('+mij+') with ('+gk+')*('+mij+'), which becomes ('+union(gk,gij)+')*('+mij+')'
                    del R2[gk]
                    del R2m[mij]
                    R2[union(gk,gj)] = [mij,0]      # Standard algorithm uses "gj" instead of "gij")
                    R2m[mij] = union(gk,gj)
                    # R2[union(gk,gij)] = [mij,0]      # Difference with the standard algorithm (we use "gij" instead of "gj")
                else:
                    # None found: insert new rectangle (gij)*(mij) in "R2"
                    R2[gij] = [mij,0]
                    R2m[mij] = gij

                # If one or both arguments have attributes mij, mark it in R1
                if mi == mij:
                    discard(R1,gi)
                    if debug:
                        print '   ('+gi+')*('+mi+') discarded in R1'
                if mj == mij:
                    discard(R1,gj)
                    if debug:
                        print '   ('+gj+')*('+mj+') discarded in R1'

                if debug:
                    printList('\n   R1:',R1)
                    printList('\n   R2:',R2)

        # the rectangles not discarded in "R1" belong to the solution
        addUndiscarded(R,R1)
        print ' - Level '+str(step-1)+':',getTimer()
        if debug: printList('\nMaximal rectangles:',R)

    # There may remain a rectangle in R2. If so, it is maximum. 
    if len(R2) > 0:
        addUndiscarded(R,R2)
        if debug:
            printList('\nFinal maximal rectangles:',R)

    Concepts = extractConcepts(R)

    rawInclusion = []
    reducedInclusion = []
    if latticeFile != u'':

        # Build concept inclusion list (transitive included)
        rawInclusion = []
        #for con1 in sorted(Concepts):
        for con1 in Concepts:
            #for con2 in [con2 for con2 in sorted(Concepts) if con2 <> con1]:
            for con2 in [con2 for con2 in Concepts if con2 <> con1]:
                if included(con1,con2):
                    rawInclusion.append([con1,con2])
        
        # Build reduced concept inclusion relation (= concept lattice)
        reducedInclusion = []
        for [con1,con2] in rawInclusion:
            status = 'basic'
            for con in [con for con in Concepts if con <> con1 and con <> con2]:
                if [con1,con] in rawInclusion and [con,con2] in rawInclusion:
                    status = 'transitive'
                    break
            if status == 'basic':
                reducedInclusion.append([con1,con2])

    diag = generateFCAfiles(Concepts,reducedInclusion,conceptFile,latticeFile,sep)

    return diag


def computeFCA_Sets(contextFile,conceptFile,latticeFile):
    from sysLib import getVarVal

    sep = getVarVal('csv-separator')

    startTimer()
    diag,G,I = loadContext(contextFile,sep)
    print '\nContext loaded in',getTimer()
    if diag != 'OK':
        return diag
    
    R = {}
    R2 = {g:(set(m.split(',')),0) for (g,m) in I}
    
    step = 0
    while len(R2) > 1:
        step += 1
        R1 = R2
        R2 = {}
        
        startTimer()
        for j in range(1,len(R1)):
            gj,mj,disc = getRect(R1,j)
            for i in range(0,j):
                gi,mi,disc = getRect(R1,i)
                mij = mi.intersection(mj)
                if mij == '':
                    continue
                gij = union(gi,gj)
                gk = getgOfm(R2,mij)
                if gk is not None:
                    del R2[gk]
                    R2[union(gk,gij)] = [mij,0]
                else:
                    R2[gij] = [mij,0]

                if mi == mij:
                    discard(R1,gi)
                if mj == mij:
                    discard(R1,gj)

        addUndiscarded(R,R1)
        print ' - Level '+str(step)+' executed in',getTimer()

    if len(R2) > 0:
        addUndiscarded(R,R2)

    Concepts = extractConcepts(R)

    rawInclusion = []
    reducedInclusion = []
    if latticeFile != u'':
        rawInclusion = []
        for con1 in Concepts:
            for con2 in [con2 for con2 in Concepts if con2 <> con1]:
                if included(con1,con2):
                    rawInclusion.append([con1,con2])
        reducedInclusion = []
        for [con1,con2] in rawInclusion:
            status = 'basic'
            for con in [con for con in Concepts if con <> con1 and con <> con2]:
                if [con1,con] in rawInclusion and [con,con2] in rawInclusion:
                    status = 'transitive'
                    break
            if status == 'basic':
                reducedInclusion.append([con1,con2])

    diag = generateFCAfiles(Concepts,reducedInclusion,conceptFile,latticeFile,sep)

    return diag


def computeFCA_Sets_Comments(contextFile,conceptFile,latticeFile):
    # < = computeFCA_Sets with comments>
    # Same as "computeFCA" but uses "sets" of attributes instead of "string lists".
    # Cannot use "sets" of objects because sets are not "hashable" as keys of dictionaries.
    
    from sysLib import getVarVal

    debug = False

    if debug: print '\nEnter computeFCA_Sets'

    sep = getVarVal('csv-separator')

    startTimer()
    diag,G,I = loadContext(contextFile,sep) # "G" is the list of object-names
                                            # "I" is a list of couples (object-name,'string-list')
                                            # where 'string-list' is the list of atributes of "object-name"
                                            # Example: [('O1','A1,A2,A3'),('O2','A3,A4'),...]
    print '\nContext loaded in',getTimer()
    if diag != 'OK':
        return diag

    print 'G:',len(G),'-',G[:5]
    print 'I:',len(I),'-',I[:5]

    if debug: print '\n+--------------------------'\
                    +'\n| STEP 0'\
                    +'\n+--------------------------'
    
    # R, R1, R2 and dictionaries {'string-list':(set('attributes'),disc)}
    # Example: 'O1,O3,O4': ({'A2', 'A5', 'A6'}, 0)
    R = {}
    R2 = {g:(set(m.split(',')),0) for (g,m) in I} # Build in R1 (via R2) the initial rectangles of size 1

    if debug: printList('\nR1 initial (1-rectangles):',R2)
    
    step = 0
    while len(R2) > 1:
        step += 1
        if debug: print '\n\n+--------------------------'\
                       +'\n| STEP '+str(step)\
                       +'\n+--------------------------'
        R1 = R2
        R2 = {}

        print '\n-- Step',step
        print 'R1:',R1[:5]
        print 'R2:',R2[:5]
        print 'R :',R
        
        startTimer()
        for j in range(1,len(R1)):
            # Extract rectangle "rj(gj,mj)"
            gj,mj,disc = getRect(R1,j)
            # According to the standard algorithm:
            # if disc:
            #     continue
            # Error because a discarded rectangle may still be used to derive other rectangles
            for i in range(0,j):
                # Extract rectangle "ri(gi,mi)"
                gi,mi,disc = getRect(R1,i)
                # Same comment about standard algorithm

                # "rj" and "ri" can generate a rectangle if they have some common attributes
                mij = mi.intersection(mj) # no need to compute gij now.
                if debug:
                    if mij != '':
                        print '\nMerge ('+gi+')*('+mi+') and ('+gj+')*('+mj+') from R1'
                        print '   Add new rectangle to R2: ('+union(gi,gj)+')*('+mij+')'
                    else:
                        print '\nMerge ('+gi+')*('+mi+') and ('+gj+')*('+mj+') from R1 - No match. Skip.' 

                if mij == '':   # No match. Skip
                    continue

                # OK, rj and ri can generate a rectangle
                
                # Now, "rij(gij,mij)" is complete
                gij = union(gi,gj)
                
                # Search "R2" for another rectangle "rk" with the same attributes: rk = (gk)*(mk=mij)
                gk = getgOfm(R2,mij)
                if gk is not None:
                    # Found one: expand it with "gij"
                    if debug:
                        print '   Found ('+gk+')*('+mij+') in R2 to merge the new rectangle with'
                        print '   Merge (!'+gij+'!)*('+mij+') with ('+gk+')*('+mij+'), which becomes ('+union(gk,gij)+')*('+mij+')'
                    del R2[gk]
                    R2[union(gk,gij)] = [mij,0]     # Difference with the standard algorithm (we use "gij" instead of "gj")
                else:
                    # None found: insert new rectangle "rij(gij,mij)" in "R2"
                    R2[gij] = [mij,0]

                # If one or both source arguments have attributes mij, mark it in R1
                if mi == mij:
                    discard(R1,gi)
                    if debug:
                        print '   ('+gi+')*('+mi+') discarded in R1'
                if mj == mij:
                    discard(R1,gj)
                    if debug:
                        print '   ('+gj+')*('+mj+') discarded in R1'

                if debug:
                    printList('\n   R1:',R1)
                    printList('\n   R2:',R2)

        # the rectangles not discarded in "R1" belong to the solution
        addUndiscarded(R,R1)
        print ' - Level '+str(step)+' executed in',getTimer()
        if debug: printList('\nMaximal rectangles:',R)

    # There may remain a rectangle in R2. If so, it is maximum. 
    if len(R2) > 0:
        addUndiscarded(R,R2)
        if debug:
            printList('\nFinal maximal rectangles:',R)

    Concepts = extractConcepts(R)

    rawInclusion = []
    reducedInclusion = []
    if latticeFile != u'':

        # Build concept inclusion list (transitive included)
        rawInclusion = []
        #for con1 in sorted(Concepts):
        for con1 in Concepts:
            #for con2 in [con2 for con2 in sorted(Concepts) if con2 <> con1]:
            for con2 in [con2 for con2 in Concepts if con2 <> con1]:
                if included(con1,con2):
                    rawInclusion.append([con1,con2])
        
        # Build reduced concept inclusion relation (= concept lattice)
        reducedInclusion = []
        for [con1,con2] in rawInclusion:
            status = 'basic'
            for con in [con for con in Concepts if con <> con1 and con <> con2]:
                if [con1,con] in rawInclusion and [con,con2] in rawInclusion:
                    status = 'transitive'
                    break
            if status == 'basic':
                reducedInclusion.append([con1,con2])

    diag = generateFCAfiles(Concepts,reducedInclusion,conceptFile,latticeFile,sep)

    return diag


'''
def computeFCA_Experimental(contextFile,conceptFile,latticeFile):
    from sysLib import getVarVal

    debug = False
    
    sep = getVarVal('csv-separator')

    startTimer()
    
    diag,G,I = loadContext(contextFile,sep)
    print 'Context loaded:',getTimer()
    if diag != 'OK':
        return diag
    
    if debug: print '\n+--------------------------'\
                    +'\n| STEP 1'\
                    +'\n+--------------------------'
    
    R = {}
    startTimer()
    R2 = buildSize1Rectangles(I) # Build R1, rectangles of size 1
    print 'Building size 1 rectangles:',getTimer()
    
    if debug: printList('\nR1 initial (1-rectangles):',R2)
    
    step = 1
    while len(R2) > 1:
        startTimer()
        step += 1
        if debug: print '\n\n+--------------------------'\
                       +'\n| STEP '+str(step)\
                       +'\n+--------------------------'
        R1 = R2
        L1 = [(g,m) for (g,[m,d]) in sorted(R1.items())] ###
        R2 = {}
        R2m = {}    ###
        for j in range(1,len(L1)):  ###
            gj,mj = getRectFromList(L1,j)    ###
            for i in range(0,j):
                gi,mi = getRectFromList(L1,i)    ###
                mij = inter(mi,mj)
                if debug:
                    if mij != '':
                        print '\nMerge ('+gi+')*('+mi+') and ('+gj+')*('+mj+') from R1'
                        print '   Add new rectangle to R2: ('+union(gi,gj)+')*('+mij+')'
                    else:
                        print '\nMerge ('+gi+')*('+mi+') and ('+gj+')*('+mj+') from R1 - No match. Skip.' 

                if mij == '':
                    continue

                # Search "R2" for another rectangle rk = (gk)*(mk=mij)
                gk = getgOfmM(R2m,mij)  ###
                gij = union(gi,gj)
                if gk is not None:
                    # Found one: expand it with "gij" or (better) "gj"
                    if debug:
                        print '   Found ('+gk+')*('+mij+') in R2 to merge the new rectangle with'
                        print '   Merge (!'+gij+'!)*('+mij+') with ('+gk+')*('+mij+'), which becomes ('+union(gk,gij)+')*('+mij+')'

                    #print '\n     --R2--'
                    #for key in sorted(R2):
                    #    print '    ',key,':',R2[key]
                        
                    #print '\n     --R2m--'
                    #for key in sorted(R2m):
                    #    print '    ',key,':',R2m[key]

                    del R2[gk]                   
                    del R2m[mij]    ###
                    R2[union(gk,gj)] = [mij,0]      # Standard algorithm uses "gj" instead of "gij")
                    R2m[mij] = union(gk,gj) ###
                    # R2[union(gk,gij)] = [mij,0]      # Difference with the standard algorithm (we use "gij" instead of "gj")
                else:
                    # None found: insert new rectangle (gij)*(mij) in "R2"
                    R2[gij] = [mij,0]
                    R2m[mij] = gij    ###

                # If one or both arguments have attributes mij, mark it in R1
                if mi == mij:
                    discard(R1,gi)
                    if debug:
                        print '   ('+gi+')*('+mi+') discarded in R1'
                if mj == mij:
                    discard(R1,gj)
                    if debug:
                        print '   ('+gj+')*('+mj+') discarded in R1'
                if debug:
                    printList('\n   R1:',R1)
                    printList('\n   R2:',R2)

        # the rectangles not discarded in "R1" belong to the solution
        addUndiscarded(R,R1)
        if debug: printList('\nMaximal rectangles:',R)
        print ' - Level '+str(step-1)+':',getTimer()

    # There may remain a rectangle in R2. If so, it is maximum. 
    if len(R2) > 0:
        addUndiscarded(R,R2)
        if debug:
            printList('\nFinal maximal rectangles:',R)

    Concepts = extractConcepts(R)

    rawInclusion = []
    reducedInclusion = []
    if latticeFile != u'':

        # Build concept inclusion list (transitive included)
        rawInclusion = []
        #for con1 in sorted(Concepts):
        for con1 in Concepts:
            #for con2 in [con2 for con2 in sorted(Concepts) if con2 <> con1]:
            for con2 in [con2 for con2 in Concepts if con2 <> con1]:
                if included(con1,con2):
                    rawInclusion.append([con1,con2])
        
        # Build reduced concept inclusion relation (= concept lattice)
        reducedInclusion = []
        for [con1,con2] in rawInclusion:
            status = 'basic'
            for con in [con for con in Concepts if con <> con1 and con <> con2]:
                if [con1,con] in rawInclusion and [con,con2] in rawInclusion:
                    status = 'transitive'
                    break
            if status == 'basic':
                reducedInclusion.append([con1,con2])

    diag = generateFCAfiles(Concepts,reducedInclusion,conceptFile,latticeFile,sep)

    return diag
'''
"""

# General constants
    
eol = '\n'

# Cryptographic constants
    
headerSecret = '-----BEGIN RSA PUBLIC KEY-----'
footerSecret = '-----END RSA PUBLIC KEY-----'
headerPublic = '-----BEGIN RSA PRIVATE KEY-----'
footerPublic = '-----END RSA PRIVATE KEY-----'


# Initialize character maps

frenchChar = [u'ä',u'á',u'à',u'â',\
              u'ç',\
              u'ë',u'é',u'è',u'ê',\
              u'ï',u'í',u'ì',u'î',\
              u'ö',u'ó',u'ò',u'ô',\
              u'ü',u'ú',u'ù',u'û',\
              u'Ä',u'Á',u'À',u'Â',\
              u'Ç',\
              u'Ë',u'É',u'È',u'Ê',\
              u'Ï',u'Í',u'Ì',u'Î',\
              u'Ö',u'Ó',u'Ò',u'Ô',\
              u'Ü',u'Ú',u'Ù',u'Û']

unFrenchChar = [u'a```',u'a``',u'a`',u'a^',\
                u'c`',\
                u'e```',u'e``',u'e`',u'e^',\
                u'i```',u'i``',u'i`',u'i^',\
                u'o```',u'o``',u'o`',u'o^',\
                u'u```',u'u``',u'u`',u'u^',\
                u'A```',u'A``',u'A`',u'A^',\
                u'C`',\
                u'E```',u'E``',u'E`',u'E^',\
                u'I```',u'I``',u'I`',u'I^',\
                u'O```',u'O``',u'O`',u'O^',\
                u'U```',u'U``',u'U`',u'U^']

# Globals for "PrettyPrint"
constantDic = {}    # dictionary of string constants
indentDic   = {}    # dictionary of indent tags
