-- ----------------------------------- --
-- Bases de donnes non relationnelles --
-- Le modle Attribut-valeur V1        --
-- ----------------------------------- --

openDB CLICOM.db;

create temporary table AV1_CLIENT(
   Entite    varchar(32) not null,
   Attribut  varchar(32) not null,
   Valeur    varchar(32),
   primary key (Entite,Attribut));

create temporary table AV1_PRODUIT(
   Entite    varchar(32) not null,
   Attribut  varchar(32) not null,
   Valeur    varchar(32) not null,
   primary key (Entite,Attribut));

create temporary table AV1_COMMANDE(
   Entite    varchar(32) not null,
   Attribut  varchar(32) not null,
   Valeur    varchar(32) not null,
   primary key (Entite,Attribut));

create temporary table AV1_DETAIL(
   Entite    varchar(32) not null,
   Attribut  varchar(32) not null,
   Valeur    varchar(32) not null,
   primary key (Entite,Attribut));


-- ---------------------------------- --
-- Vue exprimant les donnes sources --
-- ---------------------------------- --

create temporary view V_CLIENT as
select M1.Valeur as NCLI, M2.Valeur as NOM, 
       M3.Valeur as ADRESSE, 
       M4.Valeur as LOCALITE,
       M5.Valeur as CAT, 
       cast(M6.Valeur as real) as COMPTE
from   AV1_CLIENT as M1, AV1_CLIENT as M2, AV1_CLIENT as M3, 
       AV1_CLIENT as M4, AV1_CLIENT as M5, AV1_CLIENT as M6
where  M2.Entite = M1.Entite and M3.Entite = M1.Entite 
and    M4.Entite = M1.Entite and M5.Entite = M1.Entite
and    M6.Entite = M1.Entite
and    M1.Attribut = 'NCLI'
and    M2.Attribut = 'NOM'
and    M3.Attribut = 'ADRESSE'
and    M4.Attribut = 'LOCALITE'
and    M5.Attribut = 'CAT'
and    M6.Attribut = 'COMPTE';

create temporary view V_PRODUIT as
select M1.Valeur as NPRO, M2.Valeur as LIBELLE, 
       cast(M3.Valeur as char) as PRIX, 
       cast(M4.Valeur as real) as QSTOCK
from   AV1_PRODUIT AS M1, AV1_PRODUIT AS M2, AV1_PRODUIT AS M3, 
       AV1_PRODUIT AS M4
where  M2.Entite = M1.Entite and M3.Entite = M1.Entite 
and    M4.Entite = M1.Entite
and    M1.Attribut = 'NPRO'
and    M2.Attribut = 'LIBELLE'
and    M3.Attribut = 'PRIX'
and    M4.Attribut = 'QSTOCK';

create temporary view V_COMMANDE as
select M1.Valeur as NCOM, M2.Valeur as NCLI, 
       cast(M3.Valeur as char) as DATECOM
from   AV1_COMMANDE AS M1, AV1_COMMANDE AS M2, AV1_COMMANDE AS M3
where  M2.Entite = M1.Entite and M3.Entite = M1.Entite 
and    M1.Attribut = 'NCOM'
and    M2.Attribut = 'NCLI'
and    M3.Attribut = 'DATECOM';

create temporary view V_DETAIL as
select M1.Valeur as NCOM, M2.Valeur as NPRO, 
       cast(M3.Valeur as char) as QCOM
from   AV1_DETAIL AS M1, AV1_DETAIL AS M2, AV1_DETAIL AS M3
where  M2.Entite = M1.Entite and M3.Entite = M1.Entite 
and    M1.Attribut = 'NCOM'
and    M2.Attribut = 'NPRO'
and    M3.Attribut = 'QCOM';


-- ---------------------------- --
-- index d'identifiants globaux --
-- ---------------------------- --

create index CLI_ATTR on AV1_CLIENT(Entite,Attribut);
create index PRO_ATTR on AV1_PRODUIT(Entite,Attribut);
create index COM_ATTR on AV1_COMMANDE(Entite,Attribut);
create index DET_ATTR on AV1_DETAIL(Entite,Attribut);

-- ------------------------------------------ --
-- index d'identifiants des tables sources    --
-- ------------------------------------------ --

create unique index CLI_NCLI  on AV1_CLIENT(Valeur)   where Attribut = 'NCLI';
create unique index CLI_NPRO  on AV1_PRODUIT(Valeur)  where Attribut = 'NPRO';
create unique index COM_NCOM  on AV1_COMMANDE(Valeur) where Attribut = 'NCOM';
-- L'identifiant de DETAIL n'est exprimable que via un dclencheur

-- ------------------------ --
-- index de cls trangres --
-- ------------------------ --

create index COM_NCLI  on AV1_COMMANDE(Valeur) where Attribut = 'NCLI';
create index DET_NCOM  on AV1_DETAIL(Valeur)   where Attribut = 'NCOM';
create index DET_NPRO  on AV1_DETAIL(Valeur)   where Attribut = 'NPRO';


-- ---------------------- --
-- Chargement des donnes --
-- ---------------------- --

insert into AV1_CLIENT (Entite,Attribut,Valeur) 
       select NCLI,'NCLI',NCLI from CLIENT;
insert into AV1_CLIENT (Entite,Attribut,Valeur) 
       select NCLI,'NOM',NOM from CLIENT;
insert into AV1_CLIENT (Entite,Attribut,Valeur) 
       select NCLI,'ADRESSE',ADRESSE from CLIENT;
insert into AV1_CLIENT (Entite,Attribut,Valeur) 
       select NCLI,'LOCALITE',LOCALITE from CLIENT;
insert into AV1_CLIENT (Entite,Attribut,Valeur) 
       select NCLI,'CAT',CAT from CLIENT;
insert into AV1_CLIENT (Entite,Attribut,Valeur) 
       select NCLI,'COMPTE',COMPTE from CLIENT;

insert into AV1_PRODUIT (Entite,Attribut,Valeur) 
       select NPRO,'NPRO',NPRO from PRODUIT;
insert into AV1_PRODUIT (Entite,Attribut,Valeur) 
       select NPRO,'LIBELLE',LIBELLE from PRODUIT;
insert into AV1_PRODUIT (Entite,Attribut,Valeur) 
       select NPRO,'PRIX',PRIX from PRODUIT;
insert into AV1_PRODUIT (Entite,Attribut,Valeur) 
       select NPRO,'QSTOCK',QSTOCK from PRODUIT;

insert into AV1_COMMANDE (Entite,Attribut,Valeur) 
       select NCOM,'NCOM',NCOM from COMMANDE;
insert into AV1_COMMANDE (Entite,Attribut,Valeur) 
       select NCOM,'NCLI',NCLI from COMMANDE;
insert into AV1_COMMANDE (Entite,Attribut,Valeur) 
       select NCOM,'DATECOM',DATECOM from COMMANDE;

insert into AV1_DETAIL (Entite,Attribut,Valeur) 
       select NCOM||'-'||NPRO,'NCOM',NCOM from DETAIL;
insert into AV1_DETAIL (Entite,Attribut,Valeur) 
       select NCOM||'-'||NPRO,'NPRO',NPRO from DETAIL;
insert into AV1_DETAIL (Entite,Attribut,Valeur) 
       select NCOM||'-'||NPRO,'QCOM',QCOM from DETAIL;


write-b Table AV1_CLIENT;
select * from AV1_CLIENT   order by Entite;
write-b Table AV1_PRODUIT;
select * from AV1_PRODUIT  order by Entite;
write-b Table AV1_COMMANDE;
select * from AV1_COMMANDE order by Entite;
write-b Table AV1_DETAIL;
select * from AV1_DETAIL   order by Entite;

write-b Table source CLIENT reconstitue par auto-jointure multiple;
select * from V_CLIENT     order by NCLI;
write-b Table source PRODUIT reconstitue par auto-jointure multiple;
select * from V_PRODUIT    order by NPRO;
write-b Table source COMMANDE reconstitue par auto-jointure multiple;
select * from V_COMMANDE   order by NCOM;
write-b Table source DETAIL reconstitue par auto-jointure multiple;
select * from V_DETAIL     order by NCOM,NPRO;
write;

-- Variante de reconstruction de la table source CLIENT
write-b Table source CLIENT reconstitue par groupement;
select 
max(case when Attribut = 'NCLI'     then Valeur end) as NCLI,
max(case when Attribut = 'NOM'      then Valeur end) as NOM,
max(case when Attribut = 'ADRESSE'  then Valeur end) as ADRESSE,
max(case when Attribut = 'LOCALITE' then Valeur end) as LOCALITE,
max(case when Attribut = 'CAT'      then Valeur end) as CAT,
max(case when Attribut = 'COMPTE'   then cast(Valeur as real) else -999999.9 end) as COMPTE
from  AV1_CLIENT
group by Entite;

closeDB;

write-b Fin de script;
