-- ----------------------------------- --
-- Bases de donnes non relationnelles --
-- Le modle de table universelle      --
-- ----------------------------------- --

openDB CLICOM.db;

create temporary table CLICOM(
   EID       integer not null primary key autoincrement,
   TYPE      char(18) not null,
   NCLI      char(10),
   NOM       char(32),
   ADRESSE   char(60),
   LOCALITE  char(30),
   CAT       char(2),
   COMPTE    decimal(9,2),
   NPRO      char(15),
   LIBELLE   char(60),
   PRIX      integer,
   QSTOCK    integer,
   NCOM      char(12),
   DATECOM   date,
   QCOM      integer);

-- ---------------------------------- --
-- Vues exprimant les donnes sources --
-- ---------------------------------- --

create temporary view V_CLIENT as
select  NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE
from    CLICOM  where TYPE = 'CLI';

create temporary view V_PRODUIT as
select  NPRO,LIBELLE,PRIX,QSTOCK
from    CLICOM  where TYPE = 'PRO';

create temporary view V_COMMANDE as
select  NCOM,NCLI,DATECOM
from    CLICOM  where TYPE = 'COM';

create temporary view V_DETAIL as
select  NCOM,NPRO,QCOM
from    CLICOM  where TYPE = 'DET';

-- ------------------------ --
-- index d'identifiants     --
-- ------------------------ --

create unique index CLI_NCLI
on CLICOM(NCLI) where TYPE = 'CLI';

create unique index PRO_NPRO
on CLICOM(NPRO) where TYPE = 'PRO';

create unique index COM_NCOM
on CLICOM(NCOM) where TYPE = 'COM';

create unique index DET_NCOM_NPRO
on CLICOM(NCOM,NPRO) where TYPE = 'DET';

-- ------------------------ --
-- index de cls trangres --
-- ------------------------ --

create index COM_NCLI
on CLICOM(NCLI) where TYPE = 'COM';

create index DET_NPRO
on CLICOM(NPRO) where TYPE = 'DET';

-- ------------------------ --
-- Index de jointure        --
-- ------------------------ --

create index COMDET_NCOM
on CLICOM(NCOM) where TYPE in ('COM','DET');

create index CLICOM_NCLI
on CLICOM(NCLI) where TYPE in ('CLI','COM');

create index PRODET_NCPRO
on CLICOM(NPRO) where TYPE in ('PRO','DET');

-- ---------------------- --
-- Chargement des donnes --
-- ---------------------- --

insert into CLICOM(TYPE,NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE)
select 'CLI',NCLI,NOM,ADRESSE,LOCALITE,CAT,COMPTE from CLIENT;

insert into CLICOM(TYPE,NPRO,LIBELLE,PRIX,QSTOCK)
select 'PRO',NPRO,LIBELLE,PRIX,QSTOCK from PRODUIT;

insert into CLICOM(TYPE,NCOM,NCLI,DATECOM)
select 'COM',NCOM,NCLI,DATECOM from COMMANDE;

insert into CLICOM(TYPE,NCOM,NPRO,QCOM)
select 'DET',NCOM,NPRO,QCOM from DETAIL;

write-b Contenu de la table universelle CLICOM;
select * from CLICOM;


write-b Commande 30184 et ses dtails;
select  NCOM,NCLI,DATECOM,NPRO,QCOM
from    CLICOM 
where   NCOM = '30184'
order by NCOM,NPRO asc;


write-b Liste des commandes et leurs dtails;
select  NCOM,NCLI,DATECOM,NPRO,QCOM
from    CLICOM 
where   TYPE in ('COM','DET')
order by NCOM,NPRO asc;

write-b Liste des clients et leurs commandes;
select  NCLI,NOM,LOCALITE,NCOM,DATECOM
from    CLICOM 
where   TYPE in ('CLI','COM')
order by NCLI,NCOM asc;

write-b Liste des produits et leurs dtails;
select  NPRO,LIBELLE,PRIX,NCOM,QCOM
from    CLICOM 
where   TYPE in ('PRO','DET')
order by NPRO,NCOM asc;

write-b Relecture des donnes sources;
select * from V_CLIENT order by NCLI;
select * from V_PRODUIT order by NPRO;
select * from V_COMMANDE order by NCOM;
select * from V_DETAIL order by NCOM,NPRO;

closeDB;

write-b Fin de script;
