write-b -- ---------------------------------------------- --;
write   -- Recherche des identifiants d'une table anonyme --;
write   -- par analyse des donnes                        --;
write   -- Voir annexe 23                                 --;
write   -- ---------------------------------------------- --;

-- Selection d'une base de donnes --
-- ------------------------------- --
selectFile nomBD = $dbDirectory$,Base de donnes,db;
if ('$DIALOGbutton$' = 'Cancel' or '$nomBD$' = '') return;

openDB $nomBD$;

createDictionary;

-- Statistiques globales de la base de donnes --
-- ------------------------------------------- --
write-b --- Analyse prliminaire de la base de donnes ---;
for idT,nomT = [select TableID,TableName from SYS_TABLE where TableName not like 'SYS_%'];
  extract NLinT = select count(*) from $nomT$;
  extract NLinTu = select count(*) from (select distinct * from $nomT$);
  write-b La table $nomT$ contient $NLinT$ ligne(s) dont $NLinTu$ distincte(s);
  for nomC,CNull = [select ColName,ColNull from SYS_COLUMN where TableID = $idT$];
    extract NVal = select count(distinct $nomC$) from $nomT$;
    if ('$CNull$' = 'N');
      write @S3la colonne $nomC$ possde $NVal$ valeur(s) distincte(s);
    else;
      extract NValN = select count(*) from $nomT$ where $nomC$ is null;
      write @S3la colonne $nomC$ possde $NVal$ valeur(s) distinctes et $NValN$ valeur(s) NULL;
    endif;
  endfor;
endfor;


create temporary table COMB(List varchar(256),Pattern varchar(256),Size integer);
create temporary table ID(List varchar(256),Pattern varchar(256),Size integer);

-- Analyse d'une table --
-- ------------------- --
while (True);
  ask nomT = [/bSlectionner une table] Table:[select TableName from SYS_TABLE
                                               where TableName not like 'SYS_%'
                                               and   TableName not in ('ID','COMB')];
  if ('$DIALOGbutton$' = 'Cancel' or '$nomT$' = '')
     exit;
  extract idT = select TableID from SYS_TABLE where TableName = '$nomT$';

  -- Taille de la table --
  extract NColT = select count(*) from SYS_COLUMN where TableID = $idT$;

  -- Taille maximum des combinaisons --
  set NColI = 3;
  ask-u NColI = [/bTaille maximum des identifiants] Colonnes:;
  compute NColI = min(max(1,$NColI$),$NColT$);

  -- Gnration des combinaisons dans COMB --
  with LISTS(List,Last,Size) as
    (select ColName,ColSeq,1 from SYS_COLUMN
     where  TableID = $idT$
       union all
     select L.List||','||S.ColName,S.ColSeq,L.Size + 1 
     from   LISTS L,SYS_COLUMN S
     where  TableID = $idT$
     and    S.ColSeq > L.Last
     and    L.Size < $NColI$)
  insert into COMB(List,Pattern,Size) 
  select List,'%'||replace(List,',','%')||'%',Size
  from   LISTS;

  -- Nombre de combinaisons --
  extract NComb = select count(*) from COMB;
  write-b Identifiants candidats de la table $nomT$;
  write @S2 $NComb$ combinaisons de 1  $NColI$ colonnes parmi $NColT$ ont t gnres.;

  -- La table est-elle vide ? --
  extract NLinT = select count(*) from $nomT$;
  if ($NLinT$ = 0);
    write @S2 La table $nomT$ est vide. Pas de conclusion possible.
    next;
  endif;

  delete from ID;

  -- Calcul des identifiants candidats, par longueur croissante --
  set NTest = 0;
  for len = [1,$NColI$];
    -- Test de chaque combinaison de taille "len"--
    for lst,pat,siz = [select List,Pattern,Size from COMB where Size = $len$ order by List];
      extract Nval = select count(*) from (select distinct $lst$ from $nomT$);
      compute NTest = $NTest$ + 1;
      if ($Nval$ = $NLinT$) insert into ID values('$lst$','$pat$',$siz$);
    endfor;

    -- Suppression des combinaisons de taille "len" --
    delete from COMB where Size = $len$;

    -- Suppression des combinaisons contenant les identifiants de taille "len" --
    for pat = [select Pattern from ID where Size = $len$]
      delete from COMB where  List like '$pat$';

    -- Y a-t-il encore des combinaisons  analyser ? --
    extract Nc = select count(*) from COMB;
    if ($Nc$ = 0) exit;
  endfor;

  -- Affichage du diagnostic --
  write @S2 $NTest$ combinaisons ont t values.;
  extract Nid = select count(*) from ID;
  if ($Nid$ = 0) write-a @S2 aucun identifiant n'a t trouv.;
  if ($Nid$ = 1) write-a @S2 un identifiant candidat a t trouv.;
  if ($Nid$ > 1) write-a @S2 $Nid$ identifiants candidats ont t trouvs :;
  select List as "Identifiants candidats de $nomT$" from ID;
endwhile;

closeDB;

write-ab Analyse termine.;

return;

-- Table des variables --
-- ------------------- --
nomBD nom complet de la base de donnes
ID    table des identifiants 
COMB  table des combinaisons
NComb nombre de combinaisons gnres
nomT  nom de la table analyse
idT   identifiant de la table analyse
NLinT nombre de lignes de la table
NColT nombre de colonnes de la table
NColI nombre maximum de colonnes des identifiants
NTest nombre d'analyses de donnes
