-- ---------------------------------------------------------------------------- --
-- Cration de la base de donnes ZOO.db et chargement des donnes initiales    --
-- ---------------------------------------------------------------------------- --

createOrReplaceDB ZOO.db;

create table ESPECE (
     NOM_ESPECE char(32) not null,
     primary key (NOM_ESPECE));

create table ANIMAL (
     -- POIDS : en kg
     NOM_ANIMAL char(32) not null,
     POIDS      decimal(10,4) not null,
     NOM_ESPECE char(32) not null,
     primary key (NOM_ANIMAL),
     foreign key (NOM_ESPECE) references ESPECE);

create table SOINS (
     NOM_ANIMAL  char(32) not null,
     DATE_SOINS  date not null,
     COUT        decimal(6,2) not null,
     primary key (NOM_ANIMAL,DATE_SOINS),
     foreign key (NOM_ANIMAL) references ANIMAL);

create table REGIME (
     NOM_ANIMAL  char(32) not null,
     DATE_REGIME date not null,
     primary key (NOM_ANIMAL,DATE_REGIME),
     foreign key (NOM_ANIMAL) references ANIMAL);

create table INGREDIENT (
     -- COUT_UNITAIRE : par kg
     NOM_INGREDIENT char(32) not null,
     COUT_UNITAIRE  decimal(10,4) not null,
     primary key (NOM_INGREDIENT));

create table NUTRIMENT (
     NOM_NUTRIMENT char(32) not null,
     primary key (NOM_NUTRIMENT));

create table BESOINS (
     -- MIN et MAX : en gr/kg/j
     NOM_ESPECE    char(32) not null,
     NOM_NUTRIMENT char(32) not null,
     MIN           decimal(8,4) not null,
     MAX           decimal(8,4) not null,
     primary key (NOM_NUTRIMENT,NOM_ESPECE),
     foreign key (NOM_NUTRIMENT) references NUTRIMENT,
     foreign key (NOM_ESPECE) references ESPECE,
     check(MAX >= MIN));

create table COMPOSITION (
     -- QUANTITE : en gr
     NOM_INGREDIENT char(32) not null,
     NOM_ANIMAL     char(32) not null,
     DATE_REGIME    date not null,
     QUANTITE       decimal(8,4) not null,
     primary key (NOM_INGREDIENT,NOM_ANIMAL,DATE_REGIME),
     foreign key (NOM_INGREDIENT) references INGREDIENT,
     foreign key (NOM_ANIMAL,DATE_REGIME) references REGIME);

create table TENEUR (
     -- QUANTITE : en gr/gr
     NOM_INGREDIENT char(32) not null,
     NOM_NUTRIMENT  char(32) not null,
     QUANTITE       decimal(8,4) not null,
     primary key (NOM_INGREDIENT,NOM_NUTRIMENT),
     foreign key (NOM_INGREDIENT) references INGREDIENT,
     foreign key (NOM_NUTRIMENT) references NUTRIMENT);

create index FK_ANI_ESP on ANIMAL (NOM_ESPECE);
create index FK_BES_ESP on BESOINS (NOM_ESPECE);
create index FK_COM_REG on COMPOSITION (NOM_ANIMAL,DATE_REGIME);
create index FK_TEN_ING on TENEUR (NOM_NUTRIMENT);

insert into ESPECE values
        ('tortue'),('crocodile'),('serpent'),
        ('toucan'),('ara'),('vautour'),
        ('tapir'),('rhinocros'),('zbre'),
        ('loup'),('jaguar'),('lion');

insert into ANIMAL values
	('Barbie',15.8,'tortue'),
	('Ewok',127.0,'crocodile'),
	('Calvin',8.5,'serpent'),		
	('Paloma',0.5,'toucan'),
	('Beau',1.3,'ara'),
	('Ava',7.7,'vautour'),		
	('Casimir',132.0,'tapir'),
	('Elsa',830.0,'rhinocros'),
	('Elvis',285.0,'zbre'),		
	('Gina',45.5,'loup'),
	('Frida',42.0,'jaguar'),
	('Csar',190.0,'lion'),
	('Caroline',35.2,'tortue'),
	('Capitaine',308.5,'crocodile'),
	('Koko',13.0,'serpent'),		
	('Merlin',0.9,'toucan'),
	('Circ',1.6,'ara'),
	('Maya',10.5,'vautour'),		
	('Heidi',188.0,'tapir'),
	('Attila',1090.0,'rhinocros'),
	('Winnie',337.0,'zbre'),		
	('Pluto',52.0,'loup'),
	('Felix',53.0,'jaguar'),
	('Princesse',128.0,'lion'),
	('Junon',135.0,'lion');

insert into INGREDIENT values
	('multivit-ADE1',8.5),
	('multivit-ADE2',11.8),
	('multivit-ADE3',15.4),
	('granules-P06',2.0),
	('granules-P12',3.5),
	('biofin-0',18.0),
	('biofin-2',23.5),
	('salade',0.8),
	('fourrage',1.6),
	('eau',0.05),
	('viande-rouge',4),
	('viande-blanche',3.5),
	('poisson',2.2);

insert into NUTRIMENT values
	('protines-animales'),
	('protines-vgtales'),
	('vitamines-A'),
	('vitamines-D'),
	('vitamines-E'),
	('minraux'),
	('eau'),
	('lipides'),
	('glucides');

insert into BESOINS values
	('tortue','protines-animales',0.0,0.1),
	('crocodile','protines-animales',10.0,20.0),
	('serpent','protines-animales',8.0,16.0),
	('toucan','protines-animales',0.0,0.1),
	('ara','protines-animales',0.0,0.1),
	('vautour','protines-animales',10.0,20.0),
	('tapir','protines-animales',0.0,0.1),
	('rhinocros','protines-animales',0.0,0.3),
	('zbre','protines-animales',0.0,0.2),
	('loup','protines-animales',15.0,35.0),
	('jaguar','protines-animales',15.0,35.0),
	('lion','protines-animales',13.5,28.5);

insert into BESOINS values
	('tortue','protines-vgtales',8.5,18.5),
	('crocodile','protines-vgtales',0.01,0.3),
	('serpent','protines-vgtales',1.0,2.0),
	('toucan','protines-vgtales',10.0,20.0),
	('ara','protines-vgtales',12.0,25.0),
	('vautour','protines-vgtales',1.0,3.0),
	('tapir','protines-vgtales',16.0,35.0),
	('rhinocros','protines-vgtales',14.0,25.0),
	('zbre','protines-vgtales',18.0,35.0),
	('loup','protines-vgtales',1.0,3.0),
	('jaguar','protines-vgtales',1.0,3.0),
	('lion','protines-vgtales',1.0,3.0);

insert into BESOINS values
	('tortue','vitamines-A',0.01,0.04),
	('crocodile','vitamines-A',0.01,0.04),
	('serpent','vitamines-A',0.0001,0.0003),
	('toucan','vitamines-A',0.0002,0.0006),
	('ara','vitamines-A',0.0002,0.0006),
	('vautour','vitamines-A',0.0001,0.0003),
	('tapir','vitamines-A',0.0001,0.0003),
	('rhinocros','vitamines-A',0.0001,0.0003),
	('zbre','vitamines-A',0.01,0.04),
	('loup','vitamines-A',0.0002,0.0006),
	('jaguar','vitamines-A',0.0002,0.0006),
	('lion','vitamines-A',0.0001,0.0003);

insert into BESOINS values
	('tortue','vitamines-D',0.0001,0.0002),
	('crocodile','vitamines-D',0.01,0.035),
	('serpent','vitamines-D',0.0001,0.0002),
	('toucan','vitamines-D',0.0001,0.0002),
	('ara','vitamines-D',0.0001,0.0002),
	('vautour','vitamines-D',0.0001,0.0002),
	('tapir','vitamines-D',0.0001,0.0002),
	('rhinocros','vitamines-D',0.0001,0.0002),
	('zbre','vitamines-D',0.0001,0.0002),
	('loup','vitamines-D',0.01,0.05),
	('jaguar','vitamines-D',0.0001,0.0002),
	('lion','vitamines-D',0.0001,0.0002);

insert into BESOINS values
	('tortue','vitamines-E',0.0001,0.03),
	('crocodile','vitamines-E',0.005,0.03),
	('serpent','vitamines-E',0.0001,0.0003),
	('toucan','vitamines-E',0.0002,0.0006),
	('ara','vitamines-E',0.0002,0.0006),
	('vautour','vitamines-E',0.0001,0.0003),
	('tapir','vitamines-E',0.0001,0.0003),
	('rhinocros','vitamines-E',0.0001,0.0003),
	('zbre','vitamines-E',0.0001,0.0003),
	('loup','vitamines-E',0.0002,0.0006),
	('jaguar','vitamines-E',0.0002,0.0006),
	('lion','vitamines-E',0.0001,0.0003);

insert into BESOINS values
	('tortue','minraux',0.0050,0.0050),
	('crocodile','minraux',0.01,0.1),
	('serpent','minraux',0.0050,0.0050),
	('toucan','minraux',0.0080,0.0080),
	('ara','minraux',0.0080,0.0080),
	('vautour','minraux',0.0070,0.0070),
	('tapir','minraux',0.0100,0.0100),
	('rhinocros','minraux',0.0100,0.0100),
	('zbre','minraux',0.0150,0.0150),
	('loup','minraux',0.0250,0.0250),
	('jaguar','minraux',0.0350,0.0350),
	('lion','minraux',0.0200,0.0200);

insert into BESOINS values
	('tortue','eau',30.03,1000.0),
	('crocodile','eau',10.0,1000.0),
	('serpent','eau',30.03,1000.0),
	('toucan','eau',40.04,1000.0),
	('ara','eau',40.04,1000.0),
	('vautour','eau',30.03,1000.0),
	('tapir','eau',100.1,1000.0),
	('rhinocros','eau',100.1,1000.0),
	('zbre','eau',100.1,1000.0),
	('loup','eau',80.08,1000.0),
	('jaguar','eau',80.08,1000.0),
	('lion','eau',80.08,1000.0);

insert into BESOINS values
	('tortue','lipides',1.0,2.0),
	('crocodile','lipides',1.0,6.0),
	('serpent','lipides',1.0,3.0),
	('toucan','lipides',1.0,3.0),
	('ara','lipides',1.0,3.0),
	('vautour','lipides',1.0,4.0),
	('tapir','lipides',0.4,2.4),
	('rhinocros','lipides',0.4,2.4),
	('zbre','lipides',0.4,1.4),
	('loup','lipides',2.0,5.0),
	('jaguar','lipides',3.0,6.0),
	('lion','lipides',3.0,6.0);

insert into BESOINS values
	('tortue','glucides',1.8,3.0),
	('crocodile','glucides',0.2,9.0),
	('serpent','glucides',2.0,4.0),
	('toucan','glucides',2.8,4.0),
	('ara','glucides',2.8,4.0),
	('vautour','glucides',2.8,4.0),
	('tapir','glucides',2.5,5.0),
	('rhinocros','glucides',2.5,5.0),
	('zbre','glucides',2.8,5.0),
	('loup','glucides',4.5,8.0),
	('jaguar','glucides',4.1,8.0),
	('lion','glucides',4.1,8.0);

insert into TENEUR values
	('multivit-ADE1','protines-animales',0.0),
	('multivit-ADE2','protines-animales',0.0),
	('multivit-ADE3','protines-animales',0.0),
	('granules-P06','protines-animales',0.08),
	('granules-P12','protines-animales',0.10),
	('biofin-0','protines-animales',0.0),
	('biofin-2','protines-animales',0.0),
	('salade','protines-animales',0.0),
	('fourrage','protines-animales',0.0),
	('eau','protines-animales',0.0),
	('viande-rouge','protines-animales',0.28),
	('viande-blanche','protines-animales',0.26),
	('poisson','protines-animales',0.24);

insert into TENEUR values
	('multivit-ADE1','protines-vgtales',0.0),
	('multivit-ADE2','protines-vgtales',0.0),
	('multivit-ADE3','protines-vgtales',0.0),
	('granules-P06','protines-vgtales',0.20),
	('granules-P12','protines-vgtales',0.30),
	('biofin-0','protines-vgtales',0.0),
	('biofin-2','protines-vgtales',0.0),
	('salade','protines-vgtales',0.16),
	('fourrage','protines-vgtales',0.18),
	('eau','protines-vgtales',0.0),
	('viande-rouge','protines-vgtales',0.0),
	('viande-blanche','protines-vgtales',0.0),
	('poisson','protines-vgtales',0.0);

insert into TENEUR values
	('multivit-ADE1','vitamines-A',0.01),
	('multivit-ADE2','vitamines-A',0.05),
	('multivit-ADE3','vitamines-A',0.05),
	('granules-P06','vitamines-A',0.005),
	('granules-P12','vitamines-A',0.005),
	('biofin-0','vitamines-A',0.0),
	('biofin-2','vitamines-A',0.0),
	('salade','vitamines-A',0.004),
	('fourrage','vitamines-A',0.002),
	('eau','vitamines-A',0.0),
	('viande-rouge','vitamines-A',0.0006),
	('viande-blanche','vitamines-A',0.0005),
	('poisson','vitamines-A',0.0008);

insert into TENEUR values
	('multivit-ADE1','vitamines-D',0.005),
	('multivit-ADE2','vitamines-D',0.005),
	('multivit-ADE3','vitamines-D',0.010),
	('granules-P06','vitamines-D',0.005),
	('granules-P12','vitamines-D',0.005),
	('biofin-0','vitamines-D',0.0),
	('biofin-2','vitamines-D',0.0),
	('salade','vitamines-D',0.002),
	('fourrage','vitamines-D',0.004),
	('eau','vitamines-D',0.0),
	('viande-rouge','vitamines-D',0.0004),
	('viande-blanche','vitamines-D',0.0003),
	('poisson','vitamines-D',0.0006);

insert into TENEUR values
	('multivit-ADE1','vitamines-E',0.01),
	('multivit-ADE2','vitamines-E',0.02),
	('multivit-ADE3','vitamines-E',0.04),
	('granules-P06','vitamines-E',0.005),
	('granules-P12','vitamines-E',0.005),
	('biofin-0','vitamines-E',0.0),
	('biofin-2','vitamines-E',0.0),
	('salade','vitamines-E',0.005),
	('fourrage','vitamines-E',0.003),
	('eau','vitamines-E',0.0),
	('viande-rouge','vitamines-E',0.0001),
	('viande-blanche','vitamines-E',0.0002),
	('poisson','vitamines-E',0.0004);

insert into TENEUR values
	('multivit-ADE1','minraux',0.15),
	('multivit-ADE2','minraux',0.20),
	('multivit-ADE3','minraux',0.25),
	('granules-P06','minraux',0.04),
	('granules-P12','minraux',0.05),
	('biofin-0','minraux',0.03),
	('biofin-2','minraux',0.05),
	('salade','minraux',0.005),
	('fourrage','minraux',0.005),
	('eau','minraux',0.0),
	('viande-rouge','minraux',0.001),
	('viande-blanche','minraux',0.001),
	('poisson','minraux',0.004);

insert into TENEUR values
	('multivit-ADE1','eau',0.001),
	('multivit-ADE2','eau',0.001),
	('multivit-ADE3','eau',0.001),
	('granules-P06','eau',0.000),
	('granules-P12','eau',0.000),
	('biofin-0','eau',0.025),
	('biofin-2','eau',0.025),
	('salade','eau',0.70),
	('fourrage','eau',0.20),
	('eau','eau',1.0),
	('viande-rouge','eau',0.65),
	('viande-blanche','eau',0.55),
	('poisson','eau',0.60);

insert into TENEUR values
	('multivit-ADE1','lipides',0.0),
	('multivit-ADE2','lipides',0.0),
	('multivit-ADE3','lipides',0.0),
	('granules-P06','lipides',0.1),
	('granules-P12','lipides',0.15),
	('biofin-0','lipides',0.08),
	('biofin-2','lipides',0.1),
	('salade','lipides',0.0),
	('fourrage','lipides',0.0),
	('eau','lipides',0.0),
	('viande-rouge','lipides',0.15),
	('viande-blanche','lipides',0.07),
	('poisson','lipides',0.15);

insert into TENEUR values
	('multivit-ADE1','glucides',0.0),
	('multivit-ADE2','glucides',0.1),
	('multivit-ADE3','glucides',0.1),
	('granules-P06','glucides',0.06),
	('granules-P12','glucides',0.12),
	('biofin-0','glucides',0.10),
	('biofin-2','glucides',0.15),
	('salade','glucides',0.1),
	('fourrage','glucides',0.1),
	('eau','glucides',0.0),
	('viande-rouge','glucides',0.003),
	('viande-blanche','glucides',0.001),
	('poisson','glucides',0.001);

delete from COMPOSITION;
delete from REGIME;
insert into REGIME values
	('Capitaine','2018-02-22');

insert into COMPOSITION values
	('poisson','Capitaine','2018-02-22',4000.0),
	('viande-rouge','Capitaine','2018-02-22',10000.0),
	('multivit-ADE2','Capitaine','2018-02-22',2.0),
	('granules-P06','Capitaine','2018-02-22',30.0),
	('biofin-2','Capitaine','2018-02-22',12.0);

commitDB;

closeDB;