write-b -- ------------------------------- --;
write   -- Active databases                --;
write   -- Type-subtype implementation     --;
write-a -- ------------------------------- --;

set ; = ;;

createDB InMemory;

-- ----------------------------------------- --
-- THE TECHNICAL TABLES                      --
-- ----------------------------------------- --

create table _PERSON (
        PersID  char(5) not null primary key,
        Name    varchar(32) not null);

create table _PROFESSOR (
        PersID  char(5) not null primary key
                references _PERSON 
                on update cascade
                on delete cascade,
        Specialty varchar(24) not null);

create table _STUDENT (
        PersID  char(5) not null primary key
                references _PERSON
                on update cascade
                on delete cascade,
        Credits integer not null);


-- ----------------------------------------- --
-- THE USER VIEWS                            --
-- ----------------------------------------- --

create view PERSON (PersID,Name,Specialty,Credits)
   as  select P.PersID,Name,Specialty,Credits
       from _PERSON P
            left join _PROFESSOR F using (PersID)
            left join _STUDENT S using (PersID);

create view PROFESSOR (PersID,Name,Specialty)
   as  select P.PersID,Name,Specialty
       from  _PERSON P, _PROFESSOR F
       where  P.PersID = F.PersID;

create view STUDENT (PersID,Name,Credits)
   as  select P.PersID,Name,Credits
       from _PERSON P, _STUDENT S
       where P.PersID = S.PersID;


-- ----------------------------------------- --
-- THE INSERT TRIGGERS                       --
-- ----------------------------------------- --

create trigger TRG_POP_INSERT
instead of insert on PERSON
for each row
begin
   insert into _PERSON
      values (new.PersID,new.Name) $;$
   --
   insert into _PROFESSOR 
      select new.PersID,new.Specialty where new.Specialty is not null $;$
   --
   insert into _STUDENT 
      select new.PersID,new.Credits where new.Credits is not null $;$
end;

create trigger TRG_PRO_INSERT
instead of insert on PROFESSOR
for each row
when new.Specialty is not null
begin
   insert into _PERSON
      select new.PersID,new.Name
      where new.PersID not in (select PersID from _PERSON) $;$
   --
   insert into _PROFESSOR
      values (new.PersID,new.Specialty) $;$
end;

create trigger TRG_STU_INSERT
instead of insert on STUDENT
for each row
when new.Credits is not null
begin
   insert into _PERSON
      select new.PersID,new.Name
      where new.PersID not in (select PersID from _PERSON) $;$
   --
   insert into _STUDENT
      values (new.PersID,new.Credits) $;$
end;


-- ----------------------------------------- --
-- THE UPDATE TRIGGERS                       --
-- ----------------------------------------- --

create trigger TRG_PER_UPDATE1
instead of update of PersID,Name on PERSON
for each row
begin
   update _PERSON
   set   PersID = new.PersID
   where new.PersID <> old.PersID $;$
   --
   update _PERSON
   set   Name = new.Name
   where PersID = new.PersID
   and   new.Name <> old.Name $;$
end;

create trigger TRG_PER_UPDATE2
instead of update of Specialty on PERSON
for each row
begin
   -- (Needless to check the existence of the row;)
   -- (works whether it exists or not).
   update _PROFESSOR
   set   Specialty = new.Specialty
   where PersID = new.PersID
   and   new.Specialty is not null $;$
   --
   insert into _PROFESSOR 
      select new.PersID,new.Specialty
      where  new.PersID not in (select PersID from _PROFESSOR) $;$
   --
   delete from _PROFESSOR 
   where  PersID = new.PersID
   and    new.Specialty is null $;$
end;

create trigger TRG_PER_UPDATE3
instead of update of Credits on PERSON
for each row
begin
   -- (Needless to check the existence of the row;)
   -- (works whether it exists or not).
   update _STUDENT
   set   Credits = new.Credits
   where PersID = new.PersID
   and   new.Credits is not null $;$
   --
   insert into _STUDENT 
      select new.PersID,new.Credits
      where  new.PersID not in (select PersID from _STUDENT) $;$
   --
   delete from _STUDENT 
   where  PersID = new.PersID
   and    new.Credits is null $;$
end;

create trigger TRG_PRO_UPDATE
instead of update of Specialty on PROFESSOR
for each row
begin
   update _PROFESSOR
   set   Specialty = new.Specialty
   where PersID = new.PersID
   and   new.Specialty is not null $;$
   --
   delete from _PROFESSOR 
   where  PersID = new.PersID
   and    new.Specialty is null $;$
end;

create trigger TRG_STU_UPDATE
instead of update of Credits on STUDENT
for each row
begin
   update _STUDENT
   set   Credits = new.Credits
   where PersID = new.PersID
   and   new.Credits is not null $;$
   --
   delete from _STUDENT 
   where  PersID = new.PersID
   and    new.Credits is null $;$
end;


-- --------------------------------- --;
-- THE DELETE TRIGGERS               --
-- --------------------------------- --;

create trigger TRG_PER_DELETE
instead of delete on PERSON
for each row
begin
   delete from _PERSON
   where  PersID = old.PersID $;$
end;

create trigger TRG_PRO_DELETE
instead of delete on PROFESSOR
for each row
begin
   delete from _PROFESSOR
   where  PersID = old.PersID $;$
end;

write;
write -- ------------------------------------------------------ --;
write -- Insert into technical tables                           --;
write -- ------------------------------------------------------ --;

write > insert into _PERSON values('P0123','Smith');;
write > insert into _PERSON values('P0234','Stonebraker');;
write > insert into _PERSON values('P0345','Ullman');;
write > insert into _PERSON values('P0456','Van Bolle');;
write > insert into _PROFESSOR  values('P0234','DB Theory');;
write > insert into _STUDENT    values('P0345',45);;
write > insert into _PROFESSOR  values('P0456','Advanced SQL');;
write > insert into _STUDENT    values('P0456',30);;
insert into _PERSON values('P0123','Smith');
insert into _PERSON values('P0234','Stonebraker');
insert into _PERSON values('P0345','Ullman');
insert into _PERSON values('P0456','Van Bolle');
insert into _PROFESSOR  values('P0234','DB Theory');
insert into _STUDENT    values('P0345',45);
insert into _PROFESSOR  values('P0456','Advanced SQL');
insert into _STUDENT    values('P0456',30);

write;
select '' as "PERSON   ",* from PERSON order by PersID;
select '' as "PROFESSOR",* from PROFESSOR order by PersID;
select '' as "STUDENT  ",* from STUDENT order by PersID;

write-b <abortDB>;
abortDB;

write;
write -- ------------------------------------------------------ --;
write -- Insert data into subtables                             --;
write -- ------------------------------------------------------ --;
write > insert into PERSON(PersID,Name) values ('P0123','Smith');;
write > insert into PROFESSOR values ('P0234','Stonebraker','DB Theory');;
write > insert into PROFESSOR values ('P0456','Van Bolle','Advanced SQL');;
write > insert into STUDENT   values ('P0345','Ullman',45);;
write > insert into STUDENT   values ('P0456','Van Bolle',30);;
insert into PERSON(PersID,Name) values ('P0123','Smith');
insert into PROFESSOR values ('P0234','Stonebraker','DB Theory');
insert into PROFESSOR values ('P0456','Van Bolle','Advanced SQL');
insert into STUDENT   values ('P0345','Ullman',45);
insert into STUDENT   values ('P0456','Van Bolle',30);

write;
select '' as "PERSON   ",* from PERSON order by PersID;
select '' as "PROFESSOR",* from PROFESSOR order by PersID;
select '' as "STUDENT  ",* from STUDENT order by PersID;

write-b <abortDB>;
abortDB;

write;
write -- ------------------------------------------------------ --;
write -- Insert data into supertable                            --;
write -- ------------------------------------------------------ --;
write > insert into PERSON(PersID,Name) values ('P0123','Smith');;
write > insert into PERSON(PersID,Name,Specialty) values ('P0234','Stonebraker','DB Theory');;
write > insert into PERSON(PersID,Name,Credits) values ('P0345','Ullman',45);;
write > insert into PERSON(PersID,Name,Specialty,Credits) values ('P0456','Van Bolle','Advanced SQL',30);;
insert into PERSON(PersID,Name) values ('P0123','Smith');
insert into PERSON(PersID,Name,Specialty) values ('P0234','Stonebraker','DB Theory');
insert into PERSON(PersID,Name,Credits) values ('P0345','Ullman',45);
insert into PERSON(PersID,Name,Specialty,Credits) values ('P0456','Van Bolle','Advanced SQL',30);

write;
select '' as "PERSON   ",* from PERSON order by PersID;
select '' as "PROFESSOR",* from PROFESSOR order by PersID;
select '' as "STUDENT  ",* from STUDENT order by PersID;

write-b <commitDB>;
commitDB;

write;
write -- ------------------------------------------------------ --;
write -- Update                                                 --;
write -- ------------------------------------------------------ --;

write > update PERSON set Name = 'Branson' where PersID = 'P0123';;
write > update PROFESSOR set Specialty = 'DB Theory and Practice' where PersID = 'P0234';;
write > update PERSON set Specialty = 'Business models' where PersID = 'P0123';;
write > update PERSON set Credits = null where PersID = 'P0456';;

update PERSON set Name = 'Branson' where PersID = 'P0123';
update PROFESSOR set Specialty = 'DB Theory and Practice' where PersID = 'P0234';
update PERSON set Specialty = 'Business models' where PersID = 'P0123';
update PERSON set Credits = null where PersID = 'P0456';

write;
select '' as "PERSON   ",* from PERSON order by PersID;
select '' as "PROFESSOR",* from PROFESSOR order by PersID;
select '' as "STUDENT  ",* from STUDENT order by PersID;

write-b <abortDB>;
abortDB;

write;
write -- ------------------------------------------------------ --;
write -- Delete                                                 --;
write -- ------------------------------------------------------ --;

write > delete from PERSON where PersID = 'P0345';;
write > delete from PROFESSOR where PersID = 'P0234';;

delete from PERSON where PersID = 'P0345';
delete from PROFESSOR where PersID = 'P0234';

write;
select '' as "PERSON   ",* from PERSON order by PersID;
select '' as "PROFESSOR",* from PROFESSOR order by PersID;
select '' as "STUDENT  ",* from STUDENT order by PersID;

closeDB;

write-ab End of script;
return;

