-- ----------------------- --
-- Create a block          --
-- ----------------------- --

openDB BLOCK_CHAIN.db;

extract blockSize = select count(*) from BTRANSACTION where Bhash is null;
if ($blockSize$ = 0);
   write There is no pending transactions. Operation cancelled.;
   goto END;
endif;

extract NB = select count(*) from BLOCK;
if ($NB$ = 0) set previous = ;
if ($NB$ > 0) extract previous = select Bhash from BLOCK order by RecTime desc;

extract transList =
        select group_concat2(case Operation
                                when 'REGISTER'
                                  then TransID||';REGISTER;'||Source||';'
                                       ||RecTime||';'||Signature
                                when 'DEPOSIT'
                                  then TransID||';DEPOSIT;'||Recip||';'
                                       ||cast(Amount as char)||';'||RecTime||';'||Signature
                                when 'TRANSFER'
                                  then TransID||';TRANSFER;'||Source||';'||Recip||';'
                                       ||cast(Amount as char)||';'||RecTime||';'||Signature
                                else 'Transaction type unknown'
                             end,0,TransKey,1,'#')
                    from   BTRANSACTION
                    where  Bhash is null;

set recTime = $dateclock$;

set blockBody = $transList$#$recTime$#$previous$;

set challenge = 20;
set format = hexa;
set hashTrials = 5000000;
compute hashPrefix = repeat('0',$challenge$);

write-b Be patient. This "mining" operation may take "some time"!;

start-timer;
function hash,nonce,nHash = LStr:miningHash $blockBody$,$hashPrefix$,$hashTrials$,$format$;
write-b Mining time = $timer$ for $nHash$ hash computations;

if ('$nonce$' = '' and $nHash$ = 0) write An error occurred.;
if ('$nonce$' = '' and $nHash$ > 0) write No legal hash has been found. Augment the number of trials.;

write challenge = $challenge$;
write format    = $format$;
write Bhash     = $hash$;
write Transact  = $blockSize$;
write Nonce     = $nonce$;
write RecTime   = $recTime$;
write Status    = 0;
write Bprev     = $previous$;

insert into BLOCK(Bhash,Transact,Nonce,RecTime,Bprev)
       values('$hash$',$blockSize$,'$nonce$','$recTime$',
              case when '$previous$' = '' then null else '$previous$' end);

update BTRANSACTION
set   Bhash = '$hash$'
where Bhash is null;

commitDB;

write-b Block created with $blockSize$ transaction(s).;

label END;
   write;
   closeDB;

   return;
