-- ----------------------- --
-- Validate a transaction  --
-- ----------------------- --

openDB BLOCK_CHAIN.db;

write;
-- Display a summary of all the transactions.
select TransKey,Source,Operation,Recip,Amount,RecTime
from   BTRANSACTION;
select TransKey,Source,Operation,Recip,Amount,RecTime
from   BTRANSACTION_PRIV;

set TentryWidth2 = $entryWidth2$;
set entryWidth2  = 46;

ask transKey = [/bSelect a transaction to validate]
               Transaction:[!select RecTime
                                  ||' - '||Operation||', '
                                  ||case Operation
                                      when 'REGISTER' then Source
                                      when 'DEPOSIT'  then Recip||': '||Amount
                                      when 'TRANSFER' then Source||' to '||Recip||': '||Amount
                                    end,
                                    TransKey
                             from   BTRANSACTION_PRIV order by TransKey];
set entryWidth2 = $TentryWidth2$;
if ('$DIALOGbutton$' = 'Cancel' or '$transKey$' = '') goto END;

extract transID,operation,source,recip,amount,recTime,signature
        = select TransID,Operation,Source,Recip,Amount,RecTime,Signature
          from   BTRANSACTION where TransKey = '$transKey$';

write-b Validation of $operation$ transaction: [$transKey$] [$transID$];

if ('$operation$' = 'REGISTER') goto REGISTER; 
if ('$operation$' = 'DEPOSIT')  goto DEPOSIT; 
if ('$operation$' = 'TRANSFER') goto TRANSFER; 
goto END;


label REGISTER;
--------------
   -- Account "source" exists in DIRECTORY
   -- The "signature" matches the components of the transaction

   extract N,publicKey = select count(*),max(PublicKey) from DIRECTORY
                         where  AccountID = '$source$';

   -- Does account "source" exist in DIRECTORY?
   if ($N$ = 0);
      write @S3- Account "$source$" has not yet been registered.;
   else;
      write @S3- write Account "$source$" has been registered.;
   endif;

   -- Decryption of "signature" through DIRECTORY.PublicKey 
   function trans = LStr:decryptRSA {$signature$},{$publicKey$};
   if ('$transID$;$operation$;$source$;$recTime$' = '$trans$');
      write @S3- The signature of the transaction matches its components.;
      write @S6- Decrypted signature: $trans$;
   else;
      write @S3- This transaction is corrupted: its signature does not match its components.;
      write @S6- Components:          $transID$;$operation$;$bcAddress$;$recTime$;
      write @S6- Decrypted signature: $trans$;
   endif;
   goto END;

label DEPOSIT;
-------------
   -- Account "recip" exists in DIRECTORY
   -- The "signature" matches the components of the transaction

   extract N,publicKey = select count(*),max(PublicKey) from DIRECTORY
                         where  AccountID = '$recip$';
   if ($N$ = 0);
      write @S3- Account "$recip$" has not been registered.;
   else;
      write @S3- Account "$recip$" has been registered.;
   endif;

   function trans = LStr:decryptRSA {$signature$},{$publicKey$};
   if ('$transID$;$operation$;$recip$;$amount$;$recTime$' = '$trans$');
      write @S3- The signature of the transaction matches its components.;
      write @S6- Decrypted signature: $trans$;
   else;
      write @S3- This transaction is corrupted: its signature does not match its components.;
      write @S6- Components:          $transID$;$operation$;$recip$;$amount$;$recTime$;
      write @S6- Decrypted signature: $trans$;
   endif;
   goto END;

label TRANSFER;
--------------
   -- Account "source" exists in DIRECTORY
   -- Account "recip" exists in DIRECTORY
   -- The account balance of "source" is >= "amount"
   -- The "signature" matches the components of the transaction

   extract N1,publicKey = select count(*),max(PublicKey) from DIRECTORY
                          where  AccountID = '$source$';
   extract N2 = select count(*) from DIRECTORY
                where  AccountID = '$recip$';
   if ($N1$ = 0);
      write @S3- Source account "$source$" has not yet been registered.;
   else;
      write @S3- Source account "$source$" has been registered.;
   endif;

   if ($N2$ = 0);
      write @S3- Recipient account "$recip$" has not yet been registered.;
   else;
      write @S3- Recipient account "$recip$" has been registered.;
   endif;

   extract plusD = select coalesce(sum(Amount),0) from BTRANSACTION
                   where  operation = 'DEPOSIT'  and Recip = '$source$'  and RecTime < '$recTime$';
   extract plusT = select coalesce(sum(Amount),0) from BTRANSACTION
                   where  operation = 'TRANSFER' and Recip = '$source$'  and RecTime < '$recTime$';
   extract minus = select coalesce(sum(Amount),0) from BTRANSACTION
                   where  operation = 'TRANSFER' and Source = '$source$' and RecTime < '$recTime$';
   compute balance = $plusD$ + $plusT$ - $minus$;

   if ($balance$ < $amount$);
      write @S3- Insufficient balance ($balance$). The operation cannot be executed.;
   else;
      write @S3- The balance of the source account is OK ($balance$). The operation can be executed.;
   endif;

   function trans = LStr:decryptRSA {$signature$},{$publicKey$};
   if ('$transID$;$operation$;$source$;$recip$;$amount$;$recTime$' = '$trans$');
      write @S3- The signature of the transaction matches its components.;
      write @S6- Decrypted signature: $trans$;
   else;
      write @S3- This transaction is corrupt: its signature does not match its components.;
      write @S6- Components:          $transID$;$operation$;$recip$;$amount$;$recTime$;
      write @S6- Decrypted signature: $trans$;
   endif;
   goto END;

label END;
   write;
   closeDB;

   return;

