
-- Common query patterns --
   -- List of instance names --
   set listInstances
       = select D.DirNick||' ('||I.InstDate
                ||' '||I.InstTime||')' as Name,
                I.InstID
         from   DIRECTORY D,INSTANCE I
         where  D.DirID = I.DirID
         order by Name;

   -- List of file types --
   set listTypes
       = select Ext
         from   (select 'all files' as Ext
                 union select 'all directories' as Ext
                 union select 'all files and directories' as Ext
                    union
                 select distinct '*.'||lower(Extension) as Ext
                 from   NODE
                 where  Extension <> '')
         order by Ext;

   -- Getting instance name of $inst$ --
   set instName
       = select DirNick||' ('||InstDate||' '||InstTime||')'
         from   DIRECTORY D, INSTANCE I
         where  InstID = $inst$ and D.DirID = I.DirID;

   -- Set of the Hash values of clones in instance $inst$ --
   set cloneHash = select Hash from NODE
                   where  InstID = $inst$ and NType = 'F'
                   and    Hash is not null and Size > 0
                   group by Hash having count(*) > 1;

   -- Set of the Hash values of clones in instances $inst1$ AND $inst2$ --
   set cloneHash2 = select Hash
                    from   (select Hash from NODE
                            where  InstID = $inst1$ and NType = 'F'
                            and    Hash is not null and Size > 0
                               intersect
                            select Hash from NODE
                            where  InstID = $inst2$ and NType = 'F'
                            and    Hash is not null and Size > 0);

   -- Condensed version of a Hash value --
   set hashDigest = substr(Hash,1,8)||'...'||substr(Hash,56,8);

