-- ---------------------------- --
-- GENERAL STATISTICS           --
-- ---------------------------- --

if (not fileExists('dbDirectory/DIRECTORY.db'));
   showMessage Database "DIRECTORY.db" not found;
   return;
endif;

openDB DIRECTORY.db;

write-b -- GLOBAL STATISTICS --;
extract ND = select count(*) from DIRECTORY;
extract NI = select count(*) from INSTANCE;
extract NN = select count(*) from NODE;
extract Nd = select count(*) from NODE where NType = 'D';

select $ND$        as "Root directories",
       $NI$        as Instances,
       $NN$        as Nodes,
       $Nd$        as Directories,
       $NN$ - $Nd$ as Files;

write-b -- DIRECTORIES --;
select D.DirNick                    as "Root directory",
       count(*)                     as Instances,
       min(InstDate||' '||InstTime) as "First recording",
       max(InstDate||' '||InstTime) as "Last recording"
from DIRECTORY D, INSTANCE I
where D.DirID = I.DirID
group by D.DirID;

write-b -- DIRECTORIES AND INSTANCES --;
select DirNick                  as "Root directory",
       Time                     as "Instance time",
       frame(Folders,7,'>',' ') as Directories,
       frame(Files,7,'>',' ')   as " Files",
       frame(thousandSep(Size,' '),15,'>',' ') as "Total size",
       Last                     as "Last updated"
from
   (select DirNick,
           '' as Time,
           '' as Folders,
           '' as Files,
           '' as Size,
           '' as Last,
           cast(DirID as char)||'D' as Sort
    from DIRECTORY D
      union
    select '' as DirNick,
           InstDate||' '||InstTime as Time,
           (select count(*) from NODE N
            where  NType = 'D' and  N.InstID = I.InstID) as Folders,
           (select count(*) from NODE N
            where  NType = 'F' and N.InstID = I.InstID) as Files,
           (select sum(case NType when 'F' then Size else 0 end) from NODE N
            where  N.InstID = I.InstID) as Size,
           (select max(LastUpdate) from NODE N
            where  NType = 'F' and N.InstID = I.InstID) as Last,
           cast(DirID as char)||'I' as Sort
    from INSTANCE I)
order by Sort; 

closeDB;

return;