set sourceDir = $scriptDirectory$/Case-Studies/Case_Game_Book;
set maxSelectWidth = 80;
set entryWidth1 = 20;
set entryWidth2 = 17;

set game = WatchTower;


-- Max length of elementary circuits --
set circLength = 10;
-- Max length of runs --
set runLength = 10;
-- Number of circuit instances --
set multiple = 0;

-- Displaying the graph? --
set displayGraph = 1;
-- Computing the cyclic kernel? --
set cyclicKernel = 1;
-- Number of runs to display --
set displayRuns = 40;

askCombo [/b@S8GAME ANALYSIS]
         [ask-u game,circLength,runLength,multiple = 
                    [/bGAME and COMPUTING PARAMETERS]
                     Select a game:[(Simple-1,Simple-2,WatchTower,WindHammer)]
                    |Max length of elementary circuits:
                    |Max length of runs:
                    |Number of circuit instances:[(0,1,2,3)]]
        |[ask-u displayGraph,cyclicKernel,displayRuns =
                    [/bDISPLAY PARAMETERS]
                      Displaying the graph?[(0,1)]
                     |Display the cyclic kernel?[(0,1)]
                     |Number of runs to display: ];
if ('$DIALOGbutton$' = 'Cancel') return;

if ('$game$' in ('Simple-1','Simple-2','WatchTower')); 
   set start = 01;
   set win   = 99;
   set fail  = 00;
   set idlen = 4;
endif;
if ('$game$' in ('WindHammer'));
   set start = 001;
   set win   = 999;
   set fail  = 000;
   set idlen = 5;
endif;

-- --------------- --
-- Create database --
-- --------------- --

createOrReplaceDB GAMEBOOK.db;

create table EPISODE(
   EPID     varchar(3) not null primary key);

create table BRANCH(
   EPID     varchar(3) not null,
   SEQNBR   integer,
   TARGET   varchar(3) not null,
   foreign  key (EPID)   references EPISODE on delete cascade,
   foreign  key (TARGET) references EPISODE on delete cascade);

create temp table RUN(
   EXIT     varchar(3),
   Length   integer,
   Path     varchar(2048));

create temp table PATH(
   START    varchar(3),
   END      varchar(3),
   Length   integer,
   Path     varchar(2048));

create temp table LAST(
   START    varchar(3),
   END      varchar(3),
   Length   integer,
   Path     varchar(2048));

create temp table RBRANCH(
   EPID     varchar(3) not null,
   TARGET   varchar(3) not null);

commitDB;

-- ---------------------- --
-- Select and load a game --
-- ---------------------- --

execSQL $sourceDir$/_GraphData_$game$.sql;


write-b;
write @s   +--------------------------------------------------+;
write  ----+                THE SOURCE GRAPH                  +----;
write @s   +--------------------------------------------------+;
write;

write-a Source game name: "$game$";

select (select count(*) from EPISODE) as Episodes,
       (select count(*) from BRANCH) as Branches;
write;

if ($displayGraph$ = 1);
   select group_concat('('||EPID||','||TARGET||')') as Branches
   from BRANCH group by EPID;

   write-b Targets by source episode:;
   select EPID as "Source", group_concat2(TARGET,0,TARGET,1,',') as "Targets"
   from   BRANCH where EPID not in ('$fail$','$win$') group by EPID;

   write-b Sources by target episode:;
   select group_concat2(EPID,0,EPID,1,',') as "Sources", TARGET as "Target" 
   from   BRANCH where EPID not in ('$fail$','$win$') group by TARGET;
endif;

write-b;
write @s   +-------------------------------------------------+;
write  ----+              EPISODE ANALYSIS                   +----;
write @s   +-------------------------------------------------+;

write-b (Invalid) isolated episodes:;
select EPID from EPISODE
where  EPID not in (select EPID   from BRANCH)
and    EPID not in (select TARGET from BRANCH);

write-b (Invalid) reflexive branches:;
select EPID,TARGET from BRANCH where EPID = TARGET;

write-b Episodes without predecessor (only "entry" allowed):;
select EPID from EPISODE
where  EPID not in (select TARGET from BRANCH);

write-b Episodes without successor (only "exit" allowed):;
select EPID from EPISODE
where  EPID not in (select EPID from BRANCH);

write-b Ending episodes (leading to exit):;
select EPID||' (success)' as "Ending" from BRANCH where TARGET = '$win$'
  union
select EPID||' (failure)' as "Ending" from BRANCH where TARGET = '$fail$';

write-b Near-ending episodes:;
select group_concat2(B1.EPID,0,B1.EPID,1,',') as "Ending - 1",B2.EPID as "Ending"
from   BRANCH B1, BRANCH B2
where  B1.TARGET = B2.EPID
and    B2.TARGET  in ('$fail$','$win$')
group by "Ending";

write-b List of near-near-ending episodes:;
select group_concat2(B1.EPID,0,B1.EPID,1,',') as "Ending - 2",
       B2.EPID as "Ending - 1",B3.EPID as "Ending"
from   BRANCH B1, BRANCH B2, BRANCH B3
where  B1.TARGET = B2.EPID
and    B2.TARGET = B3.EPID
and    B3.TARGET  in ('$fail$','$win$')
group by "Ending - 1";

write-b;
write @s   +--------------------------------------------------+;
write  ----+             MERGING one-to-one BRANCHES          +----;
write @s   +--------------------------------------------------+;

write-b Couples of successive episodes that can be merged:;
select EPID||' + '||TARGET as Suggestion
from   BRANCH B
where  TARGET not in ('$fail$','99')
and    (select count(*) from BRANCH where  EPID = B.EPID) = 1
and    (select count(*) from BRANCH where  TARGET = B.TARGET) = 1
and    (TARGET,EPID) not in (select EPID,TARGET from BRANCH);


write-b;
write @s   +--------------------------------------------------+;
write  ----+                  CIRCUIT ANALYSIS                +----;
write @s   +--------------------------------------------------+;

if ($cyclicKernel$ = 1); 
   write-b -- ----------------- --;
   write   -- The cyclic kernel --;
   write   -- ----------------- --;

   write-b -- Computing the cyclic kernel --;
   while (True);
      extract Nin = select count(*) from EPISODE 
                    where  EPID not in (select TARGET from BRANCH);
      extract Nex = select count(*) from EPISODE 
                    where  EPID not in (select EPID from BRANCH);
      write @s  Roots: $Nin$ - Leaves: $Nex$;
      if ($Nin$ = 0 and $Nex$ = 0) exit;
      delete from EPISODE where  EPID not in (select TARGET from BRANCH);
      delete from EPISODE where  EPID not in (select EPID from BRANCH);
   endwhile;

   extract Nb = select count(*) from BRANCH;

   if ($Nb$ = 0);
      write-b -- This game is acyclic --;
   else;
      write-b -- Contents of the cyclic kernel --;
      select (select count(*) from EPISODE) as Episodes, $Nb$ as Branches;

      write-b -- Episodes of the cyclic kernel --;
      select EPID   as "Episodes" from BRANCH
         union
      select TARGET as "Episodes" from BRANCH
      order by 1;

      write-b -- Branches of the cyclic kernel --;
      select group_concat('('||EPID||','||TARGET||')') as "Branches"
      from BRANCH group by EPID;
   endif;

   -- restore database contents --
   delete from EPISODE;
   execSQL $sourceDir$/_GraphData_$game$.sql;
endif;


write-b -- ----------------------- --;
write   -- The elementary circuits --;
write   -- ----------------------- --;

create temp table CIRCUIT(Circuit varchar(1024),Length integer,EpisodeSet varchar(1024));

write-b -- Normalized elementary circuits of up to $circLength$ branches--;
start-timer;
with recursive PATHS(EPID,Target,Length,Path) as
  (select EPID,TARGET,1,'.'||EPID||'.'||TARGET||'.' from BRANCH
      union
   select P.EPID,B.TARGET,P.Length+1,P.Path||B.TARGET||'.'
   from   PATHS P, BRANCH B
   where  P.TARGET = B.EPID
   and    P.EPID <> P.TARGET
   and    P.Path not like '.%.'||B.TARGET||'.%'
   and    P.Length < $circLength$
   )
insert into CIRCUIT
select min(Path),Length,
       itemSort(substr(trim(Path,'.'),$idlen$,999),0,0,'.') as EpSet
from   PATHS
where  EPID = Target group by EpSet;
set circTime = $timer$;

select * from CIRCUIT order by Length,Circuit;

write-b -- Number of circuits by size --;
select Length as Size,count(*) as Number from CIRCUIT group by Size order by Size;

write-b -- Computing time: $circTime$;


write-b;
write @s   +--------------------------------------------------+;
write  ----+                  GAME RUN ANALYSIS               +----;
write @s   +--------------------------------------------------+;

write-b -- Game runs of up to $runLength$ branches --;

-- No circuit, linear 
if ($multiple$ = 0) set <cyclic> = P.Path not like '%.'||B.TARGET||'.%';
-- Allowing one run of each circuit
if ($multiple$ = 1) set <cyclic> = P.Path not like '%.'||B.TARGET||'.%.'||B.TARGET||'.%';
-- Allowing one run of each circuit
if ($multiple$ = 2) set <cyclic> = P.Path not like '%.'||B.TARGET||'.%.'||B.TARGET||'.%.'||B.TARGET||'.%';
-- Allowing one run of each circuit
if ($multiple$ = 3) set <cyclic> = P.Path not like '%.'||B.TARGET||'.%.'||B.TARGET||'.%.'||B.TARGET||'.%.'||B.TARGET||'.%';

start-timer;
with recursive PATHS(EPID,TARGET,Length,Path) as
  (select EPID,TARGET,1,'.'||EPID||'.'||TARGET||'.'
   from   BRANCH where EPID = '$start$'
      union
   select P.EPID,B.TARGET,P.Length+1,P.Path||B.TARGET||'.'
   from   PATHS P, BRANCH B
   where  P.TARGET = B.EPID
   and    P.EPID <> P.TARGET
   and    $<cyclic>$
   and    P.Length < $runLength$
   )
insert into RUN select TARGET,Length,Path from PATHS where Target in ('$fail$','99');
set runTime = $timer$;
write -- Time: $runTime$;

if ($displayRuns$ > 0);
   write-b -- List of first $displayRuns$ runs of up to $runLength$ branches --;
   select EXIT,Length,Path from RUN order by Length,EXIT #[1,$displayRuns$];
endif;

write-b -- Number of "Win" and "Fail" runs of up to $runLength$ branches
           and multiple circuits = $multiple$ --;
select (select count(*) from RUN where EXIT = '$win$')  as "# Win",
       (select count(*) from RUN where EXIT = '$fail$') as "# Fail",
       (select count(*) from RUN) as "# Total";


write-b;
write @s   +--------------------------------------------------+;
write  ----+                  REACHABILITY                    +----;
write @s   +--------------------------------------------------+;


write-b -- ---------------------------------------- --;
write -- Searching for the non reachable episodes --;
write-a -- ---------------------------------------- --;


write-ab -- First try --;
select ''''||EPID||'''' as "Episodes without predecessors"
from   EPISODE 
where  EPID not in (select TARGET from BRANCH);
 
write-ab -- Full procedure --;

insert into PATH 
select EPID,TARGET,1,'.'||EPID||'.'||TARGET||'.' from BRANCH where EPID = '$start$';

extract NB = select count(*) from BRANCH;

insert into RBRANCH select EPID,TARGET from BRANCH where EPID <> '$start$';

start-timer;
for length = [1,$NB$];
   -- Create last paths --
   insert into LAST
   select P.START, B.TARGET, $length$ + 1, P.Path||B.TARGET||'.'
   from   PATH P, RBRANCH B
   where  P.END = B.EPID
   and    P.Length = $length$
   and    P.Path not like '%.'||B.TARGET||'.%';

   -- If no last paths, exit --
   extract N = select count(*) from LAST;
   if ($N$ = 0) exit;

   -- Move last paths into PATH --
   insert into PATH select * from LAST;
   delete from LAST;

   -- Delete useless branches --
   delete from RBRANCH where TARGET in (select END from PATH);
endfor;
set compTime = $timer$;

select count(*) as "# Remaining paths" from PATH;

-- Reachable Episodes --;
select count(distinct END) as "# Reachable Episodes" from PATH;

-- Non reachable Episodes --;
select ''''||NEP||'''' as "Non Reachable Episodes"
from (select EPID as NEP from EPISODE
         except
      select END as NEP from PATH)
order by NEP;

write-ab Computing time = $compTime$ s.;


write-b -- ---------------------------------------- --;
write   -- Searching for non contributing episodes  --;
write-a -- ---------------------------------------- --;

delete from PATH;

insert into PATH 
select TARGET,EPID,1,'.'||TARGET||'.'||EPID||'.' from BRANCH where TARGET in ('$fail$','$win$');

extract NB = select count(*) from BRANCH;

insert into RBRANCH select TARGET,EPID from BRANCH where TARGET not in ('$fail$','$win$');

start-timer;
for length = [1,$NB$];
   -- Create last paths --
   insert into LAST
   select P.START, B.TARGET, $length$ + 1, P.Path||B.TARGET||'.'
   from   PATH P, RBRANCH B
   where  P.END = B.EPID
   and    P.Length = $length$
   and    P.Path not like '%.'||B.TARGET||'.%';

   -- If no last paths, exit --
   extract N = select count(*) from LAST;
   if ($N$ = 0) exit;

   -- Move last paths into PATH --
   insert into PATH select * from LAST;
   delete from LAST;

   -- Delete useless branches --
   delete from RBRANCH where TARGET in (select END from PATH);
endfor;
set compTime = $timer$;

select count(*) as "# Remaining paths" from PATH;

-- Non contributing Episodes --;
select count(distinct END) as "# Contributing Episodes" from PATH;

-- Non Contributing Episodes --;
select ''''||NEP||'''' as "Non Contributing Episodes"
from (select EPID as NEP from EPISODE
         except
      select END as NEP from PATH)
order by NEP;

write-ab Computing time = $compTime$ s.;

closeDB;

set maxSelectWidth = 60;

return;
