-- Various scripts for printing the indented descend of a member --

-- Due to a bug in SQLite3 module, cursor attribute "description",
-- that is to provide the list of column names of a result-set, is set to null
-- for a "with-select" that returns an empty result-set.
-- As a consequence, the SQLfast engine is unable to format this (empty) result-set 
-- in tabular layout. Not critical but annoying!
-- In such a case, a (long) warning message is printing on the output channel
-- to explain the "error".

--There are several work-arounds that are aplied below: 
-- - Avoid empty result sets by adding "something"
--   in the initialization query.
-- - Use a loop
-- - Use a "view" (recommended)

execSQL UTIL-SELECT-parameters-for-Lists.sql;

openDB KINGS.db;

onError continue;
set maxSelectWidth = 120;

ask pid = Member:[!select PiD from MEMBER order by PiD];
   if ('$pid$' = '' or '$DIALOGbutton$' = 'Cancel') return;

extract Nam,Tit = select Name,Title 
                  from   MEMBER where PiD = '$pid$';

write-b Descend of $Nam$, $Tit$;

write-b -- ------------------------ --;
write   -- Standard recursive query --;
write-a -- ------------------------ --;

-- Beware empty resultsets ! --

extract Ndesc = select count(*) from MEMBER where Father = '$pid$';
if ($Ndesc$ > 0);
   with recursive TREE(Ind,Father,PID,Path) as
   (select '','$pid$',PiD,'.'||'$pid$'||'.'||PiD||'.'
    from   MEMBER where Father = '$pid$'
      union
    select T.Ind||'  ',T.PiD,M.PiD,T.Path||M.PiD||'.'
    from   TREE T, MEMBER M
    where  T.PiD = M.Father
   )
   select Ind||'- '||M.Name||', '||M.Title as "Member (+ root)"
   from   TREE T, MEMBER M
   where  T.PiD = M.PiD
   order by T.Path;
else;
   -- EMPTY RESULTSET --
endif;

write-b -- --------------------------------------- --;
write   -- Loop-based algorithm on recursive CTE   --;
write-a -- --------------------------------------- --;

for txt = [
     with recursive TREE(Ind,Father,PID,Path) as
     (select '','$pid$',PiD,'.'||'$pid$'||'.'||PiD||'.'
      from   MEMBER where Father = '$pid$'
        union
      select T.Ind||'  ',T.PiD,M.PiD,T.Path||M.PiD||'.'
      from   TREE T, MEMBER M
      where  T.PiD = M.Father
     )
     select Ind||'- '||M.Name||', '||M.Title as ""
     from   TREE T, MEMBER M
     where  T.PiD = M.PiD 
     order by T.Path];
   write @s$txt$;
endfor;

write-b -- ----------------------- --;
write   -- View-based algorithm    --;
write-a -- ----------------------- --;

create temporary view HIERARCHY(Line,Path) as
with recursive TREE(Ind,Father,PiD,Path) as
(
 -- Initialized with a constant row to avoid empty CTE --
 select '',null,'$pid$','.'||'$pid$'||'.'
    union
 select T.Ind||'  ',T.PiD,M.PiD,T.Path||M.PiD||'.'
 from   TREE T, MEMBER M
 where  T.PiD = M.Father
)
select T.Ind||'- '||M.Name||', '||M.Title,T.Path
from   TREE T,MEMBER M
where  T.PiD = M.PiD
-- Now, the constant row is discarded --
and    T.Father is not null;

select Line from HIERARCHY order by Path;

closeDB;

onError stop;

execSQL UTIL-SELECT-parameters-standard-1.sql
