-- -------------------------------------------- --
-- Draw the KING hierarchy (version kings only) --
-- Not the most elegant one but sufficient
-- to create Figure 36.12!
-- -------------------------------------------- --

openDB KINGS.db;

-- --- kings only ---------------------

create temporary table MEMBER0(
   PiD            char(6),
   Name           char(20),
   House          char(12),
   Father         char(6),
   Title          char(20),
   DateBirth      integer,
   DateDeath      integer,
   DateReignBegin integer,
   DateReignEnd   integer);

set Clos = select PiD,PiD,'.'||PiD||'.' from MEMBER
                 union all
           select P.Ancestor, M.PiD, P.Path||M.PiD||'.'
           from   PATHS0 P,MEMBER M
           where  P.Descendant = M.Father;

with recursive PATHS0(Ancestor,Descendant,Path) as ($Clos$)
insert into MEMBER0(Father,Name,Title,PiD,House,
                     DateBirth,DateDeath,DateReignBegin,DateReignEnd) 
select P.Ancestor,K3.Name,K3.Title,P.Descendant,K3.House,
       K3.DateBirth,K3.DateDeath,K3.DateReignBegin,K3.DateReignEnd
from   MEMBER K1, PATHS0 P, MEMBER K3
where  P.Ancestor <> P.Descendant
and    P.Ancestor = K1.PiD and P.Descendant = K3.PiD
and    K1.Title = 'Roi de France'
and    K3.Title = 'Roi de France'
and not exists (select 1 from KING K3
                where  P.Path like '_%.'||K3.PiD||'.%_')
order  by P.Ancestor,P.Descendant;

insert into MEMBER0 select * from MEMBER where Father is null;

--select * from MEMBER0 order by PID;

-- -------------------------------------------------

parameter entryWidth = 40;
set pid = C00;
ask-u pid = King:[!select PID||'-'||Name||','||Title,PID
                   from   MEMBER0 order by PID];
   if ('$DIALOGbutton$' = 'Cancel') return;

extract Nam,Tit = select Name,Title 
                  from   MEMBER0 where PID = '$pid$';

set Closure = select 0,null,'$pid$','.'||'$pid$'||'.'
                 union all
              select P.Len+1,P.PID,M.PID,P.Path||M.PID||'.'
              from   PATHS P,MEMBER0 M
              where  P.PID = M.Father;

-- create table TREE --
-- ----------------- --
create temporary table TREE (
       Seq integer primary key autoincrement,
       Len integer,Father char(6),
       PID char(6),Path varchar(1024),
       X integer,Y integer,FX integer,FY integer);

with recursive PATHS(Len,Father,PiD,Path) as ($Closure$)
insert into TREE (Len,Father,PiD,Path,X)
select Len+1,Father,PiD,Path,(Len+1)*10
from   PATHS order by Path;
-- "order by Path" is necessary to generate monotonic "seq" values --

update TREE 
set Y = Seq*15 - 5,
    FX = (select T.X          from TREE T where T.PID = TREE.Father),
    FY = (select T.Seq*15 - 5 from TREE T where T.PID = TREE.Father);

--select * from TREE order by Path;

extract maxX,maxY = select max(X)+320,max(Y)+18 from TREE;

-- start generation --
-- ---------------- --
execSQL UTIL-SELECT-parameters-for-Lists.sql;
outputOpen draw.var;


-- generate SQLdraw script --
-- ------------------------ --
write area $maxX$,$maxY$;
select case when M.Title = 'Roi de France'
            then 'points " ",1,"black","blue",68@n'
            else 'points " ",1,"black","black",68@n'
       end
     ||cast(X as char)||','||cast(Y as char)||',"'
                      ||'['||M.PiD||'] '
                      ||M.Name||', '||M.Title
                      ||' ('||M.DateBirth||'-'||M.DateDeath||')'
                      ||case when M.Title = 'Roi de France'
                             then ' ('||M.DateReignBegin||'-'||M.DateReignEnd||')"@n'
                             else '"@n'
                        end
     ||case when M.Title = 'Roi de France'
            then 'points " ",1,"blue","black",31@n'
            else 'points " ",1,"black","black",31@n'
       end
     ||cast(X as char)||','||cast(Y as char)||',""'
     ||case when T.FX is null then '' 
            else '@n'
                ||'polyline " ",1,"black",""@n'
                ||cast(FX as char)||','||cast(FY as char)||',""@n'
                ||cast(FX as char)||','||cast(Y as char) ||',""@n'
                ||cast(X as char) ||','||cast(Y as char) ||',""'
       end 
from   TREE T,MEMBER0 M
where  T.PiD = M.PiD
order by Path;

-- store drawing in file KINGdrawing.draw --
-- -------------------------------------- --
outputOpen $fileDirectory$/KINGdrawing.draw;
write $draw$;
outputAppend console;

execSQL UTIL-SELECT-parameters-standard-1.sql;


-- display drawing --
-- --------------- --
showDrawing $fileDirectory$/KINGdrawing.draw
            = [/x500/y555];

closeDB;
