-- --------------------------- --
-- The Column-oriented model   --
-- --------------------------- --

write-b -- ---------------------------------- --;
write   -- COLUMN-ORIENTED version with PK Id --;
write   -- ---------------------------------- --;

openDB ORDERS.db;

create temp table CUST_NAME(
   CustID    char(10) not null primary key,
   Value     varchar(32) not null);
create temp table CUST_ADDRESS(
   CustID    char(10) not null primary key,
   Value     char(60) not null);
create temp table CUST_CITY(
   CustID    char(10) not null primary key,
   Value     char(30) not null);
create temp table CUST_CAT(
   CustID    char(10) not null primary key,
   Value     char(2));
create temp table CUST_ACCOUNT(
   CustID    char(10) not null primary key,
   Value     decimal(9,2) not null);

create temp table ORD_CUSTID(
   OrdID     char(12) not null primary key,
   Value     char(10) not null);
create temp table ORD_DATEORD(
   OrdID     char(10) not null primary key,
   Value     date not null);

create temp table DET_QORD(
   OrdID     char(10) not null,
   ProdID    char(15) not null,
   Value     decimal(9,2) not null,
   primary key (OrdID,ProdID));

create temp table PROD_DESCR(
   ProdID    char(15) not null,
   Value     char(60) not null,
   primary key (ProdID));

create temp table PROD_PRICE(
   ProdID    char(15) not null,
   Value     integer not null,
   primary key (ProdID));

create temp table PROD_QONHAND(
   ProdID    char(15) not null,
   Value     integer not null,
   primary key (ProdID));


insert into CUST_NAME(CustID,Value)    select CustID,Name from CUSTOMER;
insert into CUST_ADDRESS(CustID,Value) select CustID,Address from CUSTOMER;
insert into CUST_CITY(CustID,Value)    select CustID,City from CUSTOMER;
insert into CUST_CAT(CustID,Value)     select CustID,Cat from CUSTOMER;
insert into CUST_ACCOUNT(CustID,Value) select CustID,Account from CUSTOMER;

insert into ORD_CUSTID(OrdID,Value)    select OrdID,CustID from CUSTORDER;
insert into ORD_DATEORD(OrdID,Value)   select OrdID,DateOrd from CUSTORDER;

insert into DET_QORD(OrdID,ProdID,Value) select OrdID,ProdID,Qord from DETAIL;

insert into PROD_DESCR(ProdID,Value)   select ProdID,Description from PRODUCT;
insert into PROD_PRICE(ProdID,Value)   select ProdID,Price from PRODUCT;
insert into PROD_QONHAND(ProdID,Value) select ProdID,QonHand from PRODUCT;

write-b -- Table CUST_NAME --;
select * from CUST_NAME;
write-b -- Table CUST_ADDRESS --;
select * from CUST_ADDRESS;

write-b -- Table PROD_DESCR --;
select * from PROD_DESCR;

write-b -- Table ORD_CUSTID --;
select * from ORD_CUSTID;

write-b -- Table DET_QORD --;
select * from DET_QORD;

write-b -- Reconstruction of table CUSTOMER --;
select C1.CustID,C1.Value as Name,C2.Value as Address,
       C3.Value as City,C4.Value as Cat,C5.Value as Account
from CUST_NAME C1,CUST_ADDRESS C2,CUST_CITY C3,CUST_CAT C4,CUST_ACCOUNT C5
where C2.CustID = C1.CustID
and   C3.CustID = C1.CustID
and   C4.CustID = C1.CustID
and   C5.CustID = C1.CustID;

closeDB;

write;
write-b -- ---------------------------------------- --;
write   -- COLUMN-ORIENTED version with abstract Id --;
write   -- ---------------------------------------- --;

openDB ORDERS.db;

-- Table CUSTOMER --
-- -------------- --

create temp table CUST_CUSTID(
   Entity    integer      not null primary key autoincrement,
   Value     char(10)     not null unique
   -- The "unique" constraint translates the uniqueness property of PK --
   );
create temp table CUST_NAME(
   Entity    integer      not null primary key,
   Value     varchar(32)  not null);
create temp table CUST_ADDRESS(
   Entity    integer      not null primary key,
   Value     char(60)     not null);
create temp table CUST_CITY(
   Entity    integer      not null primary key,
   Value     char(30)     not null);
create temp table CUST_CAT(
   Entity    integer      not null primary key,
   Value     char(2));
create temp table CUST_ACCOUNT(
   Entity    integer      not null primary key,
   Value     decimal(9,2) not null);

-- First assign abstract Id to primary key values --
insert into CUST_CUSTID(Value) select CUSTID from CUSTOMER order by CUSTID;

insert into CUST_NAME(Entity,Value)
  select Entity,Name from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_ADDRESS(Entity,Value)
  select Entity,Address from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_CITY(Entity,Value)
  select Entity,City from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_CAT(Entity,Value)
  select Entity,Cat from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;
insert into CUST_ACCOUNT(Entity,Value)
  select Entity,Account from CUST_CUSTID I, CUSTOMER C where I.Value = C.CustID;

write-b -- CUSTOMER: Column-table CUST_CUSTID --;
  select * from CUST_CUSTID;
write-b -- CUSTOMER: Column-table CUST_NAME --;
  select * from CUST_NAME;
write-b -- CUSTOMER: Column-table CUST_ADDRESS --;
  select * from CUST_ADDRESS;
write-b -- CUSTOMER: Column-table CUST_CITY --;
  select * from CUST_CITY;
write-b -- CUSTOMER: Column-table CUST_CAT --;
  select * from CUST_CAT;
write-b -- CUSTOMER: Column-table CUST_ACCOUNT --;
  select * from CUST_ACCOUNT;

-- Table DETAIL   --
-- -------------- --

create temp table DET_PK(
   Entity    integer not null primary key autoincrement,
   Value1    char(12),
   Value2    char(15));
create temp table DET_ORDID(
   Entity    integer  not null,
   Value     char(12) not null);
create temp table DET_PRODID(
   Entity    integer  not null,
   Value     char(15) not null);
create temp table DET_QORD(
   Entity    integer  not null,
   Value     integer  not null);

-- First assign abstract Id to primary key values --
insert into DET_PK(Value1,Value2) select OrdID,ProdID from DETAIL order by ORDID,PRODID;

insert into DET_ORDID(Entity,Value)
  select Entity,OrdID from DET_PK K, DETAIL D where K.Value1 = D.OrdID and K.Value2 = D.ProdID;
insert into DET_PRODID(Entity,Value)
  select Entity,ProdID from DET_PK K, DETAIL D where K.Value1 = D.OrdID and K.Value2 = D.ProdID;
insert into DET_QORD(Entity,Value)
  select Entity,Qord from DET_PK K, DETAIL D where K.Value1 = D.OrdID and K.Value2 = D.ProdID;

write-b -- DETAIL: Working table DET_PK --;
  select * from DET_PK;
write-b -- DETAIL: Column-table DET_ORDID --;
  select * from DET_ORDID;
write-b -- DETAIL: Column-table DET_PRODID --;
  select * from DET_PRODID;
write-b -- DETAIL: Column-table DET_QORD --;
  select * from DET_QORD;

write-b -- Checking PK violation in column-tables of DETAIL --;
set query = select O.Value,P.Value,count(*) as Duplicate
            from   DET_ORDID O,DET_PRODID P
            where  O.Value = P.Value
            group by O.Value,P.Value
            having Duplicate > 1;

extract violations = select count(*) from ($query$);
if ($violations$ = 0);
   write [No PK violation found];
else;
   write [$violations$ duplicate PK found];
   $query$;
endif;
write;

closeDB;
