-- ------------------- --
-- The Key-Value model --
-- ------------------- --

-- Augmentation factor for table CUSTOMER --
-- N = 0: native data
-- N = 1: x2
-- N = 2: x4
-- N = 3: x8
set N = 5;

write-b -- --------------------------------- --;
write   -- KEY-VALUE data model - Version 1  --;
write   -- --------------------------------- --;

copyFile-r $dbDirectory$/ORDERS.db,$dbDirectory$/TORDERS.db;
openDB TORDERS.db;

create temp table M_CUSTOMER(
   Entity    char(10) not null,
   Attribute char(7)  not null,
   Value     varchar(60),
   primary key (Entity,Attribute));

create temp table TMP_CUSTOMER (
   CustID  char(10) not null,
   Name    char(32) not null,
   Address char(60) not null,
   City    char(30) not null,
   Cat     char(2),
   Account decimal(9,2) not null);

-- Augment the size of table CUSTOMER --
set prefix = ABCDEFGHIJKLMNOPQRSTUVWXYZ;
for I = [1,$N$];
  insert into CUSTOMER
  select item('$prefix$',$I$,'')||CustID,Name,Address,City,Cat,Account
  from CUSTOMER;
endfor;

extract NC = select count(*) from CUSTOMER;
write @S3Number of CUSTOMER rows: $NC$;

-- Load M_CUSTOMER (SQL version) --
start-timer;
insert into M_CUSTOMER (Entity, Attribute, Value) 
       select CustID,'CustID',CustID   from CUSTOMER
 union select CustID,'Name',Name       from CUSTOMER
 union select CustID,'Address',Address from CUSTOMER
 union select CustID,'City',City       from CUSTOMER
 union select CustID,'Cat',Cat         from CUSTOMER
 union select CustID,'Account',cast(Account as char) from CUSTOMER;
compute elapsed = int($timer$*1000);

extract NC = select count(*) as "Triples" from M_CUSTOMER;
write @S3Number of M_CUSTOMER rows: $NC$ triples;

write-b @S3Load M_CUSTOMER (SQL version): $elapsed$ ms.;

delete from M_CUSTOMER;

-- Load M_CUSTOMER (procedural version) --
start-timer;
for cus,nam,add,cit,cat,acc = [select CustID,Name,Address,City,Cat,Account
                               from   CUSTOMER]
   insert into M_CUSTOMER values
          ('cus','CustID','cus'),
          ('cus','Name','nam'),
          ('cus','Address','add'),
          ('cus','City','cit'),
          ('cus','Cat',case when 'cat' = '' then null else 'cat' end),
          ('cus','Account',cast($acc$ as char));
compute elapsed = int($timer$*1000);
write @S3Load M_CUSTOMER (procedural version): $elapsed$ ms.;

-- Rebuild of table CUSTOMER (SQL version 1) --
start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select M1.Value as CustID, M2.Value as Name, 
       M3.Value as Address, 
       M4.Value as City, M5.Value as Cat, 
       cast(M6.Value as decimal) AS Account
from   M_CUSTOMER AS M1, M_CUSTOMER AS M2, M_CUSTOMER AS M3, 
       M_CUSTOMER AS M4, M_CUSTOMER AS M5, M_CUSTOMER AS M6
where  M2.Entity = M1.Entity and M3.Entity = M1.Entity 
and    M4.Entity = M1.Entity and M5.Entity = M1.Entity
and    M6.Entity = M1.Entity
and    M1.Attribute = 'CustID'
and    M2.Attribute = 'Name'
and    M3.Attribute = 'Address'
and    M4.Attribute = 'City'
and    M5.Attribute = 'Cat'
and    M6.Attribute = 'Account'
order by CustID;
compute elapsed = int($timer$*1000);
write-b @S3Rebuild of table CUSTOMER (SQL version 1): $elapsed$ ms.;

delete from TMP_CUSTOMER;

-- Rebuild of table CUSTOMER (SQL version 2) --
start-timer;
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
select 
   max(case when Attribute = 'CustID' then Value else '' end) as CustID,
   max(case when Attribute = 'Name' then Value else '' end) as Name,
   max(case when Attribute = 'Address' then Value else '' end) as Address,
   max(case when Attribute = 'City' then Value else '' end) as City,
   max(case when Attribute = 'Cat' then Value else null end) as Cat,
   max(case when Attribute = 'Account' then cast(Value as decimal) else -999999.9 end) as Account
from  M_CUSTOMER
group by Entity;
compute elapsed = int($timer$*1000);
write @S3Rebuild of table CUSTOMER (SQL version 2): $elapsed$ ms.;

if ($N$ > 6) goto STOP;

delete from TMP_CUSTOMER;

-- Rebuild of table CUSTOMER (procedural version) --
start-timer;
extract ce = select Entity from M_CUSTOMER order by Entity;
for e,a,v = [select Entity,Attribute,Value
             from   M_CUSTOMER order by Entity];
   if ('SQLdiag$' = 'NONE') exit;
   if ('e' = 'ce');
      -- Build current row --
      if ('$a$' = 'CustID')  set cus = $v$;
      if ('$a$' = 'Name')    set nam = $v$;
      if ('$a$' = 'Address') set add = $v$;
      if ('$a$' = 'City')    set cit = $v$;
      if ('$a$' = 'Cat')     set cat = $v$;
      if ('$a$' = 'Account') set acc = $v$;
   else;
      -- Insert current row --
      insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
      values('cus','nam','add','cit',
             case when 'cat' = '' then null else 'cat' end,
             cast($acc$ as decimal));
      -- and start a new one --
      set ce = $e$;
      if ('$a$' = 'CustID')  set cus = $v$;
      if ('$a$' = 'Name')    set nam = $v$;
      if ('$a$' = 'Address') set add = $v$;
      if ('$a$' = 'City')    set cit = $v$;
      if ('$a$' = 'Cat')     set cat = $v$;
      if ('$a$' = 'Account') set acc = $v$;
   endif;
endfor;
-- Insert the last row --
insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
values('cus','nam','add','cit',
       case when 'cat'='' then null else 'cat' end,
       cast($acc$ as decimal));
compute elapsed = int($timer$*1000);
write @S3Rebuild of table CUSTOMER (procedural version): $elapsed$ ms.;

delete from TMP_CUSTOMER;

-- Reconstruction of table CUSTOMER (improved procedural version) --
start-timer;
extract ce = select Entity from M_CUSTOMER order by Entity;
for e,a,v = [select Entity,Attribute,Value
             from   M_CUSTOMER order by Entity];
   if ('SQLdiag$' = 'NONE') exit;
   if ('e' = 'ce');
      -- Build current row --
      if ('$a$' = 'CustID');
         set cus = $v$;
      else;
         if ('$a$' = 'Name');
            set nam = $v$;
         else;
            if ('$a$' = 'Address');
               set add = $v$;
            else;
               if ('$a$' = 'City');
                  set cit = $v$;
               else;
                  if ('$a$' = 'Cat');
                     set cat = $v$;
                  else;
                     set acc = $v$;
                  endif;
               endif;
            endif;
         endif;
      endif;
   else;
      -- Insert current row --
      insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
      values('cus','nam','add','cit',
             case when 'cat' = '' then null else 'cat' end,
             cast($acc$ as decimal));
      -- and start a new one --
      set ce = $e$;
      if ('$a$' = 'CustID');
         set cus = $v$;
      else;
         if ('$a$' = 'Name');
            set nam = $v$;
         else;
            if ('$a$' = 'Address');
               set add = $v$;
            else;
               if ('$a$' = 'City');
                  set cit = $v$;
               else;
                  if ('$a$' = 'Cat');
                     set cat = $v$;
                  else;
                     set acc = $v$;
                  endif;
               endif;
            endif;
         endif;
      endif;
   endif;
endfor;
if ('$ce$' <> '')
   -- Insert the last row --
   insert into TMP_CUSTOMER(CustID,Name,Address,City,Cat,Account)
   values('cus','nam','add','cit',
          case when 'cat'='' then null else 'cat' end,
          cast($acc$ as decimal));
compute elapsed = int($timer$*1000);
write @S3Rebuild of table CUSTOMER (improved procedural version): $elapsed$ ms.;

write;

label STOP;

closeDB;

deleteFile $dbDirectory$/TORDERS.db;

