
createDB InMemory;

write-b -- ---------------------------------------------------------- --;
write   -- CREATE TRANSACTION-TIME ATRIBUTE-BASED TEMPORAL DATABASE   --;
write   --  Small version: 3 projects, 8 employees   ---------------  --;
write-a -- ---------------------------------------------------------- --;

-- Parameters --
-- ---------- --
-- time ID of future
   set future = 999999;
-- timestamp of future
   set Tfuture = 9999-12-31;

-- starting timestamp
   set startDate = 2017-07-24;
-- max time increment
   set maxIncr = 5;

ask-u startDate,maxIncr = [/bParameters for transaction time generation]
                           Start date:|Max increment:;

-- random increment to compute next "end" timestamp
   set Dincr = random_i(0,$maxIncr$);


create table CLOCK(
   TimeID integer not null primary key,
   TTime  char(10) not null);
insert into CLOCK(TTime) values ('$startDate$');


create table H_THEME(
   TITLE    char(20)   not null,
   start    integer    not null,
   end      integer    not null default $future$,
   THEME    char(22)   not null,
   primary key (TITLE,start));

create table H_BUDGET(
   TITLE    char(20)   not null,
   start    integer    not null,
   end      integer    not null default $future$,
   BUDGET   decimal(8) not null,
   primary key (TITLE,start));

create table H_NAME(
   CODE     char(5)    not null,
   start    integer    not null,
   end      integer    not null default $future$,
   NAME     char(10)   not null,
   primary key (CODE,start));

create table H_STATUS(
   CODE     char(5)    not null,
   start    integer    not null,
   end      integer    not null default $future$,
   STATUS   char(1)    not null,
   primary key (CODE,start));

create table H_CITY(
   CODE     char(5)    not null,
   start    integer    not null,
   end      integer    not null default $future$,
   CITY     char(10)   not null,
   primary key (CODE,start));

create table H_PROJECT(
   CODE     char(5)    not null,
   start    integer    not null,
   end      integer    not null default $future$,
   PROJECT  char(20)   not null,
   primary key (CODE,start));


-- Current projects --
-- ---------------- --
create view PROJECT(TITLE,THEME,BUDGET)
as select T.TITLE,T.THEME,B.BUDGET
   from   H_THEME T, H_BUDGET B
   where  T.end = $future$
   and    B.end = $future$
   and    B.TITLE = T.TITLE;

-- Current employees --
-- ----------------- --
create view EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT)
as select N.CODE,N.NAME,S.STATUS,C.CITY,P.PROJECT
   from   H_NAME N, H_STATUS S, H_CITY C, H_PROJECT P
   where  N.end = $future$
   and    S.end = $future$
   and    C.end = $future$
   and    P.end = $future$
   and    S.CODE = N.CODE
   and    C.CODE = N.CODE
   and    P.CODE = N.CODE;

-- Allow ';' at the end of a query in the body of a trigger --
set ; = ;;

-- ---------------------------------- --
--         PROJECT MANAGEMENT         --
-- ---------------------------------- --

-- Start a project --
-- --------------- --

create trigger TRG_PRO_INSERT instead of insert on PROJECT
begin
   select raise(ABORT,'duplicate entity PK')
   where exists(select * from H_THEME where TITLE = new.TITLE) $;$
--
   insert into CLOCK(TTime)
      select addToDate(max(TTime),$Dincr$) from CLOCK $;$
--
   insert into H_THEME(TITLE,start,THEME) values
      (new.TITLE,(select max(TimeID) from CLOCK),new.THEME) $;$
--
   insert into H_BUDGET(TITLE,start,BUDGET) values
      (new.TITLE,(select max(TimeID) from CLOCK),new.BUDGET) $;$
end;

-- ---------------- --
-- Update a project --
-- ---------------- --

set technique = 0;

if (not $technique$ = 0) goto SKIP0;

-- ---------------------------------------- --
-- All reactions packed in a single trigger --
-- ---------------------------------------- --

create trigger TRG_PRO_UPDATE
instead of update on PROJECT
when new.THEME <> old.THEME or new.BUDGET <> old.BUDGET
begin
   insert into CLOCK(TTime)
      select addToDate(max(TTime),$Dincr$) from CLOCK$;$
--
   update H_THEME
   set    end = (select max(TimeID) from CLOCK) 
   where  TITLE = old.TITLE and end = 999999 and new.THEME <> old.THEME$;$
--
   insert into H_THEME(TITLE,start,THEME)
      select old.TITLE,(select max(TimeID) from CLOCK),new.THEME
      where  new.THEME <> old.THEME $;$
--
   update H_BUDGET
   set    end = (select max(TimeID) from CLOCK) 
   where  TITLE = old.TITLE and end = 999999 and new.BUDGET <> old.BUDGET$;$
--
   insert into H_BUDGET(TITLE,start,BUDGET)
      select old.TITLE,(select max(TimeID) from CLOCK),new.BUDGET
      where  new.BUDGET <> old.BUDGET $;$
end;
label SKIP0;

if (not $technique$ = 1) goto SKIP1;

-- ------------------------------------------- --
-- One trigger for each column + one for CLOCK --
-- ------------------------------------------- --
create trigger TRG_THEME_UPDATE 
instead of update of THEME on PROJECT
when new.THEME <> old.THEME
begin
   update H_THEME
   set    end = (select max(TimeID) from CLOCK) 
   where  TITLE = old.TITLE and end = 999999 $;$
   insert into H_THEME(TITLE,start,THEME) values
      (old.TITLE,(select max(TimeID) from CLOCK),new.THEME) $;$
end;

create trigger TRG_BUDGET_UPDATE 
instead of update of BUDGET on PROJECT
when new.BUDGET <> old.BUDGET
begin
   update H_BUDGET
   set    end = (select max(TimeID) from CLOCK) 
   where  TITLE = old.TITLE and end = 999999$;$
   insert into H_BUDGET(TITLE,start,BUDGET) values
     (old.TITLE,(select max(TimeID) from CLOCK),new.BUDGET)$;$
end;

-- This trigger must fire first. So, declared the last one. --
create trigger TRG_NEW_CLOCK 
instead of update of THEME,BUDGET on PROJECT
when new.THEME <> old.THEME or new.BUDGET <> old.BUDGET
begin
   insert into CLOCK(TTime)
      select addToDate(max(TTime),$Dincr$) from CLOCK$;$
end;
label SKIP1;


-- Stop a project --
-- -------------- --
create trigger TRG_PRO_DELETE
instead of delete on PROJECT
begin
-- 1. The Entity must exist --
   -- Otherwise, no diagnostic --
-- 2. There can be no dependent EMPLOYEE entities anymore --
   select raise(ABORT,'dependent EMPLOYEEs still exist')
   where  exists(select * from H_PROJECT
                 where  PROJECT = old.TITLE
                 and    end = $future$) $;$
-- 3. create a time event in CLOCK
   insert into CLOCK(TTime) select addToDate(max(TTime),$Dincr$) from CLOCK $;$
-- 4. close the current state
   update H_THEME
   set    end = (select max(TimeID) from CLOCK) 
   where  TITLE = old.TITLE and end = $future$ $;$
   update H_BUDGET
   set    end = (select max(TimeID) from CLOCK) 
   where  TITLE = old.TITLE and end = $future$ $;$
end;


-- ---------------------------------- --
--         EMPLOYEE MANAGEMENT        --
-- ---------------------------------- --

-- Create an employee --
-- ------------------ --

create trigger TRG_EMP_INSERT instead of insert on EMPLOYEE
begin
-- 1. The Entity PK must be new --
   select raise(ABORT,'duplicate entity PK')
   where  exists(select * from H_NAME where CODE = new.CODE) $;$
-- 2. The target entity of an Entity FK must exist --
   select raise(ABORT,'target PROJECT does not exist')
   where  not exists(select * from H_THEME
                     where  TITLE = new.PROJECT
                     and    end = $future$) $;$
-- 3. create a time event in CLOCK
   insert into CLOCK(TTime)
      select addToDate(max(TTime),$Dincr$) from CLOCK $;$
-- 4. insert data in H_NAME, etc. with the last time ID 
   insert into H_NAME(CODE,start,NAME) values
      (new.CODE,(select max(TimeID) from CLOCK),new.NAME) $;$
   insert into H_STATUS(CODE,start,STATUS) values
      (new.CODE,(select max(TimeID) from CLOCK),new.STATUS) $;$
   insert into H_CITY(CODE,start,CITY) values
      (new.CODE,(select max(TimeID) from CLOCK),new.CITY) $;$
   insert into H_PROJECT(CODE,start,PROJECT) values
      (new.CODE,(select max(TimeID) from CLOCK),new.PROJECT) $;$
end;


-- ------------------ --
-- Update an employee --
-- ------------------ --

create trigger TRG_EMP_UPDATE
instead of update on EMPLOYEE
when new.NAME <> old.NAME or new.STATUS  <> old.STATUS
  or new.CITY <> old.CITY or new.PROJECT <> old.PROJECT
begin
-- 3. The target entity of the new Entity FK must exist --
   select raise(ABORT,'target PROJECT does not exist')
   where  new.PROJECT <> old.PROJECT
   and    not exists(select * from H_THEME
                     where  TITLE = new.PROJECT
                     and    end = $future$) $;$
-- 4. create a time event in CLOCK
   insert into CLOCK(TTime)
      select addToDate(max(TTime),$Dincr$) from CLOCK $;$
-- 5.1 
   update H_NAME
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = 999999 and new.NAME <> old.NAME $;$
--
   insert into H_NAME(CODE,start,NAME)
      select old.CODE,(select max(TimeID) from CLOCK),new.NAME
      where  new.NAME <> old.NAME $;$
-- 5.2 
   update H_STATUS
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = 999999 and new.STATUS <> old.STATUS $;$
--
   insert into H_STATUS(CODE,start,STATUS)
      select old.CODE,(select max(TimeID) from CLOCK),new.STATUS
      where  new.STATUS <> old.STATUS $;$
-- 5.3 
   update H_CITY
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = 999999 and new.CITY <> old.CITY $;$
--
   insert into H_CITY(CODE,start,CITY)
      select old.CODE,(select max(TimeID) from CLOCK),new.CITY
      where  new.CITY <> old.CITY $;$
-- 5.4 
   update H_PROJECT
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = 999999 and new.PROJECT <> old.PROJECT $;$
--
   insert into H_PROJECT(CODE,start,PROJECT)
      select old.CODE,(select max(TimeID) from CLOCK),new.PROJECT
      where  new.PROJECT <> old.PROJECT $;$
end;


-- Delete an employee --
-- ------------------ --
create trigger TRG_EMP_DELETE
instead of delete on EMPLOYEE
begin
-- 3. create a time event in CLOCK
   insert into CLOCK(TTime) select addToDate(max(TTime),$Dincr$) from CLOCK $;$
-- 4. close the current state
   update H_NAME
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = $future$ $;$
   update H_STATUS
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = $future$ $;$
   update H_CITY
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = $future$ $;$
   update H_PROJECT
   set    end = (select max(TimeID) from CLOCK) 
   where  CODE = old.CODE and end = $future$ $;$
end;


-- ---------------------------------- --
--         INITIALIZE DATA            --
-- ---------------------------------- --

-- 2
insert into PROJECT(TITLE,THEME,BUDGET) values ('BIOTECH','Biotechnology',180000);
-- 3
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('G96','Godin','F','Genve','BIOTECH');
-- 4
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('M158','Mercier','F','Paris','BIOTECH');
-- 5
update EMPLOYEE set STATUS = 'P' where CODE = 'G96';
-- 6
update EMPLOYEE set STATUS = 'P' where CODE = 'M158';
-- 7
update PROJECT set THEME = 'Genetic engineering', BUDGET = 160000 where TITLE = 'BIOTECH';
-- 8
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('A68','Albert','P','Mons','BIOTECH');
-- 9
update PROJECT set BUDGET = 120000 where TITLE = 'BIOTECH';
-- 10
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('D122','Declercq','P','Mons','BIOTECH');
-- 11
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('C45','Carlier','F','Lille','BIOTECH');
-- 12
insert into PROJECT(TITLE,THEME,BUDGET) values ('SURVEYOR','Satellite monitoring',310000);
-- 13
update EMPLOYEE set PROJECT = 'SURVEYOR' where CODE = 'A68';
-- 14
update EMPLOYEE set PROJECT = 'SURVEYOR' where CODE = 'M158';
-- 15
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('D107','Delecourt','F','Grenoble','SURVEYOR');
-- 16
update EMPLOYEE set PROJECT = 'SURVEYOR' where CODE = 'D122';
-- 17
update EMPLOYEE set CITY = 'Genve' where CODE = 'D107';
-- 18
update PROJECT set BUDGET = 375000 where TITLE = 'SURVEYOR';
-- 19
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('N240','Nguyen','F','Toulouse','BIOTECH');
-- 20
update PROJECT set BUDGET = 140000 where TITLE = 'BIOTECH';
-- 21
insert into PROJECT(TITLE,THEME,BUDGET) values ('AGRO-2000','Crop improvement',65000);
-- 22
update EMPLOYEE set PROJECT = 'AGRO-2000', STATUS = 'F' where CODE = 'G96';
-- 23
update EMPLOYEE set PROJECT = 'AGRO-2000' where CODE = 'C45';
-- 24
update EMPLOYEE set CITY = 'Grenoble', PROJECT = 'SURVEYOR' where CODE = 'N240';
-- 25
insert into EMPLOYEE(CODE,NAME,STATUS,CITY,PROJECT) values ('A237','Antoine','P','Grenoble','AGRO-2000');
-- 26
update EMPLOYEE set CITY = 'Paris' where CODE = 'A68';
-- 27
update EMPLOYEE set STATUS = 'P', CITY = 'Grenoble' where CODE = 'D107';
-- 28
update EMPLOYEE set PROJECT = 'BIOTECH' where CODE = 'C45';
-- 29
update PROJECT set BUDGET = 345000 where TITLE = 'SURVEYOR';
-- 30
update EMPLOYEE set CITY = 'Genve', PROJECT = 'AGRO-2000' where CODE = 'N240';
-- 31
update EMPLOYEE set PROJECT = 'BIOTECH' where CODE = 'M158';
-- 32
update EMPLOYEE set CITY = 'Genve', PROJECT = 'BIOTECH' where CODE = 'D107';
-- 33
update PROJECT set BUDGET = 75000 where TITLE = 'AGRO-2000';
-- 34
update EMPLOYEE set CITY = 'Charleroi',PROJECT = 'AGRO-2000' where CODE = 'D122';
-- 35
delete from EMPLOYEE where CODE = 'A68';
-- 36
delete from PROJECT where TITLE = 'SURVEYOR';
-- 37
update PROJECT set BUDGET = 82000 where TITLE = 'AGRO-2000';
-- 38
delete from EMPLOYEE where CODE = 'D122';
-- 39
delete from EMPLOYEE where CODE = 'N240';
-- 40
update PROJECT set THEME = 'Biotechnology' where TITLE = 'BIOTECH';

commitDB;


write-b Current projects;
select * from PROJECT order by TITLE;

write-b Current employees;
select * from EMPLOYEE order by CODE;

write-b History of THEME;
select * from H_THEME  order by TITLE,start;

write-b History of BUDGET;
select * from H_BUDGET  order by TITLE,start;

write-b History of NAME;
select * from H_NAME  order by CODE,start;

write-b History of STATUS;
select * from H_STATUS  order by CODE,start;

write-b History of CITY;
select * from H_CITY  order by CODE,start;

write-b History of PROJECT;
select * from H_PROJECT  order by CODE,start;

write-b Restoring entity history of PROJECT;
select T.TITLE,max(T.start,P.start) as Tstart,
       min(P.end,T.end) as Tend,THEME,BUDGET
from   H_THEME T, H_BUDGET P
where  T.TITLE = P.TITLE
and    (P.start < T.end)  and  (T.start < P.end)
order by T.TITLE, Tstart;

write-b Restoring entity history of EMPLOYEE;
select N.CODE,max(N.start,S.start,C.start,P.start) as Tstart,
              min(S.end,N.end,C.end,P.end) as Tend,
       NAME,STATUS,CITY,PROJECT
from   H_NAME N, H_STATUS S, H_CITY C, H_PROJECT P
where  N.CODE = S.CODE and N.CODE = C.CODE and N.CODE = P.CODE
and    Tstart < Tend
order by N.CODE, Tstart;

write-b Clock events;
select * from CLOCK;

closeDB;

parameter commitmode = default;

write-b End of script;

return;