-- Database schema for transaction time with granularity "date" --

-- J-L Hainaut - May 7, 2019


write-b -- ---------------------------------------------------------- --;
write   -- DERIVE TRANSACTION-TIME DOCUMENT-BASED TEMPORAL DATABASE   --;
write   --  Small version: 8 employees                                --;
write-a -- ---------------------------------------------------------- --;

openDB TDB.db;

-- H_EMPLOYEE must be non-empty and transaction-time.
onError continue;
   extract N = select count(*) from H_EMPLOYEE;
   if ('$SQLdiag$' <> 'OK');
      showMessage Database "TDB.db" is not correct.
                  @nPlease regenerate the standard transaction-time database.;
      closeDB;
      return;
   endif;
onError stop;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' <> 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a transaction time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;

-- Some columns will be very wide --
set maxSelectWidth = 160;

-- First, we project H_EMPLOYEE on SALARY, CITY and PROJECT --
function LTemp:project {select CODE,SALARY  from H_EMPLOYEE into H_SAL};
function LTemp:project {select CODE,CITY    from H_EMPLOYEE into H_CIT};
function LTemp:project {select CODE,PROJECT from H_EMPLOYEE into H_PRO};

-- and we show the result --
write-b -- H_PRO = H_EMPLOYEE[CODE,PROJECT] --;
select * from H_PRO order by CODE,start;
-- select * from H_CIT order by CODE,start;
-- select * from H_PRO order by CODE,start;


write-b -- ------------------------------------------------- --;
write   -- Documents expressed as JSON objects               --;
write-a -- ------------------------------------------------- --;


with 
-- --
   HH_SAL(CODE,SALARY,Salaries)
   as (select CODE,SALARY,group_concat('['||start||','||end||']')
       from   H_SAL group by CODE,SALARY),
--
   HH_CIT(CODE,CITY,Cities)
   as (select CODE,CITY,group_concat('['||start||','||end||']')
       from   H_CIT group by CODE,CITY),
--
   HH_PRO(CODE,PROJECT,Projects)
   as (select CODE,PROJECT,group_concat('['||start||','||end||']')
       from   H_PRO group by CODE,PROJECT)
-- --
select sal.CODE,sal.Salaries,cit.Cities,pro.Projects
from (select CODE,'['||
          group_concat('{"value":' || SALARY || ',"intervals":['||Salaries||']}')
          || ']' as Salaries
      from   HH_SAL group by CODE) sal,
     (select CODE,'['||
          group_concat('{"value":' || CITY || ',"intervals":['||Cities||']}')
          || ']' as Cities
      from   HH_CIT group by CODE) cit,
     (select CODE,'['||
          group_concat('{"value":' || PROJECT || ',"intervals":['||Projects||']}')
          || ']' as Projects
      from   HH_PRO group by CODE) pro
where sal.CODE = cit.CODE
and   sal.CODE = pro.CODE;

write;
write-b -- ------------------------------------------------- --;
write   -- Documents expressed as SQLfast multilists         --;
write-a -- ------------------------------------------------- --;

write -- List syntax:;
write --    <values>    = <val+Inter>[;<val+Inter>]*;
write --    <val+Inter> = <value>:<intervals>;
write --    <intervals> = <interval>[,<interval>]*;
write --    <interval>  = <start>-<end>;

-- We aggregate the intervals of each project value of each employee --
--       producing HH_SAL, HH_CIT, HH_PRO                            --
-- ----------------------------------------------------------------- --

create temp table HH_PRO as
   select CODE,PROJECT,group_concat(start||'-'||end) as "Projects"
   from H_PRO group by CODE,PROJECT;

write-b -- HH_PRO --;
select * from HH_PRO order by CODE,PROJECT;

-- We aggregate the couples <PROJECT value>, <List of intervals> of each employee --
--          producing "Projects"                                                  --
-- ------------------------------------------------------------------------------ --

write-b -- PROJECT history of each employee entity --;
select CODE,group_concat(PROJECT || ':' ||Projects,';') as Projects
from   HH_PRO
group by CODE;

create temp table DOC_EMPLOYEE(CODE     char(5),
                               Salaries varchar(1024),
                               Cities   varchar(1024),
                               Projects varchar(1024));

with 
-- --
   HH_SAL(CODE,SALARY,Salaries)
   as (select CODE,SALARY,group_concat(start||'-'||end,',')
       from   H_SAL group by CODE,SALARY),
--
   HH_CIT(CODE,CITY,Cities)
   as (select CODE,CITY,group_concat(start||'-'||end,',')
       from   H_CIT group by CODE,CITY),
--
   HH_PRO(CODE,PROJECT,Projects)
   as (select CODE,PROJECT,group_concat(start||'-'||end,',')
       from   H_PRO group by CODE,PROJECT)
-- --
insert into DOC_EMPLOYEE
select sal.CODE,sal.Salaries,cit.Cities,pro.Projects
from (select CODE,group_concat(SALARY || ':'||Salaries,';')
                  as Salaries
      from   HH_SAL group by CODE) sal,
     (select CODE,group_concat(CITY || ':'||Cities,';')
                  as Cities
      from   HH_CIT group by CODE) cit,
     (select CODE,group_concat(PROJECT || ':'||Projects,';')
                  as Projects
      from   HH_PRO group by CODE) pro
where sal.CODE = cit.CODE
and   sal.CODE = pro.CODE;

write-b -- DOC_EMPLOYEE: the document based history with SQLfast lists --;
select * from DOC_EMPLOYEE;


write-b -- ------------------------------------------------- --;
write   -- Rebuilding H_EMPLOYEE from Document-based history --;
write   --     the case of H_PRO                             --;
write   -- ------------------------------------------------- --;

delete from H_PRO;

for cod,pro = [select CODE,Projects from DOC_EMPLOYEE];
   for project = [item(;) $pro$];
      compute value = item('$project$',1,':');
      compute intervals = item('$project$',2,':');
      for interval = [item(,) $intervals$];
         compute start = item('$interval$',1,'-');
         compute end   = item('$interval$',2,'-');
         insert into H_PRO(CODE,PROJECT,start,end)
                    values ('$cod$','$value$',$start$,$end$);
      endfor;
   endfor;
endfor;

write-b -- Reconstruction of H_PRO from DOC_EMPLOYEE --;
select * from H_PRO order by CODE,start;

closeDB;

write-b End of script;

return;

-- List syntax:
--    <values>    = <val+Inter> [; <val+Inter>]*
--    <val+Inter> = <value> : <intervals>
--    <intervals> = <interval> [, <interval>]*
--    <interval>  = <start> - <end>


