write-b -- ---------------------------------------------------------- --;
write   -- TEMPORAL QUERIES - TEMPORAL RESULT                         --;
write   -- Applicable to valid time database only                     --;
write-a -- ---------------------------------------------------------- --;


set future = '9999-12-31';

openDB TDB.db;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


write-b -- History of employees between 2016-01-01 and 2016-12-31 --;
select CODE,
       NAME,SALARY,CITY,PROJECT,
       max(start,'2016-01-01') as "Start", min(end,'2016-12-31' ) as "End"
from   H_EMPLOYEE
where  start < '2016-12-31' and '2016-01-01' < end 
order by NAME,"Start","End";

write-b -- Show the life span of the projects --;
select TITLE,min(start) as Birth, 
       case when max(end) = $future$ then null else max(end) end as Death
from   H_PROJECT
group by TITLE;

write-b -- Start date, end date and duration of each closed project --;
select TITLE, min(start) as "Start", max(end) as "End",
       durationDays(min(start),max(end)) as Duration
from   H_PROJECT
group by TITLE
having max(end) < $future$;

write-b -- Better through CTE --;
with MinMax(TITLE,MinStart,MaxEnd)
as (select TITLE, min(start) as MinStart, max(end) as MaxEnd
    from   H_PROJECT
    group by TITLE
    having MaxEnd < $future$)
select TITLE,MinStart as "Start",MaxEnd as "End",
       durationDays(MinStart,MaxEnd) as Duration
from   MinMax;

write-b -- Start date and current age of each active project --;
with StartAge(TITLE,MinStart)
as (select TITLE, min(start)
    from   H_PROJECT
    group by TITLE
    having max(end) = $future$)
select TITLE,MinStart as "Start",
       durationDays(MinStart,'$date$') as Age
from   StartAge;

write-b -- When did the budget for each project exceeds 150,000 --;
select TITLE,BUDGET,start,end
from   H_PROJECT
where  BUDGET > 150000;

write-b -- What was (and in what periods) the maximum budget for each project? --;
-- Too slow for large tables --
select TITLE,BUDGET,start,end
from   H_PROJECT P
where  BUDGET = (select max(BUDGET)
                 from   H_PROJECT
                 where  TITLE = P.TITLE);

closeDB;

write-b End of script;

return;
