
write-b -- ================================================================ --;
write   --            AGGREGATION ON UNPARTITIONED INTERVALS                --;
write   --            H_EMPLOYE(PROJECT,CITY --> CODE,SALARY)               --;
write-a -- ================================================================ --;

-- timestamp of future
set Tfuture = 9999-12-31;
set limit = 130;

set style = 2;
selectOne-u style = [/bStatistics based on UNPARTITIONED intervals.
                       @nChoose predicative or procedural interval computing:]
                     Predicative:||Procedural:;
if ('$DIALOGbutton$' = 'Cancel') return;

openDB TDB.db;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;

extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b -- Size of the working set --;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;

if ($style$ = 2) goto PROCEDURAL;


-- --------------------------------------------
label PREDICATIVE;
-- --------------------------------------------

write-b -- Predicative computation of VALUED_STABLE_INTERVAL --

create temp table EVENT as
   select Instant
   from (select start as Instant from H_EMPLOYEE
           union
         select end as Instant from H_EMPLOYEE);

create temp table STABLE_INTERVAL as
   select Instant as Start,
          (select min(Instant)
           from   EVENT
           where  Instant > E.Instant) as End
   from   EVENT E
   where  End is not null;

create temp table VALUED_STABLE_INTERVAL as
   select E.PROJECT,E.CITY,E.CODE,E.SALARY,S.Start,S.End
   from   STABLE_INTERVAL S,H_EMPLOYEE E
   where  S.Start >= E.start and S.End <= E.end;

drop table EVENT;
drop table STABLE_INTERVAL;


goto COMMON;

-- --------------------------------------------
label PROCEDURAL;
-- --------------------------------------------

write-b -- Procedural computation of VALUED_STABLE_INTERVAL --

function status = LTemp:valuedInterval {select PROJECT,CITY,CODE,SALARY
                                        from   H_EMPLOYEE
                                        into   VALUED_STABLE_INTERVAL};


-- --------------------------------------------
label COMMON;
-- --------------------------------------------

write;
write-b -- ================================================================ --;
write   --        Examining unpartitioned VALUED_STABLE_INTERVAL            --;
write -- ================================================================ --;

write-ab All the aggregation computations will be derived from the same table VALUED_STABLE_INTERVAL;

select thousandSep(count(*),',') as "VALUED_STABLE_INTERVAL" from VALUED_STABLE_INTERVAL;

write-b @sEach row is the contribution of one employee to one stable interval;
   select * from VALUED_STABLE_INTERVAL
   order by PROJECT,CITY,start limit $limit$;

write-b @sAn interval may be contributed by several employees.;
   select PROJECT,CITY,group_concat(CODE) as Employees,Start,End
   from   VALUED_STABLE_INTERVAL
   group by PROJECT,CITY,Start
   order by PROJECT,CITY,Start limit $limit$;

write-b @sAn employee may contribute to several intervals.;
   select CODE,count() as "Count",group_concat(Start) as Intervals
   from   VALUED_STABLE_INTERVAL
   group by CODE
   order by CODE limit $limit$;


write;
write-b -- ============================================================== --;
write   --                       EVOLUTION of COUNTING                    --;
write   -- ============================================================== --;

--write-b -- Reminder: VALUED_STABLE_INTERVAL order by PROJECT,Start --;
--select * from VALUED_STABLE_INTERVAL order by PROJECT,Start limit $limit$;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of the number of employees per (PROJECT,CITY)        --;
write   -- -------------------------------------------------------------- --;

create temp table H_COUNT as
   select PROJECT,CITY,count(*) as Number,Start,End
   from   VALUED_STABLE_INTERVAL
   group by PROJECT,CITY,Start;

select thousandSep(count(*),',') as "H_COUNT" from H_COUNT;
write-b -- Number of employees by PROJECT,CITY,Start --;
select * from H_COUNT order by PROJECT,CITY,Start limit $limit$;

function status = LTemp:reduce {select PROJECT,CITY,Number from H_COUNT};

select thousandSep(count(*),',') as "H_COUNT reduced" from H_COUNT;

write-b -- Number of employees by PROJECT,CITY,Start --;
select * from H_COUNT order by PROJECT,CITY,Start limit $limit$;

write-b -- Number of employees by PROJECT,Start,CITY --;
select * from H_COUNT order by PROJECT,Start,CITY limit $limit$;

write-b -- Number of employees by CITY,PROJECT,Start --;
select * from H_COUNT order by CITY,PROJECT,Start limit $limit$;

write-b -- Number of employees by CITY,Start,PROJECT --;
select * from H_COUNT order by CITY,Start,PROJECT limit $limit$;

drop table H_COUNT;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of the number of employees per PROJECT               --;
write   -- -------------------------------------------------------------- --;

create temp table H_COUNT as
   --select PROJECT,count(distinct CODE) as Number,Start,End
   select PROJECT,count(*) as Number,Start,End
   from   VALUED_STABLE_INTERVAL
   group by PROJECT,Start,End;

select thousandSep(count(*),',') as "H_COUNT" from H_COUNT;

function status = LTemp:reduce {select PROJECT,Number from H_COUNT};

select thousandSep(count(*),',') as "H_COUNT reduced" from H_COUNT;

write-b -- Number of employees order by PROJECT,Start --;
select * from H_COUNT order by PROJECT,Start limit $limit$;

drop table H_COUNT;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of the number of employees per CITY                  --;
write   -- -------------------------------------------------------------- --;

create temp table H_COUNT as
   --select CITY,count(distinct CODE) as Number,Start,End
   select CITY,count(*) as Number,Start,End
   from   VALUED_STABLE_INTERVAL
   group by CITY,Start,End;

select thousandSep(count(*),',') as "H_COUNT" from H_COUNT;

function status = LTemp:reduce {select CITY,Number from H_COUNT};

select thousandSep(count(*),',') as "H_COUNT reduced" from H_COUNT;
write-b -- Number of employees order by CITY,Start --;
select * from H_COUNT order by CITY,Start,End limit $limit$;


write;
write-b -- ============================================================== --;
write   --           EVOLUTION of Average, Min, Max SALARY                --;
write   -- ============================================================== --;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of average, Min, Max SALARY per (PROJECT,CITY)                    --;
write   -- -------------------------------------------------------------- --;

create temp table H_AVG_SALARY as
   select  PROJECT,CITY,
           --group_concat(distinct SALARY) as Salaries,
           round(avg(SALARY),1) as Average,
           min(SALARY) as Min,max(SALARY) as Max,Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,CITY,Start,End;

select thousandSep(count(*),',') as "H_AVG_SALARY" from H_AVG_SALARY;
write-b @sH_AVG_SALARY: average, min, max SALARY per (PROJECT,CITY);
select * from H_AVG_SALARY order by PROJECT,CITY,Start,End limit $limit$;

function status = LTemp:reduce {select PROJECT,CITY,Average,Min,Max
                                from   H_AVG_SALARY};

select thousandSep(count(*),',') as "H_AVG_SALARY reduced" from H_AVG_SALARY;
write-b @sH_AVG_SALARY reduced: average, min, max SALARY per (PROJECT,CITY);
select * from H_AVG_SALARY order by PROJECT,CITY,Start,End limit $limit$;

drop table H_AVG_SALARY;


write-b -- -------------------------------------------------------------- --;
write   -- Evolution of Average, Min, Max SALARY per (PROJECT)            --;
write   -- -------------------------------------------------------------- --;

create temp table H_AVG_SALARY as
   select  PROJECT,
           --group_concat(distinct SALARY) as Salaries,
           round(avg(SALARY),1) as Average,
           min(SALARY) as Min,max(SALARY) as Max,Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,Start,End;

select thousandSep(count(*),',') as "H_AVG_SALARY" from H_AVG_SALARY;

function status = LTemp:reduce {select PROJECT,Average,Min,Max from H_AVG_SALARY};

select thousandSep(count(*),',') as "H_AVG_SALARY reduced" from H_AVG_SALARY;
write-b @sH_AVG_SALARY reduced: average, min, max SALARY per (PROJECT);
select * from H_AVG_SALARY order by PROJECT,Start,End limit $limit$;

drop table H_AVG_SALARY;


write-b -- -------------------------------------------------------------- --;
write   -- Evolution of Average, Min, Max SALARY per (CITY)               --;
write   -- -------------------------------------------------------------- --;

create temp table H_AVG_SALARY as
   select  CITY,
           --group_concat(distinct SALARY) as Salaries,
           round(avg(SALARY),1) as Average,
           min(SALARY) as Min,max(SALARY) as Max,Start,End
   from    VALUED_STABLE_INTERVAL
   group by CITY,Start,End;

select thousandSep(count(*),',') as "H_AVG_SALARY" from H_AVG_SALARY;

function status = LTemp:reduce {select CITY,Average,Min,Max from H_AVG_SALARY};

select thousandSep(count(*),',') as "H_AVG_SALARY reduced" from H_AVG_SALARY;
write-b @sH_AVG_SALARY reduced: average, min, max SALARY per (CITY);
select * from H_AVG_SALARY order by CITY,Start,End limit $limit$;


write;
write-b -- ============================================================== --;
write   --                       EVOLUTION of SUMS                        --;
write   -- ============================================================== --;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of salary cost per (PROJECT,CITY) + detail           --;
write   -- -------------------------------------------------------------- --;

create temp table H_COST as
   select  PROJECT,CITY,
           group_concat(SALARY) as Salaries,
           round(sum(SALARY)/30.42,2) AS DailySal,
           --durationDays(Start,min(End,'$maxDate$')) as Duration,
           julianday(min(End,'$maxDate$')) - julianday(Start) as Duration,
           --round(sum(SALARY)/30.42 * durationDays(Start,min(End,'$maxDate$')),2) as TCost,
           round(sum(SALARY)/30.42 * (julianday(min(End,'$maxDate$')) - julianday(Start)),2) as TCost,
           Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,CITY,Start,End;

select thousandSep(count(*),',') as "H_COST" from H_COST;
write-b @sH_COST: average SALARY cost per (PROJECT,CITY) + detail;
select * from H_COST order by PROJECT,CITY,Start,End limit $limit$;
drop table H_COST;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of salary cost per (PROJECT,CITY)                    --;
write   -- -------------------------------------------------------------- --;

create temp table H_COST as
   select  PROJECT,CITY,
           sum(SALARY)/30.42 * (julianday(min(End,'$maxDate$')) - julianday(Start)) as TCost,
           Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,CITY,Start,End;

select thousandSep(count(*),',') as "H_COST" from H_COST;
write-b @sH_COST: average SALARY cost per (PROJECT,CITY);
select PROJECT,CITY,round(TCost,2) as "TCost",Start,End
from H_COST order by PROJECT,CITY,Start limit $limit$;

write-b -- -------------------------------------------------------------- --;
write   -- Evolution of salary cost of each PROJECT                       --;
write   -- -------------------------------------------------------------- --;

select PROJECT,round(sum(TCost),1) as "TCost",
       min(Start) as "Start",max(End) as "End"
from   H_COST
group by PROJECT,Start;


write-b -- -------------------------------------------------------------- --;
write   -- Salary cost per (PROJECT,CITY)                                 --;
write-a -- -------------------------------------------------------------- --;

select PROJECT,CITY,thousandSep(int(sum(TCost)),',') as TotalCost,
       min(Start) as "Start", max(End) as "End"
from   H_COST
group by PROJECT,CITY limit $limit$;

write-b -- -------------------------------------------------------------- --;
write   -- Salary cost per PROJECT                                        --;
write-a -- -------------------------------------------------------------- --;

select PROJECT,thousandSep(int(sum(TCost)),',') as TotalCost,
       min(Start) as "Start", max(End) as "End"
from   H_COST
group by PROJECT limit $limit$;

write-b -- -------------------------------------------------------------- --;
write   -- Salary cost per CITY                                           --;
write-a -- -------------------------------------------------------------- --;

select CITY,thousandSep(int(sum(TCost)),',') as TotalCost,
       min(Start) as "Start", max(End) as "End"
from   H_COST
group by CITY limit $limit$;

drop table H_COST;

closeDB;

write-b End of script;

return;
