
write-b -- ---------------------------------------------------------- --;
write   -- TEMPORAL (INNER) JOIN                                      --;
write   -- Applicable to valid and transaction time databases         --;
write-a -- ---------------------------------------------------------- --;

openDB TDB.db;

-- Identify transaction vs valid time data --
-- --------------------------------------- --

extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int')  set future,snap = 999999,20;
   if ('$T$' = 'char') set future,snap = '9999-12-31','2016-01-01';
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


write-b   -- Joining non temporal tables                    --;
write     -- ---------------------------------------------- --;

select CODE,NAME,E.PROJECT,THEME
from   EMPLOYEE E,PROJECT P
where  E.PROJECT = P.TITLE;

select CODE,NAME,E.PROJECT,THEME
from   EMPLOYEE E join PROJECT P
       on (E.PROJECT = P.TITLE);

write-b   -- Joining temporal tables - First invalid trial  --;
write     -- ---------------------------------------------- --;

select CODE,NAME,E.PROJECT,THEME,E.start,E.end
from   H_EMPLOYEE E,H_PROJECT P
where  E.PROJECT = P.TITLE
and    E.start = P.start;

write-b   -- Joining temporal tables - Correct algorithm    --;
write     -- ---------------------------------------------- --;

select CODE,NAME,PROJECT,THEME,
       max(P.start,E.start) as "Start", min(P.end,E.end) as "End"
from   H_EMPLOYEE E, H_PROJECT P
where  E.PROJECT = P.TITLE
and    (P.start < E.end)  and  (E.start < P.end)
order by CODE,Start,PROJECT;

write-b   -- Joining temporal tables - (Mercier,BIOTECH)    --;
write     -- ---------------------------------------------- --;

select CODE,NAME,PROJECT,THEME,
       max(P.start,E.start) as "Start", min(P.end,E.end) as "End"
from   H_EMPLOYEE E, H_PROJECT P
where  E.PROJECT = P.TITLE
and    E.NAME = 'Mercier' and E.PROJECT = 'BIOTECH'
and    (P.start < E.end)  and  (E.start < P.end)
order by CODE,Start,PROJECT;

write-b   -- Snapshot ($snap$) of a join - Inline version   --;
write     -- ---------------------------------------------- --;

select CODE,NAME,PROJECT,THEME
from  (select CODE,NAME,PROJECT,THEME,
       max(P.start,E.start) as "Start", min(P.end,E.end) as "End"
       from   H_EMPLOYEE E, H_PROJECT P
       where  E.PROJECT = P.TITLE
       and    (P.start < E.end)  and  (E.start < P.end))
where  "Start" <= $snap$ and $snap$ < "End";

write-b   -- Snapshot ($snap$) of a join - View version     --;
write     -- ---------------------------------------------- --;

create temp view H_EMPLOYEE_PROJECT(CODE,NAME,PROJECT,THEME,start,end)
as select CODE, NAME, PROJECT, THEME, max(P.start,E.start), min(P.end,E.end)
   from   H_EMPLOYEE E, H_PROJECT P
   where  E.PROJECT = P.TITLE
   and    (P.start < E.end)  and  (E.start < P.end);

select CODE, NAME, PROJECT, THEME
from   H_EMPLOYEE_PROJECT
where  start <= $snap$ and $snap$ < end;

closeDB;

write-b End of script;

return;
