write-b -- ---------------------------------------------------------- --;
write   -- TEMPORAL OUTER JOIN                                        --;
write   -- Applicable to valid and transaction time databases         --;
write-a -- ---------------------------------------------------------- --;

createDB InMemory;

write-b -- --------------------------------------------- --;
write   -- DATABASE INITIALIZATION                       --;
write   -- --------------------------------------------- --;


write-b @s           1    2    3    4    5    6    7    8    9   10   11   12   13;
write   PRO(p1):    |    |<---|--->|    |    |<---|----|--->|    |<---|--->|<-->|;
write   PRO(p2):    |    |    |<---|--->|<---|----|--->|    |    |<---|--->|    |;
write;
write   EMP(e1.p1): |<---|--->|<---|--->|    |<-->|    |<---|----|--->|    |    |;
write   EMP(e2.p1): |    |    |<---|----|--->|    |    |    |    |    |    |    |;

write   EMP(e1.p2): |    |    |    |    |    |    |    |    |    |    |    |    |;
write   EMP(e2.p2): |    |    |    |    |    |    |<---|----|--->|    |<-->|    |;


create table PRO(ProID,Name,Budg,start,end);
create table EMP(EmpID,Sal,Proj,start,end);

insert into PRO(ProID,Name,Budg,start,end) values
       ('p1','bio',120, 2, 4)
      ,('p1','bio',105, 6 ,9)
      ,('p1','bio',130,10,12)
      ,('p1','bio',142,12,13)
      ,('p2','tec',125, 3, 5)
      ,('p2','tec',130, 5, 8)
      ,('p2','tec',142,10,12)
;
insert into EMP(EmpID,Sal,Proj,start,end) values
       ('e1',10,'p1', 1, 3)
      ,('e1',12,'p1', 3 ,5)
      ,('e1',15,'p1', 6, 7)
      ,('e1',16,'p1', 8,11)
      ,('e2',12,'p1', 3, 6)
      ,('e2',15,'p2', 7,10)
      ,('e2',16,'p2',11,12)
;

write;
write-b Source table PRO;
select * from PRO;

write-b Source table EMP;
select * from EMP;

write;
write-b -- --------------------------------------------- --;
write   -- INNER JOIN  EMP * PRO                         --;
write   -- --------------------------------------------- --;

write-b Inner join EMP * PRO;
create temp table EMP_PRO as
select EmpID,Sal,ProID,Name,Budg,
       max(EMP.start,PRO.start) as "Start", min(EMP.end,PRO.end) as "End"
from   EMP,PRO
where  EMP.Proj = PRO.ProID
and    (EMP.start < PRO.end) and (PRO.start < EMP.end)
order by EmpID,Start;

select * from EMP_PRO order by EmpID,Start;

write-b Trying to extract source table EMP from Inner join;
function status = LTemp:project {select EmpID,Sal,ProID from EMP_PRO into EMP1};
select * from EMP1 order by EmpID,Start;
 
write-b Trying to extract source table PRO from Inner join;
function status = LTemp:project {select ProID,Name,Budg from EMP_PRO into PRO1};
select * from PRO1 order by ProID,Start;

-- ----------------------------------------------------- --
-- Select a technique                                    --
-- ----------------------------------------------------- --

set tech = 1;
selectOne-u tech = [/bSelect join technique] procedural technique (LTemp):|predicative technique (SQL):;
   if ('$DIALOGbutton$' = 'Cancel') return;
if ($tech$ = 1) goto PROCEDURAL;
if ($tech$ = 2) goto PREDICATIVE;
return;

label PROCEDURAL;

write;
write-b -- ==================================================================== --;
write   --                        PROCEDURAL TECHNIQUE                          --;
write-a -- ==================================================================== --;

write-b -- --------------------------------------------- --;
write   -- LEFT OUTER JOIN PRO * EMP                     --;
write   -- --------------------------------------------- --;

--function status = LTemp:normalize {update EMP wrt PRO on Proj = ProID into EMP_COMPLETED};
function status = LTemp:normalize EMP,PRO,{Proj},{ProID},EMP_COMPLETED;
if ($status$ > 0) write Error in "LTemp:normalize": status = $status$;

onError continue;
  write-b Missing intervals in EMP (table MISSING);
  select * from MISSING order by Proj,start;
  drop table if exists MISSING;
onError stop;

write-b Table EMP completed (table EMP_COMPLETED);
select EmpID,Sal,Proj,start,end from EMP_COMPLETED order by Proj,start;

drop table if exists LEFT;
create temp table LEFT as
  select PRO.ProID,Name,Budg,EmpID,Sal,
         max(PRO.start,EMPC.start) as "Start", min(PRO.end,EMPC.end) as "End"
  from   PRO, EMP_COMPLETED EMPC
  where  PRO.ProID = EMPC.Proj
  and    (PRO.start < EMPC.end) and (EMPC.start < PRO.end);

write-b Left outer join PRO * EMP;
select * from LEFT order by ProID,EmpID,Start;


write;
write-b -- --------------------------------------------- --;
write   -- RIGHT OUTER JOIN PRO * EMP                    --;
write   -- --------------------------------------------- --;

--function status = LTemp:normalize {update PRO wrt EMP on ProID = Proj into PRO_COMPLETED};
function status = LTemp:normalize PRO,EMP,{ProID},{Proj},PRO_COMPLETED;
if ($status$ > 0) write Error in "LTemp:normalize": status = $status$;

onError continue;
  write-b Missing intervals in PRO (table MISSING);
  select * from MISSING order by ProID,start;
  drop table if exists MISSING;
onError stop;

write-b Table PRO completed (table PRO_COMPLETED);
select * from PRO_COMPLETED order by ProID,start;

create temp table RIGHT as
  select EmpID,Sal,EMP.Proj,Name,Budg,
         max(EMP.start,PROC.start) as "Start", min(EMP.end,PROC.end) as "End"
  from   EMP, PRO_COMPLETED PROC
  where  EMP.Proj = PROC.ProID
  and    (EMP.start < PROC.end) and (PROC.start < EMP.end);

write-b Right outer join PR0 * EMP;
select * from RIGHT order by Proj,EmpID,Start;
--select Proj as ProID,Name,Budg,EmpID,Sal,Start,End from RIGHT order by Proj,EmpID,Start;


write;
write-b -- --------------------------------------------- --;
write   -- FULL OUTER JOIN PRO * EMP                     --;
write   -- --------------------------------------------- --;

create temp table FULL as
  select ProID,Name,Budg,EmpID,Sal,Start,End from LEFT
     union
  select Proj as ProID,Name,Budg,EmpID,Sal,Start,End from RIGHT;


write-b Full outer join PRO * EMP (sorted by ProID,EmpID,Start);
select ProID,Name,Budg,EmpID,Sal,Start,End from FULL order by ProID,EmpID,Start;

write-b Full outer join PRO * EMP (sorted by EmpID,ProID,Start);
select EmpID,Sal,ProID,Name,Budg,Start,End from FULL order by EmpID,ProID,Start;


write;
write-b -- --------------------------------------------- --;
write   -- Recovering source tables from full outer join --;
write   -- --------------------------------------------- --;

write-b Recovering PRO from FULL outer join;
function status = LTemp:project {select ProID,Name,Budg from FULL into NewPRO};
if ($status$ > 0) write Error in "LTemp:project": status = $status$;
delete from NewPRO where (Name,Budg) is (null,null);

select ProID,Name,Budg,Start as "start",End as "end" 
from   NewPRO order by ProID,start;


write-b Recovering EMP from FULL outer join;
function status = LTemp:project {select EmpID,Sal,ProID from FULL into NewEMP};
if ($status$ > 0) write Error in "LTemp:project": status = $status$;
delete from NewEMP where (EmpID,Sal) is (null,null); 

select EmpID,Sal,ProID as Proj,Start as "start",End as "end" 
from   NewEMP order by EmpID,start;

closeDB;

write-b End of script;

return;


label PREDICATIVE;

write;
write-b -- ==================================================================== --;
write   --                       PREDICATIVE TECHNIQUE                          --;
write   -- ==================================================================== --;


write;
write-b -- --------------------------------------------- --;
write   -- LEFT OUTER JOIN PRO * EMP                     --;
write   -- --------------------------------------------- --;

-- ------------------------- --
-- MISSING INTERVALS IN EMP  --
-- ------------------------- --

create temp table EMP_MISSING as
  select EMPa.Proj, null as "EmpID",null as "Sal",EMPa.end as Tstart, EMPb.start as Tend
  from   EMP EMPa, EMP EMPb
  where  EMPa.Proj = EMPb.Proj
  and    EMPa.end < EMPb.start
  and not exists (select *
                  from   EMP EMPc
                  where  EMPc.Proj = EMPa.Proj
                  and    EMPc.start < EMPb.start
                  and    EMPa.end < EMPc.end)
     union
  select EMP.Proj,null,null,min(PRO.start) as Tstart,min(EMP.start) as Tend
  from PRO, EMP
  where PRO.ProID = EMP.Proj
  group by EMP.Proj
  having Tstart < Tend
     union
  select EMP.Proj,null,null,max(EMP.end) as Tstart,max(PRO.end) as Tend
  from PRO, EMP
  where PRO.ProID = EMP.Proj
  group by EMP.Proj
  having Tstart < Tend
     union
  select ProID as Proj,null as EmpID,null as Sal,
         min(start) as "start",max(end) as "end"
  from   PRO
  where  ProID not in (select Proj from EMP)
  group by ProID;

write-b Missing intervals in EMP (table EMP_MISSING);
select EmpID,Sal,Proj,Tstart,Tend from EMP_MISSING order by Proj,Tstart;


-- ------------------------- --
-- COMPLETING EMP            --
-- ------------------------- --

create temp table EMP_COMPLETED as
  select Proj,EmpID,Sal,start as Tstart,end as Tend from EMP
     union
  select Proj,null,null,Tstart,Tend from EMP_MISSING;

write-b Table EMP completed (table EMP_COMPLETED);
select EmpID,Sal,Proj,Tstart,Tend from EMP_COMPLETED order by Proj,Tstart;


-- ------------------------- --
-- LEFT OUTER JOIN PRO * EMP --
-- ------------------------- --

drop table if exists LEFT;
create temp table LEFT as
  select PRO.ProID,Name,Budg,EmpID,Sal,
         max(PRO.start,EMPC.Tstart) as "Start", min(PRO.end,EMPC.Tend) as "End"
  from   PRO, EMP_COMPLETED EMPC
  where  PRO.ProID = EMPC.Proj
  and    (PRO.start < EMPC.Tend) and (EMPC.Tstart < PRO.end);

write-b Left outer join PRO * EMP;
select * from LEFT order by ProID,EmpID,Start;



write;
write-b -- --------------------------------------------- --;
write   -- RIGHT OUTER JOIN PRO * EMP                    --;
write   -- --------------------------------------------- --;

-- ------------------------- --
-- MISSING INTERVALS IN PRO  --
-- ------------------------- --

create temp table PRO_MISSING as
  select PROa.ProID, null as "Name",null as "Budg",PROa.end as Tstart, PROb.start as Tend
  from   PRO PROa, PRO PROb
  where  PROa.ProID = PROb.ProID
  and    PROa.end < PROb.start
  and not exists (select *
                  from   PRO PROc
                  where  PROc.ProID = PROa.ProID
                  and    PROc.start < PROb.start
                  and    PROa.end < PROc.end)
     union
  select PRO.ProID,null,null,min(EMP.start) as Tstart,min(PRO.start) as Tend
  from PRO, EMP
  where PRO.ProID = EMP.Proj
  group by PRO.ProID
  having Tstart < Tend
     union
  select PRO.ProID,null,null,max(PRO.end) as Tstart,max(EMP.end) as Tend
  from PRO, EMP
  where PRO.ProID = EMP.Proj
  group by PRO.ProID
  having Tstart < Tend
     union
  select Proj as ProID,null,null,
         min(start) as "start",max(end) as "end"
  from   EMP
  where  Proj not in (select ProID from PRO)
  group by ProJ;

write-b Missing intervals in PRO (table PRO_MISSING);
select ProID,Name,Budg,Tstart,Tend from PRO_MISSING;

-- ---------------------------- --
-- COMPLETING PRO               --
-- ---------------------------- --

create temp table PRO_COMPLETED as
  select ProID,Name,Budg,start as Tstart,end as Tend from PRO
     union
  select ProID,null,null,Tstart,Tend from PRO_MISSING;

write-b Table PRO completed (table PRO_COMPLETED);
select * from PRO_COMPLETED order by ProID,Tstart;

-- ---------------------------- --
-- RIGHT OUTER JOIN PRO * EMP   --
-- ---------------------------- --

drop table if exists RIGHT;
create temp table RIGHT as
  select EmpID,Sal,EMP.Proj,Name,Budg,
         max(EMP.start,PROC.Tstart) as "Start", min(EMP.end,PROC.Tend) as "End"
  from   EMP, PRO_COMPLETED PROC
  where  EMP.Proj = PROC.ProID
  and    (EMP.start < PROC.Tend) and (PROC.Tstart < EMP.end);

write-b Right outer join PRO * EMP;
select * from RIGHT order by EmpID,Start;


write;
write-b -- --------------------------------------------- --;
write   -- FULL OUTER JOIN PRO * EMP                     --;
write   -- --------------------------------------------- --;

create temp table FULL as
  select ProID,Name,Budg,EmpID,Sal,Start,End from LEFT
     union
  select Proj as ProID,Name,Budg,EmpID,Sal,Start,End from RIGHT;


write-b Full outer join PRO * EMP (sorted by ProID,EmpID,Start);
select ProID,Name,Budg,EmpID,Sal,Start,End from FULL order by ProID,EmpID,Start;

write-b Full outer join PRO * EMP (sorted by EmpID,ProID,Start);
select EmpID,Sal,ProID,Name,Budg,Start,End from FULL order by EmpID,ProID,Start;


write;
write-b -- --------------------------------------------- --;
write   -- Recovering source tables from full outer join --;
write   -- --------------------------------------------- --;

write-b Recovering PRO from FULL outer join;
function status = LTemp:project {select ProID,Name,Budg from FULL into NewPRO};
if ($status$ > 0) write Error in "LTemp:project": status = $status$;
delete from NewPRO where (Name,Budg) is (null,null);

select ProID,Name,Budg,Start as "start",End as "end" 
from   NewPRO order by ProID,start;


write-b Recovering EMP from FULL outer join;
function status = LTemp:project {select EmpID,Sal,ProID from FULL into NewEMP};
if ($status$ > 0) write Error in "LTemp:project": status = $status$;
delete from NewEMP where (EmpID,Sal) is (null,null); 

select EmpID,Sal,ProID as Proj,Start as "start",End as "end" 
from   NewEMP order by EmpID,start;


closeDB;

write-b End of script;

return;

