
set Tfuture = 9999-12-31;
set limit = 10;

write-b -- ================================================================ --;
write   -- AGGREGATION ON UNPARTITIONED INTERVALS                           --;
write   -- Applicable to valid time databases only                          --;
write   -- ================================================================ --;

-- Select a database --
set db = TDB;
ask-u db = [/bSelect a database] 
            Database:[(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database "$db$" not found.
               @nPlease load the experimental temporal databases from SQLfast website;
   return;
endif;


openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;


set style = 2;
selectOne-u style = [/bStatistics based on UNPARTITIONED intervals.
                     @nChoose predicative or procedural interval computing:]
                     Predicative:||Procedural:;
if ('$DIALOGbutton$' = 'Cancel') return;


extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;


if ($style$ = 2) goto PROCEDURAL;


-- ---------------
label PREDICATIVE;
-- ---------------

write-b -- ==================================================================== --;
write   -- S1. Compute table VALUED_STABLE_INTERVAL (PROJECT,CITY,CODE,SALARY)  --;
write   --     Predicative technique                                            --;
write-a -- ==================================================================== --;

start-timer;
create temp table EVENT as
   select Instant
   from (select start as Instant from H_EMPLOYEE
           union
         select end as Instant from H_EMPLOYEE);

create temp table STABLE_INTERVAL as
   select Instant as Start,
          (select min(Instant)
           from   EVENT
           where  Instant > E.Instant) as End
   from   EVENT E
   where  End is not null;

create temp table VALUED_STABLE_INTERVAL as
   select E.PROJECT,E.CITY,E.CODE,E.SALARY,S.Start,S.End
   from   STABLE_INTERVAL S,H_EMPLOYEE E
   where  S.Start >= E.start and S.End <= E.end;

set S1 = $timer$;
extract S1size = select thousandSep(count(*),',') from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,start limit $limit$;

drop table EVENT;
drop table STABLE_INTERVAL;

goto COMMON;

-- --------------
label PROCEDURAL;
-- --------------

write-b -- ==================================================================== --;
write   -- S1. Compute table VALUED_STABLE_INTERVAL (PROJECT,CITY,CODE,SALARY)  --;
write   --     Procedural technique                                             --;
write-a -- ==================================================================== --;

start-timer;
function status = LTemp:valuedInterval {select PROJECT,CITY,CODE,SALARY
                                        from   H_EMPLOYEE
                                        into   VALUED_STABLE_INTERVAL};

set S1 = $timer$;
extract S1size = select thousandSep(count(*),',') from VALUED_STABLE_INTERVAL;
select * from VALUED_STABLE_INTERVAL order by PROJECT,start limit $limit$;



-- ---------
label COMMON;
-- ---------

write;
write-b -- ============================================================== --;
write   --                       EVOLUTION of COUNTING                    --;
write   -- ============================================================== --;


write-b -- -------------------------------------------------------------- --;
write   -- S2. Evolution of the number of employees per PROJECT           --;
write   -- -------------------------------------------------------------- --;

start-timer;
create temp table H_COUNT as
   select PROJECT,count(*) as Number,Start,End
   from   VALUED_STABLE_INTERVAL
   group by PROJECT,Start,End;

set S2 = $timer$;
extract S2size = select thousandSep(count(*),',') from H_COUNT;


write-b -- -------------------------------------------------------------- --;
write   -- S3. Reducing                                                   --;
write   -- -------------------------------------------------------------- --;
start-timer;

function status = LTemp:reduce {select PROJECT,Number from H_COUNT};

set S3 = $timer$;
extract S3size = select thousandSep(count(*),',') from H_COUNT;
select * from H_COUNT order by PROJECT,Start limit $limit$;

drop table H_COUNT;



write;
write-b -- ============================================================== --;
write   --           EVOLUTION of Average, Min, Max SALARY                --;
write   -- ============================================================== --;


write-b -- -------------------------------------------------------------- --;
write   -- S4. Evolution of Average, Min, Max SALARY per (PROJECT)        --;
write   -- -------------------------------------------------------------- --;

start-timer;
create temp table H_AVG_SALARY as
   select  PROJECT,
           --group_concat(distinct SALARY) as Salaries,
           round(avg(SALARY),1) as Average,
           min(SALARY) as Min,max(SALARY) as Max,Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,Start,End;

set S4 = $timer$;
extract S4size = select thousandSep(count(*),',') from H_AVG_SALARY;

write-b -- -------------------------------------------------------------- --;
write   -- S5. Reducing                                                   --;
write   -- -------------------------------------------------------------- --;

start-timer;
function status = LTemp:reduce {select PROJECT,Average,Min,Max from H_AVG_SALARY};

set S5 = $timer$;
extract S5size = select thousandSep(count(*),',') from H_AVG_SALARY;
select * from H_AVG_SALARY order by PROJECT,Start limit $limit$;

drop table H_AVG_SALARY;


write;
write-b -- ============================================================== --;
write   --                       EVOLUTION of SUMS                        --;
write   -- ============================================================== --;


write-b -- -------------------------------------------------------------- --;
write   -- S6. Evolution of salary cost per PROJECT                       --;
write   -- -------------------------------------------------------------- --;

start-timer;
create temp table H_COST as
   select  PROJECT,
           round(sum(SALARY)/30.42 * (julianday(min(End,'$maxDate$')) - julianday(Start)),'1') as TCost,
           Start,End
   from    VALUED_STABLE_INTERVAL
   group by PROJECT,Start,End;

set S6 = $timer$;
extract S6size = select thousandSep(count(*),',') from H_COST;
select * from H_COST order by PROJECT,Start limit $limit$;


closeDB;



-- ========================== PERFORMANCE SYNTHESIS =========================== --

write;
write-ab PERFORMANCE SYNTHESIS;

createDB InMemory;

if ($style$ = 1) write -- Predicative technique --;
if ($style$ = 2) write -- Procedural technique  --;
select thousandSep($Nemp$,',') as "#employees", 'S1' as Query, 'VALUED_STABLE_INTERVAL' as Operation, '$S1$' as Time, '$S1size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S2' as Query, 'COUNTING' as Operation, '$S2$' as Time, '$S2size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S3' as Query, 'reduce' as Operation, '$S3$' as Time, '$S3size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S4' as Query, 'STATISTICS' as Operation, '$S4$' as Time, '$S4size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S5' as Query, 'reduce' as Operation, '$S5$' as Time, '$S5size$' as Rows
  union
select thousandSep($Nemp$,',') as "#employees", 'S6' as Query, 'COST' as Operation, '$S6$' as Time, '$S6size$' as Rows
order by Query;


closeDB;

write-b End of script;

return;