
set Tfuture = 9999-12-31;
set limit = 10;

write-b -- ========================================================== --;
write   -- Temporal queries with non-temporal result                  --;
write   -- Applicable to valid time databases only                    --;
write-a -- ========================================================== --;

-- Select a database --
set db = TDB;
ask-u db = [/bSelect a database] 
            Database:[(TDB,TDB-100,TDB-200,TDB-500,TDB-1000,TDB-2000,TDB-5000,TDB-10000,TDB-20000,TDB-50000,TDB-100000,TDB-200000)];
if ('$DIALOGbutton$' = 'Cancel') return;

set db = $db$.db;

if (not fileExists('$dbDirectory$/$db$'));
   showMessage Database "$db$" not found.
               @nPlease load the experimental temporal databases from SQLfast website;
   return;
endif;

openDB $db$;

-- H_EMPLOYEE must be non-empty and valid-time.
extract N = select count(*) from H_EMPLOYEE;
if ($N$ > 0);
   extract T = select typeof(start) from H_EMPLOYEE limit 1;
   if ('$T$' = 'int');
      showMessage WARNING:
                  @n@S3TDB.db is not a valid time database.
                  @n@S3Operation aborted.;
      closeDB;
      return;
   endif;
else;
   showMessage Database empty.@nOperation aborted.;
   closeDB;
   return;
endif;

extract Nemp = select count(distinct CODE) from H_EMPLOYEE;
extract maxDate = select addToDate(max(end),100) from H_EMPLOYEE where end < '$Tfuture$';

write-b -- Size of the working set --;
select thousandSep($Nemp$,',') as "EMPLOYEE",
       thousandSep(count(*),',') as "H_EMPLOYEE"
from H_EMPLOYEE;


write-b -- ----------------------------------------------------------------------------- --;
write   -- Q1: Show the state of employee D10626 on 2000-01-01                           --;
write   -- ----------------------------------------------------------------------------- --;
start-timer;
create temp table RES as
   select CODE,NAME,SALARY,CITY,PROJECT
   from   H_EMPLOYEE
   where  CODE = 'D10626'
   and    start <= '2000-01-01' and '2000-01-01' < end;
set Q1 = $timer$;
extract Q1size = select thousandSep(count(*),',');
select * from RES limit $limit$;
drop table RES;

write-b -- ----------------------------------------------------------------------------- --;
write   -- Q2: How many employees were assigned to project SURVEYOR on 2002-02-01        --;
write   -- ----------------------------------------------------------------------------- --;
start-timer;
create temp table RES as
   select count(distinct CODE) as Staff
   from   H_EMPLOYEE
   where  PROJECT = 'SURVEYOR'
   and    start <= '2002-02-01' and '2002-02-01' < end;
set Q2 = $timer$;
extract Q2size = select thousandSep(count(*),',');
select * from RES limit $limit$;
drop table RES;

write-b -- ----------------------------------------------------------------------------- --;
write   -- Q3: How many projects has employee D10626 been assigned to during his career? --;
write   -- ----------------------------------------------------------------------------- --;
start-timer;
create temp table RES as
   select count(distinct PROJECT) as Projects
   from   H_EMPLOYEE where  CODE = 'D10626';
set Q3 = $timer$;
extract Q3size = select thousandSep(count(*),',');
select * from RES limit $limit$;
drop table RES;

write-b -- ---------------------------------------------------------------------------------- --;
write   -- Q4: Which project(s) has employee D10626 been assigned to the longest?            --;
write   -- ---------------------------------------------------------------------------------- --;
start-timer;
create temp table RES as
   select PROJECT, sum(durationDays(start,min(end,'$maxDate$'))) as Duration
   from   H_EMPLOYEE
   where  CODE = 'D10626'
   group by PROJECT
   having Duration = (select sum(durationDays(start,min(end,'$maxDate$'))) as Dur
                      from   H_EMPLOYEE
                      where  CODE = 'D10626'
                      group by PROJECT order by Dur desc limit 1);
set Q4 = $timer$;
extract Q4size = select thousandSep(count(*),',');
select * from RES limit $limit$;
drop table RES;

write;
write-ab PERFORMANCE SYNTHESIS;

select thousandSep($Nemp$,',') as "#employees",'Q1' as Query,$Q1$ as Time,$Q1size$ as Rows
  union
select thousandSep($Nemp$,',') as "#employees",'Q2' as Query,$Q2$ as Time,$Q2size$ as Rows
  union
select thousandSep($Nemp$,',') as "#employees",'Q3' as Query,$Q3$ as Time,$Q3size$ as Rows
  union
select thousandSep($Nemp$,',') as "#employees",'Q4' as Query,$Q4$ as Time,$Q3size$ as Rows
order by Query;


closeDB;

write-b End of script;

return;
