-- TOPOLOGICAL-SORTING.sql --

-- Select a graph to sort. Result in variable "graph" and "graphFile" --
execSQL $sourceDir$/__Select-a-Graph.sql;
   if ('$DIALOGbutton$' = 'Cancel') return;

-- Create analysis database TOPO.db --
createOrReplaceDB TOPO.db;
execSQL $sourceDir$/__Create-Data-Structures.sql;

-- Load a schema graph -
execSQL $graphFile$;

-- Load working tables --
insert into NODE_W select * from NODE;
insert into LINK_W select * from LINK;

-- Remove root and leaf nodes --
execSQL $sourceDir$/__Remove-Roots-Leaves.sql;

-- If circuits exist, remove optional links and redo --
extract Nnode = select count(*) from NODE_W;
if ($Nnode$ > 0);
   insert into LINK_UPDATE select * from LINK_W where Opt = 'Y';
   delete from LINK_W where Opt = 'Y';
   execSQL $sourceDir$/__Remove-Roots-Leaves.sql;
endif;

-- Evaluate the hard cyclic kernel --
extract Nnode = select count(*) from NODE_W;
if ($Nnode$ = 0);
   write-b -- The hard cyclic kernel is empty. The graph is acyclic --;
   insert into NODE_A select * from NODE;
   insert into LINK_A select * from LINK
                      where  LinkID not in (select LinkID
                                            from LINK_UPDATE);
else;
   write-b -- The hard cyclic kernel is not empty. The graph is not acyclic --;
   --write @S8-- Compute-CLOSURE;
   execSQL $sourceDir$/__Compute-CLOSURE.sql;
   --write @S8-- Compute-SUPER_CIRCUIT;
   execSQL $sourceDir$/__Compute-SUPER_CIRCUIT.sql;
   --write @S8-- Compute-TRANSLATION;
   execSQL $sourceDir$/__Compute-TRANSLATION.sql;
   --write @S8-- Move-to-Acyclic-Tables;
   execSQL $sourceDir$/__Move-to-Acyclic-Tables.sql;
   --write @S8-- Restore-Optional-Links;
   execSQL $sourceDir$/__Restore-Optional-Links.sql;
endif;

-- Select the output format --
set style = Neutral;
set header = ;
set trailer = ;
ask-u style = [/bSelect presentation style for sorted items.]
              Style:[(Neutral,SQL loading)];
if ('$style$' = 'SQL loading');
  set header = execSQL load_;
  set trailer = .sql;;
endif;

-- Generate the sorted list of nodes --
write-b -- Sorted list of nodes --;
execSQL $sourceDir$/__Generate-Loading-Script.sql;

closeDB;