-- TOPOLOGICAL-SORTING (Tracing).sql                                        --
-- Same as TOPOLOGICAL-SORTING.sql + timing information and control display --

set limit = 60;
ask-u limit = [/bSome tables are very large.
               @nSpecify the maximum number of rows to display]
               Number of rows:;

-- Select a graph to sort. Result in variable "graph" and "graphFile" --
execSQL $sourceDir$/__Select-a-Graph.sql;
   if ('$DIALOGbutton$' = 'Cancel') return;
write-b -- Graph selected: "$graph$" --;

-- Create analysis database TOPO.db --
deleteDB TOPO.db;
createOrReplaceDB TOPO.db;
execSQL $sourceDir$/__Create-Data-Structures.sql;
write -- Database TOPO.db Created --;

-- Load a schema graph -
execSQL $graphFile$;
write -- Graph $graphFile$ loaded --;

   write-b ==================================================;
   write   =                  SOURCE GRAPH                  =;
   write   ==================================================;
   write-b Table NODE;
   select * from NODE #[1,$limit$];
   write-b Table LINK;
   select * from LINK #[1,$limit$];

-- Load working tables --
write -- Working tables "NODE_W" and "LINK_W" loaded --;
insert into NODE_W select * from NODE;
insert into LINK_W select * from LINK;

-- Remove root and leaf nodes --
execSQL $sourceDir$/__Remove-Roots-Leaves.sql;
extract Nnode = select count(*) from NODE_W;
extract Nlink = select count(*) from LINK_W;
write -- Root and leaf nodes removed --;

   write-b ==================================================;
   write   =                  CYCLIC KERNEL                 =;
   write   ==================================================;
   write-b Table NODE_W;
   select * from NODE_W #[1,$limit$];
   write-b Table LINK_W;
   select * from LINK_W #[1,$limit$];
   write-b Source nodes not in table NODE_W;
   select * from NODE except select * from NODE_W #[1,$limit$];
   write-b Source links not in table LINK_W;
   select * from LINK except select * from LINK_W #[1,$limit$];

-- If circuits exist, remove optional links and redo --
extract Nnode = select count(*) from NODE_W;
if ($Nnode$ > 0);
   insert into LINK_UPDATE
       select * from LINK_W where Opt = 'Y';
   delete from LINK_W where Opt = 'Y';
   execSQL $sourceDir$/__Remove-Roots-Leaves.sql;
endif;
extract Nnode = select count(*) from NODE_W;
extract Nlink = select count(*) from LINK_W;

   write-b ==================================================;
   write   =               HARD CYCLIC KERNEL               =;
   write   ==================================================;
   write-b [Hard cyclic kernel ($Nnode$ nodes and $Nlink$ links) computed];
   write-b Table NODE_W;
   select * from NODE_W #[1,$limit$];
   write-b Table LINK_W;
   select * from LINK_W #[1,$limit$];
   write-b Table LINK_UPDATE;
   select * from LINK_UPDATE #[1,$limit$];

extract Nnode = select count(*) from NODE_W;
if ($Nnode$ = 0);
   write-b -- The hard cyclic kernel is empty. The graph is acyclic --;
   insert into NODE_A select * from NODE;
   insert into LINK_A select * from LINK
                      where  LinkID not in (select LinkID
                                            from LINK_UPDATE);
else;
   write-b -- The hard cyclic kernel is not empty. The graph is not acyclic --;
   execSQL $sourceDir$/__Compute-CLOSURE.sql;
   execSQL $sourceDir$/__Compute-SUPER_CIRCUIT.sql;
   execSQL $sourceDir$/__Compute-TRANSLATION.sql;
   execSQL $sourceDir$/__Move-to-Acyclic-Tables.sql;
   execSQL $sourceDir$/__Restore-Optional-Links.sql;
endif;

   write-b ==================================================;
   write   =              FINAL ACYCLIC graph               =;
   write   ==================================================;
   write-b [Final acyclic schema produced in $timer$ s.];
   
   write-b Table CLOSURE (beware, this list may be fairly big!;
   select * from CLOSURE order by Path #[1,$limit$];

   write-b Table SUPER_CIRCUIT;
   select * from SUPER_CIRCUIT #[1,$limit$];

   write-b Composition of super circuits;
   select CircuitID as "Super circuit",
          group_concat(Node,',') as Composition
   from   SUPER_CIRCUIT group by CircuitID
   order by CircuitID #[1,$limit$];

   write-b Table TRANSLATION;
   select * from TRANSLATION #[1,$limit$];
   write;

   write-b Table NODE_A;
   select * from NODE_A #[1,$limit$];

   write-b Table LINK_A;
   select * from LINK_A #[1,$limit$];

   write-b Table LINK_UPDATE;
   select * from LINK_UPDATE #[1,$limit$];

compute schName = pureNodeName('$graph$');
write-b Loading sequence for schema "$schName$";

execSQL $sourceDir$/__Generate-Loading-Script.sql;

   write-b =========================================================;
   write   = Topological sorting of the final graph and generation =;
   write   =========================================================;
   write-b Table SORTED;
   select * from SORTED #[1,$limit$];
   write-b Table SORTED by level;
   select Level,group_concat(NodeName,', ') as Members
   from   SORTED group by Level #[1,$limit$];
   write;
   write Composition of super circuits (reminder);
   select CircuitID as "Super circuit",
          group_concat(Node,',') as Composition
   from   SUPER_CIRCUIT group by CircuitID
   order by CircuitID #[1,$limit$];

closeDB;
