-- write CREATE DATA STRUCTURES;

create temporary table NODE(
                 NodeID   char(12) not null primary key,
                 NodeName varchar(64));

create temporary table LINK(
                 LinkID   integer not null primary key,
                 FromNode char(12) not null,
                 ToNode   char(12) not null,
                 Opt      char(1) not null,
                 foreign key (FromNode) references NODE
                         on delete cascade,
                 foreign key (ToNode) references NODE
                         on delete cascade);

-- Final loadable acyclic schema --
-- ----------------------------- --

create temporary table NODE_A(
                 NodeID   char(12),
                 NodeName varchar(64));

create temporary table LINK_A(
                 LinkID   integer,
                 FromNode char(12),
                 ToNode   char(12),
                 Opt      char(1));

create temporary table LINK_UPDATE(
                 LinkID   integer,
                 FromNode char(12),
                 ToNode   char(12),
                 Opt      char(1));

create temporary table SORTED(
                 Level    integer,
                 NodeName varchar(64));

-- Working tables --
-- -------------- --

create temporary table NODE_W(
                 NodeID   char(12) not null primary key,
                 NodeName varchar(64));

create temporary table LINK_W(
                 LinkID   integer,
                 FromNode char(12),
                 ToNode   char(12),
                 Opt      char(1),
                 foreign key (FromNode) references NODE_W
                         on delete cascade,
                 foreign key (ToNode) references NODE_W
                         on delete cascade);

create temporary table CLOSURE(
                 FromNode      varchar(12),
                 ToNode        varchar(12),
                 Path          varchar(1024));

create temporary table SUPER_CIRCUIT(
                 Node         varchar(12),
                 SuperCircuit varchar(1024),
                 CircuitID    varchar(12)); 

create temporary table TRANSLATION(
                 Source varchar(12),
                 Substitute varchar(12),
                 Super char(1));

create temporary table RESTORED_LINK(
                 LinkID   integer,
                 FromNode char(12),
                 ToNode   char(12),
                 Opt      char(1));


-- ROOT and LEAF tables and triggers for working schema --
-- ---------------------------------------------------- --

create temporary view ROOT
as select NodeID,NodeName from NODE_W
   where  NodeID not in (select FromNode from LINK_W);

create temporary view LEAF
as select NodeID,NodeName from NODE_W
   where  NodeID not in (select ToNode from LINK_W);

create temporary trigger Delete_ROOT
instead of delete on ROOT
for each row
begin
   delete from NODE_W where NodeID = old.NodeID; end;

create temporary trigger Delete_LEAF
instead of delete on LEAF
for each row
begin
   delete from NODE_W where NodeID = old.NodeID; end;

