-- --------------- --
-- Text processing --
-- --------------- --

createDB InMemory;

create table TWEETS(TweetID char(24),Body CLOB);
insert into TWEETS values
  ('T01','How will cities recover from covid-19'),
  ('T02','More cases in small cities! OK?'),
  ('T03','Still no drug?'),
  ('T04','OK but how to protect poor cities from Covid 19?'),
  ('T05','Why not reuse the drug that cured Covid 18?'),
  ('T06','We should investigate the role of Illuminati!');

write-b -- The source tweets --;
select * from TWEETS order by TweetID;


create view WORDS(TweetID,Rank,Word) as
with recursive
   SEQ(N) as
      (select 1 union all select N+1 from SEQ where N < 1000),
   CAN(TweetID,Body) as
      (select TweetID,
              replace(translate(simplify(Body),
                                '.,''-!?"0123456789',
                                '   $$$$$$$$$$$$$$'),'$','')
       from TWEETS),
   WRD(TweetID,N,Word) as 
       (select C.TweetID,N,trim(item(C.Body,N,' ')) as Word
        from SEQ,(select TweetID,Body from CAN) C
        where N <= itemLen(C.Body,' ') and Word <> '')
select * from WRD;

write-b -- Extract the words from the source tweets --;
select * from WORDS;

write-b -- Rebuild the source tweets --;
select TweetID,group_concat(Word,' ') as Tweet
from   WORDS
group by TweetID;

write-b -- Create the STOP table --;
create table STOP(Word varchar(16));
insert into STOP values ('a'),('are'),('as'),('but'),('from'), 
           ('how'),('in'),('is'),('it'),('no'),('not'),('of'),
           ('ok'), ('that'),('the'),('to'),('we'),('will');

-- View of the significant words --
create view SWORDS as 
   select TweetID,Rank,Word from WORDS
   where  Word not in (select Word from STOP);

write-b -- Compute significant word frequencies --;
select Word,count(*) as Frequency 
from   SWORDS
group by Word
order by Frequency desc,Word;

write-b Where can I find significant words? --;
select Word,group_concat(TweetID) as Source
from   SWORDS
group by Word
order by itemLen(Source,',') desc,Word;;

write-b -- Compute pairs of significant words --;
select W1.Word||' '||W2.Word as "Word pairs",W1.TweetID as "Found in"
from   SWORDS W1, SWORDS W2
where  W1.TweetID = W2.TweetID
and    W2.Rank = (select min(Rank) from SWORDS
                  where  TweetID = W1.TweetID
                  and    Rank > W1.Rank)
order by W1.TweetID;

closeDB;

write-b End of script;
