write-b -- ---------------------- --;
write   -- Checking set inclusion --;
write   -- ---------------------- --;

write-b -- In table T, each A value has an associated set of B values  --;
write   -- Retrieve all the couples (a1,a2) of A values such that      --;
write   -- the B set of a2 is included into the B set of a1.           --;

createDB InMemory;

-- T0 includes the initial subset of rows for T
create table T0(A integer, B char(10));

-- T is built from N x loading of T0 rows
create table T(A integer, B char(10), primary key (A,B));

insert into T0 values
  (1,'a'),(1,'b'),(1,'c'),
  (2,'b'),(2,'c'),
  (3,'c'),
  (4,'c'),(4,'d'),
  (5,'e'),(5,'f'),(5,'g'),
  (6,'f'),(6,'g'),
  (7,'g'),
  (8,'g'),(8,'h'),
  (9,'k'),(9,'l'),(9,'m'),(9,'n'),(9,'o'),
  (10,'k'),(10,'l'),(10,'b'),(10,'c'),
  (11,'k'),(11,'l'),(11,'n'),
  (12,'k'),(12,'l'),(12,'g'),(12,'h'),(12,'c'),(12,'d'),
  (13,'k'),(13,'l'),(13,'n'),(13,'g'),
  (14,'e'),(14,'f'),(14,'g'),(14,'h'),(14,'k'),(14,'l'),
  (15,'k'),(15,'l'),
  (16,'a'),(16,'c'),(16,'d'),(16,'e'),(16,'f'),(16,'g'),
  (16,'h'),(16,'k'),(16,'l'),(16,'m'),(16,'n'),(16,'o'),
  (17,'h'),
  (18,'f'),
  (19,'c'),(19,'d'),(19,'g'),(19,'h');

set N = 30;
set Algo = 12345;
set time1,time2,time3,time4,time5 = 0,0,0,0,0;

ask-u N,Algo = Size parameter (1 to 200)|Select the algorithms to execute (ex: 12345);
if ('$DIALOGbutton$' = 'Cancel') stop;

-- Build the contents of table T
compute max = 100*$N$;
for I = [1,$max$,100] insert into T select A*$I$, B from T0;

-- ----------------------------------------------------- --

extract TS = select count(*) from T;
extract AS = select count(distinct A) from T;
write-b Size of table T  = $TS$ rows;
write Number of A values = $AS$;

--create view AB1(A1,BList1,Label,A2,BList2) as
--with AB(A,BList) as
--    (select A,group_concat(B,',') from T group by A)
--select C1.A,C1.BList,'includes',C2.A,C2.BList
--from   AB C1, AB C2
--where  C1.A <> C2.A
--and    itemInclude(C1.BList,C2.BList,',');
--start-timer;
--extract C = select count(*) from AB1;
--write Number of inclusion relationships: $C$;
--write-b -- Through itemList inclusion:    $timer$ --;

if ('$Algo$' not like '%1%') goto SKIP1;
 
-- --------------------- --
-- division operator (1) --
-- --------------------- --

create view AB1(A1,Label,A2) as
select distinct T1.A,'includes',T2.A
from   T T1,T T2
where  T1.A <> T2.A
and    not exists (select *
                   from   T T3
                   where  T3.A = T2.A
                   and    not exists (select *
                                      from   T T4
                                      where  T4.A = T1.A
                                      and    T4.B = T3.B
                                      )
                   );

start-timer;
extract C = select count(*) from AB1;
write Number of inclusion relationships: $C$;
set time1 = $timer$;
write -- Through division operator (1):  $time1$ --;

label SKIP1;
if ('$Algo$' not like '%2%') goto SKIP2;

-- --------------------- --
-- division operator (2) --
-- --------------------- --

create view AB2(A1,Label,A2) as
select distinct T1.A,'includes',T2.A
from   T T1,T T2
where  T1.A <> T2.A
and    not exists (select *
                   from   T
                   where  A = T2.A
                   and   B not in (select B from T where A = T1.A)
                   );
start-timer;
extract C = select count(*) from AB2;
set time2 = $timer$;
write -- Through division operator (2):  $time2$ --;

label SKIP2;
if ('$Algo$' not like '%3%') goto SKIP3;

-- --------------------- --
-- set difference        --
-- --------------------- --

create view AB3(A1,Label,A2) as
select distinct T1.A, 'includes',T2.A
from   T T1,T T2
where T1.A <> T2.A
and   not exists (select B from T where  A = T2.A
                    except
                  select B from T where  A = T1.A);
start-timer;
extract C = select count(*) from AB3;
set time3 = $timer$;
write -- Through set difference:         $time3$ --;

label SKIP3;
if ('$Algo$' not like '%4%') goto SKIP4;

-- ---------------------- --
-- itemList inclusion (1) --
-- ---------------------- --

create view AB4(A1,Label,A2) as
with AB(A,BList) as
    (select A,group_concat(B,',') from T group by A)
select C1.A,'includes',C2.A
from   AB C1, AB C2
where  C1.A <> C2.A
and    itemInclude(C1.BList,C2.BList,',');
start-timer;
extract C = select count(*) from AB4;
set time4 = $timer$;
write -- Through itemList inclusion (1): $time4$ --;

label SKIP4;
if ('$Algo$' not like '%5%') goto SKIP5;

-- ---------------------- --
-- itemList inclusion (2) --
-- ---------------------- --

create table TT(A integer primary key, BSet varchar(1000));
insert into TT
select A,group_concat(B,',') from T group by A;

create view AB5(A1,Label,A2) as
select C1.A,'includes',C2.A
from   TT C1, TT C2
where  C1.A <> C2.A
and    itemInclude(C1.BSet,C2.BSet,',');
start-timer;
extract C = select count(*) from AB5;
set time5 = $timer$;
write -- Through itemList inclusion (2): $time5$ --;

label SKIP5;

write-b Figures to drop in a spreadsheet:;
write $TS$;
write $AS$;
write $C$;
compute times = replace('$time1$@n$time2$@n$time3$@n$time4$@n$time5$','.',',');
write $times$;

closeDB;

write-b End of script;
