write-b -- Assigning sequence numbers to the rows of a table --;
write -- --------------------------------------------------- --;

createDB InMemory;

create temp table T(C1 char(12));
insert into T values
      ('B112'),('C123'),('B332'),('F010'),('K111'),('S127'),
      ('B512'),('B062'),('C400'),('C003'),('K729'),('F011'),
      ('L422'),('S712'),('D063'),('F400');
 
-- Augmenting table T --
set prefix = ABCDEFGHIJKLMNOP;
for I = [1,11];
  insert into T
  select item('$prefix$',$I$,'')||C1 from T;
endfor;

-- Compute table size --
extract T = select count(*) from T;
write-ab -- Table size: $T$ rows;


-- ----------------------------------- --
-- Mathematical definition (C. Ladish) --
-- ----------------------------------- --
start-timer;
--
create temp table T4 as
select (select count(*) as Seq
        from   T T2
        where  T2.C1 <= T1.C1) as Seq,
        C1
from T T1 order by C1;
--
compute elapsed = int($timer$*1000);
drop table T4;
write -- Mathematical definition: $elapsed$ ms. --;

-- ------------------------------------------------------- --
-- Extracting rowid from source table (possibly unordered) --
-- ------------------------------------------------------- --
start-timer;
--
create temp table T1(Seq integer,C1 char(12));
insert into T1
   select rowid as Seq,* from T order by C1;
--
compute elapsed = int($timer$*1000);
drop table T1;
write -- Extracting rowid from source table (possibly unordered): $elapsed$ ms. --;

-- ------------------------------------------------------- --
-- Extracting rowid from target table (ordered)            --
-- ------------------------------------------------------- --
start-timer;
--
create temp table T2 as
  select 0 as Seq,C1 from T order by C1;
update T2 set Seq = rowid;
--
compute elapsed = int($timer$*1000);
drop table T2;
write -- Extracting rowid from target table (ordered): $elapsed$ ms. --;

-- ------------------------------------------------------- --
-- Autoincremented column in target table                  --;
-- ------------------------------------------------------- --
start-timer;
--
create temp table T3(Seq integer not null primary key autoincrement,C1);
insert into T3(C1) select * from T order by C1;
--
compute elapsed = int($timer$*1000);
drop table T3;
write -- Autoincremented column in temp table : $elapsed$ ms. --;

-- ---------------------------------------- --
-- Computing sequence numbers through a CTE --
-- ---------------------------------------- --
start-timer;
--
create table T0 as
with SEQUENCE(Seq,C1) as
     (select 1,C1
      from (select C1 from T order by C1 limit 1)
         union
      select S.Seq+1,T.C1
      from   T, SEQUENCE S
      where  T.C1 = (select min(C1)
                     from   T
                     where  C1 > S.C1)
      )
select * from SEQUENCE;
--
compute elapsed = int($timer$*1000);
drop table T0;
write -- Computing sequence numbers through a CTE: $elapsed$ ms. --;

-- ------------------------------------------------------- --
-- Window function                                         --;
-- ------------------------------------------------------- --

start-timer;
--
create temp table T5 as 
select row_number() over (order by C1) as C1 from T;
--
compute elapsed = int($timer$*1000);
drop table T5;
write -- Window function: $elapsed$ ms. --;

-- ------------------------------------------------------- --
-- PK in SQLfast list                                      --;
-- ------------------------------------------------------- --
set itemSpeed = 1;
start-timer;
--
create temp table T5 as 
with recursive SEQUENCE(Seq) as
     --(select 1 union all select Seq+1 from SEQUENCE where Seq < (select count(*) from T)),
     (select 1 union all select Seq+1 from SEQUENCE where Seq < $T$),
     LIST(Id) as
     (select group_concat(C1,',') from T)
     --(select group_concat2(C1,1,C1,0,',') from T)
select Seq,item(Id,Seq,',') as C1 from SEQUENCE,LIST;
--
compute elapsed = int($timer$*1000);
write -- PK in SQLfast list: $elapsed$ ms. --;

closeDB;

write-b End of script;

