-- ------------------------ --
-- Photo Album - Variations --
-- ------------------------ --

dropAllVariables;
set dir = Images/Planes;

-- --------------------------------------- --
-- Load picture descriptions in a database --
-- --------------------------------------- --
createDB InMemory;
create table PICTURE(ID       integer not null primary key autoincrement,
                     FullName varchar(128),
                     Name     varchar(64));

fileNames Files/dir.txt = $dir$;

for fullName = [file Files/dir.txt]
  insert into PICTURE(FullName,Name) values ('$fullName$',lastPathNode('$fullName$'));

select * from PICTURE;

-- -------------------------- --
-- A minimal picture viewer   --
-- -------------------------- --
for fullName,name = [select FullName,Name from PICTURE];
   showPicture $fullName$ = [/b Picture slideshow ($name$)];
   if ('$DIALOGbutton$' = 'Cancel') exit;
endfor;

-- ------------------------------------------------- --
-- Showing a mosaic of pictures to select (4 images) --
-- ------------------------------------------------- --

for id,name,path = [select ID,Name,FullName from PICTURE limit 4];
   set nam$id$ = $name$;
   set path$id$ = $path$;
   set p$id$ = 0;
endfor;
 
askCombo [/bFor each picture, click on button 'select' to select it (1 box)]
         [selectOne p1 = {$nam1$}||select]
        |[showPicture $path1$ = [/x300/y200]]
       ||[selectOne p2 = {$nam2$}||select]
        |[showPicture $path2$ = [/x300/y200]]
      |||[selectOne p3 = {$nam3$}||select]
        |[showPicture $path3$ = [/x300/y200]]
       ||[selectOne p4 = {$nam4$}||select]
        |[showPicture $path4$ = [/x300/y200]];

write;
for i = [1,4]
   if ($p$i$$) write Picture "$nam$i$$" selected;


-- ----------------------------------------------------- --
-- Showing a mosaic of pictures to select (all pictures) --
-- ----------------------------------------------------- --

extract N = select count(*) from PICTURE;

outputOpen $scriptDirectory$/_Pictures.sql;
 
for first = [1,$N$,4];

   for id,nam,path,pos = [select ID,Name,FullName,
                                 row_number() over ()                               
                          from PICTURE
                          where ID >= $first$ limit 4];
      if ($pos$ = 1);
         -- first picture
         write askCombo [/bFor each picture, click on button 'select' to select it (all pictures)];
         write @S9[selectOne p$id$ = {$nam$}||select];
         write @S8|[showPicture $path$ = [/x300/y200]];
      else;
         -- pictures 2, 3, 4
         if ($pos$ in (2,4);
            -- go to right
            write @S7||[selectOne p$id$ = {$nam$}||select];
            write @S8|[showPicture $path$ = [/x300/y200]];
         else;
            -- go to left
            write @S6|||[selectOne p$id$ = {$nam$}||select];
            write @S8|[showPicture $path$ = [/x300/y200]];
         endif;
      endif;
   endfor;
   write @S9;;

   substitution off;
   write-a if ('$DIALOGbutton$' = 'Cancel') return;;
   substitution on;

endfor;

outputAppend window;

execSQL $scriptDirectory$/_Pictures.sql;
if ('$DIALOGbutton$' = 'Cancel') goto END;

extract picList = select group_concat(pureNodeName(Name)) from PICTURE;
write;
for n = [1,$N$];
   if ($p$n$$ = 1);
      compute pic = item('$picList$',$n$,',');
      write Picture "$pic$" selected;
   endif;
endfor;

label END;
   closeDB;
   deleteFile-e $scriptDirectory$/_Pictures.sql;
   write-b End of script.;

