-- --------------------------------------------- --
-- Loading and extracting pictures in a database --
-- --------------------------------------------- --

-- Creating the database
-- ---------------------

createDB InMemory;
create table PHOTO(ID       integer not null primary key,
                   FullName varchar(128),
                   Name     char(64),
                   Size     integer,
                   Picture  BLOB);

-- Getting the list of photo names
-- -------------------------------
selectDirectory dir = Images;
fileNames $fileDirectory$/dir.txt = $dir$;

-- Loading photos in the database
-- ------------------------------
start-timer;
set id = 1;
for name = [file $fileDirectory$/dir.txt];
   function siz,x,x,x = LFile:infoFile {$name$};
   BfileToVariable varLOB = $name$;
   insert into PHOTO(ID,FullName,Name,Size)
          values ($id$,'$name$',lastPathNode('$name$'),$siz$);
   insertLOB varLOB = Picture from PHOTO where ID = $id$;
   compute id = $id$ + 1;
endfor;
commitDB;
write-ab -- Loading time: $timer$;

select ID,FullName,Name,
       frame(thousandSep(Size,','),8,'>',' ') as Size,
       thousandSep(sum(Size) over (),',') as Total,
       Picture
from PHOTO;

-- Extracting photos from the database
-- -----------------------------------
start-timer;
for name,photo = [select Name,Picture from PHOTO] 
   VariableToFile photo = $dir$/X_$name$;
write-ab -- Extraction time: $timer$;

showMessage Have a look at directory $dir$!
            @nDo you want to delete the new files?;

if ('$DIALOGbutton$' = 'OK');
   for name = [select Name from PHOTO];
      deleteFile $dir$/X_$name$;
      write $dir$/X-$name$ deleted;
   endfor;
endif;

deleteFile-e $fileDirectory$/dir.txt;
write-b File "dir.txt" deleted;

closeDB;

write-b End of script;
