write;
write -- ----------------- --;
write -- The set operators --;
write -- ----------------- --;

createDB InMemory;

create table T1(A integer);
create table T2(A integer);
insert into T1 values     (2),(3),(4),(5),   (7);
insert into T2 values (1),    (3),(4),(5),(6);

select group_concat(A) "T1" from T1;
select group_concat(A) "T2" from T2;

write-b -- T1 union T2 --;
select A from T1 union select A from T2;
select group_concat(A) as "T1 union T2"
from (select A from T1 union select A from T2);

write-b -- T1 intersect T2 --;
select A from T1 intersect select A from T2;
select group_concat(A) as "T1 inter T2"
from (select A from T1 intersect select A from T2);

write-b -- T1 except T2 --;
select A from T1 except select A from T2;
select group_concat(A) as "T1 difference T2"
from (select A from T1 except select A from T2);

write-b -- T1 symmetric difference T2 --;
select A from (select A from T1 except select A from T2)
   union
select A from (select A from T2 except select A from T1);
select group_concat(A) as "T1 symm. difference T2"
from (select A from (select A from T1 except select A from T2)
         union
      select A from (select A from T2 except select A from T1));

write;
write -- ------------------------------ --;
write -- The set operators (simulation) --;
write -- ------------------------------ --;

write-b -- T1 union T2 (simulation) --;
select coalesce(T1.A,T2.A) as A
from T1 full outer join T2 on (T1.A = T2.A);

write-b -- T1 intersect T2 (simulation) --;
select A from T1
where  A in (select A from T2);

write-b -- T1 except T2 (simulation) --;
select A from T1
where  A not in (select A from T2);

closeDB;


write;
write -- ----------------------------------- --;
write -- The multiset operators (simulation) --;
write -- ----------------------------------- --;

createDB InMemory;

create table T1(A integer);
create table T2(A integer);
insert into T1 values (1),(1),(2),(3),(3),(4),(4),(6),(7);
insert into T2 values (1),(2),(2),(3),(3),(5),(5),(6),(8);

select group_concat(A) "T1" from T1;
select group_concat(A) "T2" from T2;

-- Simulation of T1 union all T2 --
-- ----------------------------- --

with recursive
 TU(A,Multi) as
     (select A,sum(N)
      from (select A,count(*) as N,'T1' from T1 group by A
              union
            select A,count(*) as N,'T2' from T2 group by A)
      group by A),
--
 UNFOLD(A,N) as
     (select A,Multi from TU
        union
      select A,N-1 from UNFOLD where N > 1)
--
select group_concat(A) as "T1 union all T2" 
from (select A from UNFOLD order by A);

-- Simulation of T1 except all T2 --
-- ------------------------------ --

with recursive
 V1(A,N) as (select A,count(*) from T1 group by A),
 V2(A,N) as (select A,count(*) from T2 group by A),
 --
 TE(A,Multi) as
   (select V1.A, coalesce(V1.N - V2.N,V1.N) as SN
    from  (V1 left join V2 on V1.A = V2.A)
    where SN > 0),
 --
 UNFOLD(A,N) as
   (select A,Multi from TE
      union
    select A,N-1 from UNFOLD where N > 1)
select group_concat(A) as "T1 except all T2"
from (select A from UNFOLD order by A);

-- Simulation of T1 intersect all T2 --
-- --------------------------------- --

with recursive
 V1(A,N) as (select A,count(*) from T1 group by A),
 V2(A,N) as (select A,count(*) from T2 group by A),
 --
 TI(A,Multi) as
   (select V1.A, coalesce(min(V1.N,V2.N),0) as SN
    from   (V1 left join V2 on V1.A = V2.A)
    where  SN > 0),
 --
 UNFOLD(A,N) as
   (select A,Multi from TI
      union all
    select A,N-1 from UNFOLD where  N > 1)
select group_concat(A) as "T1 intersect all T2"
from (select A from UNFOLD order by A);

closeDB;

write-b End of script;
