-- --------------------- --
-- Solving logic puzzles --
-- --------------------- --

write-b -- The dwellers --;
write   -- ------------ --;

createDB InMemory;

create table LIVES as 
with 
  NAMES(Name) as 
    (values ('Baker'),('Cooper'),('Fletcher'),('Miller'),('Smith')),
  FLOORS(Floor) as
    (values (1),(2),(3),(4),(5))
select Name,Floor from NAMES,FLOORS;

write-b -- All the possible assignments --;
select * from LIVES;

write-b -- A (unique) solution --;
select distinct
       Baker.Floor    as FBaker,
       Cooper.Floor   as FCooper,
       Fletcher.Floor as FFletcher,
       Miller.Floor   as FMiller,
       Smith.Floor    as FSmith
from LIVES Baker, LIVES Cooper, LIVES Fletcher, 
     LIVES Miller, LIVES Smith
where Baker.Floor < 5
and   Cooper.Floor > 1
and   Cooper.Floor < Miller.Floor
and   abs(Cooper.Floor - Fletcher.Floor) > 1
and   abs(Smith.Floor - Fletcher.Floor) > 1
and   Fletcher.Floor between 2 and 4
and   not (   Baker.Floor  = Cooper.Floor 
           or Baker.Floor  = Fletcher.Floor
           or Baker.Floor  = Miller.Floor
           or Baker.Floor  = Smith.Floor
           or Cooper.Floor = Fletcher.Floor
           or Cooper.Floor = Miller.Floor
           or Cooper.Floor = Smith.Floor
           or Fletcher.Floor = Miller.Floor
           or Fletcher.Floor = Smith.Floor
           or Miller.Floor   = Smith.Floor);

closeDB;


write-b -- The eight queens problem --;
write   -- ------------------------ --;

createDB InMemory;

create table LAYOUT as
  with recursive
  BASE(N) as (values (1),(2),(3),(4),(5),(6),(7),(8)),
  PERM(Digits) as
   (select N from BASE
     union all
    select PERM.Digits||B1.N
    from  PERM,BASE B1
    where PERM.Digits not like '%'||B1.N||'%'
    and   length(PERM.Digits) < 8
    and   not exists
           (select *
            from  BASE B2
            where B2.N <= length(PERM.Digits)
            and length(PERM.Digits) - B2.N + 1
                 =
                abs(B1.N - cast(substr(PERM.Digits,B2.N,1)
                                     as int)
                    )
            )
   )
  select Digits as Permutations from PERM
  where  length(PERM.Digits) = 8;

select * from LAYOUT;

closeDB;

write-b End of script;
