-- --------------------- --
-- Unnest-Nest Operators --
-- --------------------- --

createDB InMemory;
create table T(Id integer,Name varchar(32),Cities varchar(32));
insert into T values (1,'Adam',    'London,Paris,Madrid'),
                     (2,'Brian',   'Paris,Genve'),
                     (3,'Christie', null),
                     (4,'David',   'London');

write-b -- Source unnormalized table;
select * from T;

create table T_unnest as
with recursive
   TT(Id,Name,Cities) as
      (select Id,Name,case when Cities is null then '' else Cities end
       from   T),
   S(Id,Idx,City) as
      (select id,0,'' from TT
         union all
       select S.Id,S.Idx+1,item(TT.Cities,Idx+1,',')
       from   S,TT
       where  TT.Id = S.Id and S.Idx < itemLen(TT.Cities,',')
       )
select S.Id,T.Name,case when length(S.City) > 0 
                        then S.City
                        else null
                   end as City
from   S,T
where  S.id = T.Id and S.Idx > 0;

write-b -- Normalized table;
select * from T_unnest
order by Id;

write-b -- Nesting the normalized table;
select id,Name,group_concat(City) as Cities
from   T_unnest
group by Id,Name;

closeDB;


write-ab -- Now, a parametric version of the "unnest" operator --;

createDB InMemory;
create table T(Id integer,Name varchar(32),Cities varchar(32));
insert into T values (1,'Adam',    'London,Paris,Madrid'),
                     (2,'Brian',   'Paris,Genve'),
                     (3,'Christie', null),
                     (4,'David',   'London');

-- Parameters
--   table:  name of the source N1NF table
--   key:    column names of the primary/unique key of the source table
--   single: list of names of the single-valued columns 
--   multi:  name of the multivalued column to unnest
--   result: name of the resulting 1NF table
--   new:    name of the single-valued column derived from "multi"

set table = T;
set key = Id;
set single = Id,Name;
set multi = Cities;
set result = T_unnest;
set new = City;

-- prefixing the names in "single" with 'TT.'
compute sLen = itemLen('$single$',',');
compute TTsingle = itemJoin(rtrim(repeat('TT,',$sLen$),','),'$single$',',','.',',');

create table $result$ as
with recursive
   TT($single$,$multi$) as
      (select $single$,case when $multi$ is null then '' else $multi$ end
       from   $table$),
   S($single$,$new$,Idx) as
      (select $key$,'','',0 from TT
         union all
       select $TTsingle$,item($multi$,Idx+1,','),S.Idx+1
       from   S,TT
       where  TT.$key$ = S.$key$ and S.Idx < itemLen($multi$,',')
       )
select $single$,case when length($new$) > 0 
                        then $new$
                        else null
                   end as $new$
from   S
where  Idx > 0;

select * from $result$ order by $single$;

closeDB;

write-b End of script;

