-- ------------------------------------------------------- --
-- Create the persistent Dictionary of a selected database --
-- ------------------------------------------------------- --
-- July 2020 --

set maxSelectWidth = 160;

-- Select current database and build the name of its persistent dictionary --
-- ----------------------------------------------------------------------- --
selectFile current-db = $dbDirectory$,Database,db;
   if ('$DIALOGbutton$' = 'Cancel') return;
compute dic-db = pathPrefix('$current-db$')||'/Dictionary-of-'||lastPathNode('$current-db$');

-- Delete former persistent Dictionary --
-- ----------------------------------- --
deleteDB $dic-db$;

-- Open current database and create its directory --
-- ---------------------------------------------- --
openDB $current-db$;
createDictionary;

-- Add some useful views (merely adding TableName) --
-- ----------------------------------------------- --

create temp view SYS_COLUMN_FULL(ColID,TableID,TableName,ColSeq,ColName,ColType,ColLen1,ColLen2,AutoNum,ColNull,DefVal,PKseq,Ext)
as select ColID,C.TableID,TableName,ColSeq,ColName,ColType,ColLen1,ColLen2,AutoNum,ColNull,DefVal,PKseq,C.Ext
   from   SYS_COLUMN C,SYS_TABLE T
   where  C.TableID = T.TableID;

create temp view SYS_VIEW_FULL(TableID,TableName,Expression,Ext)
as select V.TableID,TableName,Expression,V.Ext
   from   SYS_VIEW V,SYS_TABLE T
   where  V.TableID = T.TableID;

create temp view SYS_KEY_FULL(KeyID,KeyType,SourceTableID,SourceTableName,KeyName,TargetTableID,TargetTableName,KeyTarget,onDelete,onUpdate,Match,KeyNull,Ext)
as select KeyID,KeyType,K.TableID,TableName,KeyName,TableTarget,
          (select TableName from SYS_TABLE where TableID = (select TableID from SYS_KEY where KeyID = K.KeyTarget)),
          KeyTarget,onDelete,onUpdate,Match,KeyNull,K.Ext
   from   SYS_KEY K,SYS_TABLE T
   where  K.TableID = T.TableID;

create temp view SYS_CHECK_FULL(CheckID,TableID,TableName,ConstrName,Condition,Ext)
as select CheckID,C.TableID,TableName,ConstrName,Condition,C.Ext
   from   SYS_CHECK C,SYS_TABLE T
   where  C.TableID = T.TableID;
   

create temp view SYS_TRIGGER_FULL(TrigID,TableID,TableName,TrigName,TrigWhen,TrigEvent,TrigCol,TrigScope,TrigCond,Ext) 
as select TrigID,R.TableID,T.TableName,TrigName,TrigWhen,TrigEvent,TrigCol,TrigScope,TrigCond,R.Ext 
   from   SYS_TRIGGER R,SYS_TABLE T 
   where  R.TableID = T.TableID;

create temp view SYS_TRIGGER_COMP_FULL(TrigStatID,TrigID,TrigName,StatSeq,TableID,StatTable,StatOper,StatCol,StatExpr,Ext)
as select TrigStatID,C.TrigID,R.TrigName,StatSeq,coalesce((select TableID from SYS_TABLE where TableName = C.StatTable),''),StatTable,StatOper,StatCol,StatExpr,C.Ext 
   from   SYS_TRIGGER_COMP C,SYS_TRIGGER R
   where  C.TrigID = R.TrigID;

set sysList = SYS_TABLE,SYS_COLUMN,SYS_VIEW,SYS_KEY,SYS_KEY_COMP,SYS_INTER_TABLE,
              SYS_CHECK,SYS_INDEX,SYS_INDEX_COMP,SYS_TRIGGER,SYS_TRIGGER_COMP;
set sysList = $sysList$,SYS_COLUMN_FULL,SYS_VIEW_FULL,SYS_KEY_FULL,SYS_CHECK_FULL,SYS_TRIGGER_FULL,SYS_TRIGGER_COMP_FULL;

-- Create, attach, fill and close the persistent Dictionary --
-- -------------------------------------------------------- --
generic attach '$dic-db$' as dic;
for sys = [item(,) $sysList$]
    create table dic.$sys$ as select * from $sys$;
commitDB;
generic detach database 'dic';
closeDB;

-- Check the content of the persistent Dictionary --
-- ---------------------------------------------- --
showMessage Display the content of the dictionary?;
if ('$DIALOGbutton$' = 'OK');
   openDB $dic-db$;
   for sys = [item(,) $sysList$];
      write-b System table $sys$:;
      select * from $sys$;
   endfor;
endif;

closeDB;
